# Standard Libraries and Gems

## Standard libraries

| Library               | Purpose                                                               | MRI                                 | JRuby                                                     | TruffleRuby                                      | Tests                                 | Notes                                                                                    |
| --------------------- | --------------------------------------------------------------------- | ----------------------------------- | --------------------------------------------------------- | ------------------------------------------------ | ------------------------------------- | ---------------------------------------------------------------------------------------- |
| `abbrev`              | Abbreviations for strings                                             | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `base64`              |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | Specs                                 |                                                                                          |
| `benchmark`           |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `bigdecimal`          |                                                                       | C extension                         | Ruby and Java extension                                   | Ruby and Java primitives                         | Specs, MRI                            |                                                                                          |
| `cgi`                 |                                                                       | Ruby and C extension                | Ruby and Java extension                                   | As MRI                                           | Specs (limited), MRI                  |                                                                                          |
| `cmath`               | Complex math additions to `Math`                                      | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `continuation`        |                                                                       | C extension                         | Ruby and Java (one-shot continuations only)               | Unsupported                                      | MRI (language)                        | Deprecated in favour of `Fiber`                                                          |
| `coverage`            |                                                                       | C extension                         | Java extension, think it needs a command-line flag though | Java primitives                                  | Specs (limited), MRI                  |                                                                                          |
| `csv`                 |                                                                       | Ruby                                | As MRI (not in source repo)                               | As MRI                                           | Specs, MRI                            |                                                                                          |
| `date`                |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `dbm`                 | Key-value store manager                                               | C extension                         | As MRI                                                    | Unsupported                                      | MRI                                   |                                                                                          |
| `debug`               | Debugger using `tracer`                                               | Ruby                                | As MRI, but think it needs a command-line flag            | As MRI                                           | C extension parts in MRI              | Use `--inspect` instead                                                                  |
| `delegate`            | Method-call forwarding                                                | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `digest`              |                                                                       | C extension                         | Java extension                                            | Ruby and Java primitives                         | Specs, MRI                            | Some third-party C extensions link against the C extension                               |
| `drb`                 | Distributed object system                                             | Ruby                                | As MRI                                                    | As MRI                                           | Specs (limited), MRI                  |                                                                                          |
| `e2mmap`              | Utility for defining exceptions                                       | Ruby                                | As MRI                                                    | As MRI                                           | None                                  |                                                                                          |
| `English`             | Aliases for Global variables                                          | Ruby                                | As MRI                                                    | As MRI                                           | Specs                                 |                                                                                          |
| `erb`                 | Template system                                                       | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `etc`                 | Utility to read from `/etc`                                           | C extension                         | Java extension                                            | As MRI                                           | Specs, MRI                            |                                                                                          |
| `fcntl`               |                                                                       | C extension                         | Java extension                                            | Ruby                                             | None                                  |                                                                                          |
| `ffi`                 |                                                                       |                                     | Java extension                                            | Ruby using `Polyglot` but it's limited           | In the third-party FFi gem repository | Non-standard - third-party gem but defaults to included version on other implementations |
| `fiber`               |                                                                       | C extension                         | Java extension                                            | Java primitives                                  | Specs                                 |                                                                                          |
| `fiddle`              | FFI                                                                   | C extension                         | Ruby using `FFI`                                          | Ruby using `Polyglot`.                           | MRI                                   |                                                                                          |
| `fileutils`           |                                                                       | Ruby                                | As MRI (not in source repo)                               | As MRI                                           | MRI                                   |                                                                                          |
| `find`                | Traverse file systems                                                 | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `forwardable`         | Method-call forwarding                                                | Ruby                                | As MRI                                                    | As MRI                                           | MRI                                   |                                                                                          |
| `gdbm`                | Key-value store                                                       | C extension                         | Unsupported                                               | Unsupported                                      | MRI                                   |                                                                                          |
| `getoptlong`          | Command-line parsing                                                  | Ruby                                | As MRI                                                    | As MRI                                           | Specs                                 |                                                                                          |
| `io/console`          |                                                                       | C extension                         | Java extension                                            | Ruby, but limited                                | MRI                                   |                                                                                          |
| `io/nonblock`         |                                                                       | C extension                         | Java extension                                            | Ruby                                             | MRI                                   |                                                                                          |
| `io/wait`             |                                                                       | C extension                         | Java extension                                            | Ruby, but limited                                | MRI                                   |                                                                                          |
| `ipaddr`              |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `irb`                 | Interactive shell                                                     | Ruby                                | As MRI                                                    | As MRI                                           | MRI                                   |                                                                                          |
| `json`                |                                                                       | C extension                         | Java extension (upstreamed)                               | Ruby (from original gem)                         | MRI                                   |                                                                                          |
| `logger`              |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `mathn`               | Redefine number operators to be rational                              | Ruby and C extension                | Java extension                                            | Unsupported                                      | Specs                                 |                                                                                          |
| `matrix`              |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `mkmf`                | Generate `Makefile`files for C extensions, used by `extconf.rb` files | Ruby                                | As MRI, but not relevant                                  | As MRI                                           | MRI                                   |                                                                                          |
| `monitor`             |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `mutex_m`             | Mix-in a mutex to classes                                             | Ruby                                | As MRI                                                    | As MRI                                           | MRI                                   |                                                                                          |
| `net`                 |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `nkf`                 | Network Kanji Filter - Japanese transcodings                          | C extension                         | Java extension                                            | As MRI                                           | MRI                                   |                                                                                          |
| `objspace`            | Extensions to `ObjectSpace`                                           | C extension                         | Ruby using Java interop, but limited                      | Java primitives                                  | MRI                                   |                                                                                          |
| `observer`            | Publish/subscribe utility                                             | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `open-uri`            | Monkey patch `Kernel#open` to work with URIs                          | Ruby                                | As MRI                                                    | As MRI                                           | MRI                                   |                                                                                          |
| `open3`               |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `openssl`             |                                                                       | C extension                         | Java extension, but shipped separately                    | As MRI                                           | Specs (limited), MRI                  |                                                                                          |
| `optparse`            | Command-line parsing                                                  | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            | Aliased as `optionparser`                                                                |
| `ostruct`             | Hashes as objects                                                     | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `pathname`            |                                                                       | C extension                         | Java extension                                            | Ruby (from RubySL, from MRI?)                    | Specs, MRI                            |                                                                                          |
| `pp`                  | `Kernel#pp`                                                           | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `prettyprint`         |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | MRI                                   |                                                                                          |
| `prime`               | Prime numbers                                                         | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `profile`             | Interface to `profiler`                                               | Ruby                                | As MRI                                                    | As MRI, but not sure it really works             | C extension parts in MRI              | Use `--cpusampler` instead                                                               |
| `profiler`            | Profiler using `TracePoint`                                           | Ruby                                | As MRI                                                    | As MRI, but not sure it really works             | C extension parts in MRI              | Use `--cpusampler` instead                                                               |
| `pstore`              | Persist objects to files                                              | Ruby                                | As MRI                                                    | As MRI                                           | MRI                                   |                                                                                          |
| `psych`               | YAML parser, implementation for `yaml`                                | C extension                         | Java extension using SnakeYAML                            | As MRI                                           | Specs (via YAML), MRI                 |                                                                                          |
| `pty`                 | Terminals                                                             | C extension                         | Ruby using FFI                                            | Unsupported                                      | MRI                                   |                                                                                          |
| `racc` (runtime only) | Parser generator                                                      | Ruby and C extension                | Ruby and Java extension                                   | Ruby (no extension)                              | None                                  | We use the pure Ruby version of the extension                                            |
| `rake`                | Ruby `make` equivalent                                                | Ruby                                | As MRI                                                    | As MRI                                           | None                                  |                                                                                          |
| `rbconfig`            | System and build configuration information                            | Generated Ruby                      | Java extension                                            | Ruby                                             | MRI                                   |                                                                                          |
| `rdoc`                | Documentation system                                                  | Ruby                                | As MRI (not in source repo)                               | As MRI                                           | MRI                                   |                                                                                          |
| `readline`            |                                                                       | C extension                         | Java extension (not in source repo)                       | Ruby and Java primitives                         | Specs, MRI                            |                                                                                          |
| `resolv`              | DNS resolver                                                          | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `resolv-replace`      | Monkey patches to use `resolv`                                        | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `rexml`               | XML tools                                                             | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `rinda`               | Ruby port of the Linda system used by `drb`                           | Ruby                                | As MRI                                                    | As MRI                                           | MRI                                   |                                                                                          |
| `ripper`              | Ruby parser                                                           | C extension                         | Java extension                                            | Unupported                                       | MRI                                   |                                                                                          |
| `rss`                 |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | MRI                                   |                                                                                          |
| `rubygems`            | Package manager                                                       | Ruby                                | As MRI                                                    | As MRI                                           | MRI                                   |                                                                                          |
| `scanf`               |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | MRI                                   |                                                                                          |
| `sdbm`                | Key-value store                                                       | C extension                         | Unsupported                                               | Unsupported                                      | MRI                                   |                                                                                          |
| `securerandom`        |                                                                       | Ruby                                | Java extension                                            | Ruby (from RubySL, from MRI?)                    | Specs, MRI                            |                                                                                          |
| `set`                 |                                                                       | Ruby                                | Java extension                                            | As MRI                                           | Specs, MRI                            |                                                                                          |
| `shell`               | Unix shell utility routines                                           | Ruby                                | As MRI                                                    | As MRI                                           | MRI                                   |                                                                                          |
| `shellwords`          | Handle strings like Bash does                                         | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `singleton`           |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `socket`              |                                                                       | C extension                         | Java extension                                            | Ruby (from RubySL) using FFI                     | Specs, MRI                            |                                                                                          |
| `stringio`            |                                                                       | C extension                         | Java extension                                            | Ruby (from RubySL)                               | Specs, MRI                            |                                                                                          |
| `strscan`             |                                                                       | C extension                         | Java extension                                            | Ruby (from RubySL)                               | Specs, MRI                            |                                                                                          |
| `sync`                | Two-phase commit utility                                              | Ruby                                | As MRI                                                    | As MRI                                           | MRI                                   |                                                                                          |
| `syslog`              |                                                                       | C extension                         | Ruby using FFI                                            | As MRI                                           | Specs, MRI                            |                                                                                          |
| `tempfile`            |                                                                       | Ruby                                | Java extension                                            | As MRI                                           | Specs, MRI                            |                                                                                          |
| `thread`              |                                                                       | C extension                         | Java extension                                            | Java primitives                                  | Specs                                 | Built into VM binary in MRI, considered core                                             |
| `thwait`              | Join multiple threads                                                 | Ruby                                | As MRI                                                    | As MRI                                           | None                                  |                                                                                          |
| `time`                |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `timeout`             |                                                                       | Ruby                                | Java extension                                            | Ruby (from RubySL)                               | Specs                                 |                                                                                          |
| `tmpdir`              |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `tracer`              | Wrapper around `set_trace_func`can be used as a command-line utility  | Ruby                                | As MRI, think it needs a command-line flag though         | As MRI                                           | MRI                                   |                                                                                          |
| `tsort`               | Topological sorter                                                    | Ruby                                | As MRI                                                    | As MRI                                           | MRI                                   |                                                                                          |
| `un`                  | Unix shell command equivalents, similar to BusyBox                    | Ruby                                | As MRI                                                    | As MRI                                           | None                                  |                                                                                          |
| `unicode_normalize`   |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | MRI                                   |                                                                                          |
| `uri`                 |                                                                       | Ruby                                | As MRI                                                    | As MRI                                           | Specs, MRI                            |                                                                                          |
| `weakref`             |                                                                       | Ruby                                | As MRI                                                    | Ruby with Java primitives                        | Specs, MRI                            |                                                                                          |
| `webrick`             | Web server                                                            | Ruby                                | As MRI                                                    | As MRI                                           | MRI                                   |                                                                                          |
| `win32`               |                                                                       | C extension and Ruby using `Fiddle` | Ruby using FFI but it looks different to standard         | Unsupported                                      | MRI                                   |                                                                                          |
| `win32ole`            |                                                                       | C extension                         | Java extension in third-party gem `jruby-win32ole`        | Unsupported                                      | Specs, MRI                            |                                                                                          |
| `yaml`                |                                                                       | Ruby (just runs `psych`)            | Ruby                                                      | As MRI                                           | MRI                                   |                                                                                          |
| `zlib`                |                                                                       | C extension                         | Java extension                                            | As MRI                                           | MRI                                   |                                                                                          |

(*As MRI* sometimes means with a few patches applied.)

## Standard gems

See https://stdgems.org.

> Default gems: These gems are part of Ruby and you can always require them
> directly. You cannot remove them. They are maintained by Ruby core.
> 
> Bundled gems: The behavior of bundled gems is similar to normal gems, but
> they get automatically installed when you install Ruby. They can be
> uninstalled and they are maintained outside of Ruby core.

It is not clear yet how installing default gems from RubyGems will work with
TruffleRuby in the future, as some default gems we use differ from the originals.

### Default gems

To list default gems, use:
```
$ gem list | grep default:
```

### Bundled gems

The list of bundled gems is available in MRI under `gems/bundled_gems`
```
$ cat gems/bundled_gems
```

* `did_you_mean` 1.3.0 (helper to suggest method names on method missing)
* `minitest` 5.11.3 (test framework)
* `net-telnet` 0.2.0
* `power_assert` 1.1.3
* `rake` 12.3.2 (hybrid bundled gem and standard library)
* `test-unit` 3.2.9 (test framework)
* `xmlrpc` 0.3.0 (RPC using XML)
