/*
 * Copyright (c) 2015, 2019 Oracle and/or its affiliates. All rights reserved. This
 * code is released under a tri EPL/GPL/LGPL license. You can use it,
 * redistribute it and/or modify it under the terms of the:
 *
 * Eclipse Public License version 2.0, or
 * GNU General Public License version 2, or
 * GNU Lesser General Public License version 2.1.
 */
package org.truffleruby.language.objects;

import org.truffleruby.Layouts;
import org.truffleruby.language.Nil;
import org.truffleruby.language.RubyContextNode;

import com.oracle.truffle.api.dsl.Cached;
import com.oracle.truffle.api.dsl.Fallback;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.object.DynamicObject;

public abstract class IsTaintedNode extends RubyContextNode {

    public static IsTaintedNode create() {
        return IsTaintedNodeGen.create();
    }

    public abstract boolean executeIsTainted(Object object);

    @Specialization
    protected boolean isTainted(boolean object) {
        return false;
    }

    @Specialization
    protected boolean isTainted(int object) {
        return false;
    }

    @Specialization
    protected boolean isTainted(long object) {
        return false;
    }

    @Specialization
    protected boolean isTainted(double object) {
        return false;
    }

    @Specialization
    protected boolean isTainted(Nil object) {
        return false;
    }

    @Specialization(guards = "isRubySymbol(object)")
    protected boolean isTaintedSymbol(DynamicObject object) {
        return false;
    }

    @Specialization(guards = "!isRubySymbol(object)")
    protected boolean isTainted(DynamicObject object,
            @Cached ReadObjectFieldNode readTaintedNode) {
        return (boolean) readTaintedNode.execute(object, Layouts.TAINTED_IDENTIFIER, false);
    }

    @Fallback
    protected boolean isTainted(Object object) {
        return false;
    }
}
