// @formatter:off
/*
 ***** BEGIN LICENSE BLOCK *****
 * Version: EPL 2.0/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Eclipse Public
 * License Version 1.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.eclipse.org/legal/epl-v10.html
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * Copyright (C) 2013-2017 The JRuby Team (jruby@jruby.org)
 * 
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the EPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the EPL, the GPL or the LGPL.
 ***** END LICENSE BLOCK *****/          
package org.truffleruby.parser.parser;

public class YyTables {
   private static short[] combine(short[] t1, short[] t2, 
                                  short[] t3, short[] t4) {
      short[] t = new short[t1.length + t2.length + t3.length + t4.length];
      int index = 0;
      System.arraycopy(t1, 0, t, index, t1.length);
      index += t1.length;
      System.arraycopy(t2, 0, t, index, t2.length);
      index += t2.length;
      System.arraycopy(t3, 0, t, index, t3.length);
      index += t3.length;
      System.arraycopy(t4, 0, t, index, t4.length);
      return t;
   }

   public static final short[] yyTable() {
      return combine(yyTable1(), yyTable2(), yyTable3(), yyTable4());
   }

   public static final short[] yyCheck() {
      return combine(yyCheck1(), yyCheck2(), yyCheck3(), yyCheck4());
   }
   private static final short[] yyTable1() {
      return new short[] {

          305,  378,  308,  487,  307,   88,   88,  134,  134,  418, 
          214,  420,  250,  250,  250,  228,  302,  276,  250,  250, 
          354,  664,  250,  230,  645,   83,   83,  683,  465,  427, 
          603,  324,  352,  288,  288,  228,  107,  288,  670,  645, 
          632,  633,  645,  230,  215,  215,   74,  440,   88,  579, 
          133,  133,  350,  616,  250,  260,  360,  643,  682,    8, 
          133,  682,  431,  997,  646,  215,  683,  292,  292,    8, 
          645,  292,  301,  659,  254,  257,  634,  581,   73,  295, 
           86,  291,  291,  579,  304,  291,  707,  673,  645,  295, 
          215,  599,  435,  607,  753,  133,  612,   72,  579,  331, 
          335,  433,  350,  573,  526,  550,  580,  629,  528,  782, 
          934,  581,  514,  330,  745,  645,  274,  634,    8,  645, 
          644, 1027,  692,  288,  430,  802,  581,  591,  755,   71, 
          389,  250,  604,   88,  599,  932,  607,  573,  295,  811, 
          580, 1020,  297,  645,  395,  133,  704,  314,  260,  795, 
          644,  523,  573,  376,  550,  580,  300,  432,  785,  805, 
          787,  591,  376,  301,  645,  304,  573,  523,  319,  256, 
          823,  461,  310,  565,  645,  775,  591,  748,  710,  389, 
          997,  604,  841,  304,  644,  522,  640,  432,  877,   83, 
          276,  312,  645,  395,  884,   86,  565,  475,  736,  256, 
          573,  525,  565,  410,  640,  345,  487,  315,  745,  644, 
          745,  749,   72,   53,  275,  573,  781, 1059,  255,  623, 
          410,  604,  565,  300, 1027,  256,  786,  645,  683,  382, 
          746,  436,  836,  509,  932,  377,  412,   88,  256,  375, 
           83,  380,  106,  106,  377, 1020,  383,  378,  255,  106, 
          106,  106,  842,  319,  345,  106,  106,  365,  381,  106, 
          376,  682,  682,  379,  250,  250,  565,  500,  377,  934, 
          604,  534,  807,   75,  255,  924,  645,  466,  517,  530, 
          449,  634,  745,   81,  718,  106,   79,  255,  450,  719, 
          932,  106,  736,  367,  736,  942,  371,  645,  634,  373, 
          250,  260,  250,  250,  324,  422,  250,  645,  250,  551, 
          867,  892,  819,   88,  552,  288,  423,  288,  288,   88, 
           76,   88,  724,  725,   75,  470,  471,  890,  438,  869, 
          250,  790,  439,  872,   81,  501,  502,   79,  932,  512, 
          516,  295,  295,  295,  579,   17,  295,  295,  295,  292, 
          295,  292,  350,  289,  295,  558,  645,  296,  726,  634, 
          559,  642,  304,  291,  260,  291,  736,  424,  106,  645, 
          106,   76,  295,  295,  295,  295,  295,  645,   74,  915, 
          428,  579,  250,  250,  250,  250,   88,  250,  250,  303, 
          498,  613,  815,  617,   17,  133,  646,  627,  631,  646, 
          550,  616,  441,  632,  633,  745,  566,  745,  986,  625, 
           73,  324,  518,  519,  579,  303,  645,  634,  295,  928, 
          250,  295,    8,  645,  429,  250,  581,  250,   88,  350, 
           72,  250,  250,  507,   88,  288,  452,  550,   84,  808, 
          667,  319,  295,  562,  563,  564,  565,  250,  274,   88, 
          250,  274,  573,  832,   83,  580,  608,  683,   83,  608, 
           88,  573,  288,  250,  633,  530,  295,  988,   88,  609, 
          550,  965,  133,  821,  106,  645,  591,  250,  288,  389, 
          690,  604,  456,  608,  645,   92,   92,  135,  135,   84, 
          622,  682,  288,  395,  622,  456,  609,  277,  573,  608, 
          455,  106,  106,  462,  867,  565,  645,  250,  867,   88, 
           16,  228,  609,  608,  669,  669,  604,  536,  537,  230, 
           88,   83,  276,  312,  539,  276,  609,  815,   92,   83, 
         1083,  573,  351,  665,  565,  681,  250,  106,  350,  106, 
          106,  215,   75,  106, 1030,  106,  275,  608,  319,  275, 
          106,  288,   81,  604,  530,   79,  106,   97,  106,   16, 
          609,  382,  599,  832,  921,   77,  745,  106,  754,  536, 
          537,  375,  608,  380,  700,  684,  539,  761,  383,  378, 
          655,  133,  351,  688,  689,  609,  604,  645,  663,   76, 
          381,   97,  376,  463,  759,  379,  599,   17,   17,   17, 
          377,  697,  765,   17,   17,   75,   17,  768,   97,  464, 
           82,  599,  867,   92,  417,   81,   77,  761,   79,  106, 
          106,  106,  106,  106,  106,  106,  852,  350,  789,  384, 
          385,  822,  469,  761,  645,  488,  645,  491,  519,  478, 
          250,   88,  384,  385,  478,  645, 1064,  761,   95,   95, 
          758,   88,   76,  832,  788,  832,  467,  106,  764,  228, 
          278,   82,  106,  764,  106,  106,  565,  230,  106,  106, 
          133,  106,  794,  472,  467,  983,   80,   17,  761,  250, 
          779,  985,  482,  645,  106,  920,  106,  106,  215,  215, 
          589,   95,  758,  444,  645,  764,  886,  106,   17,   88, 
          106,  780,  447,  480,  350,  106,  530,   84,  480,  644, 
          774,  476,  102,  102,  106,  565, 1041,   92,  622,   83, 
          797,  794,   17,  868,  589,  873,  875,   80,  832, 1042, 
          483,  880,  882,  784,  994,  486,  287,  506,  816,  589, 
          812,  617,  386,  946,  106,  511,  106,  133,  950,  825, 
          826,  447,  827,  510,   88,  102,   88,  106,   88,  250, 
          250,  515,   16,   16,   16,  250,  250,  521,   16,   16, 
           84,   16,  832,  106,  832,  250,   95,  527,  817,  350, 
          536,  537,  107,  529,   78,  535,  820,  539,  645,  546, 
          645,  386,  544,   92,   52,  414,  550,   88,  626,   92, 
          250,   92,  415,  419,   52,  832,  724,  725,  554, 1040, 
          868,   88,   88,   88,  654,  460,  557,   59,   60,  645, 
          645, 1010,  133,  401,  894,  865,   97,  403,  404,  315, 
          868,  898,  351,  634,   77,   78,  504,  645,  647,  645, 
          102,  645,   16,  652,   88,   88,  645,  961,  645,  656, 
          666,  414,  685,   52,  686, 1071,  669,  599,  415,  416, 
          687,  907,  693,   16,  446,  446,   92,  332,  696,  954, 
          446,  701,  896,  712,   96,  519,  870,  106,  106,   82, 
           95,  250,  519,  519,  747,  772, 1085,   16,  106,   97, 
          773,  645,  756,  776,  599,   88,   88,   77,  757,  645, 
         1053, 1034,  762,  980,  645,  763,  783,   88,   92,  351, 
          800,  599,  133,  794,   92,  806,  106,  903,  801,  329, 
          333,  803,  923,  116,  938,   43,  126,  599,  122,   92, 
          414,  912,  913,  914,  804,   43,  106,  415,  443,  818, 
           92,  839,   82,  110,  102,   80,  133,  713,   92,  714, 
          715,  716,  717,  837,  845,  847,   95,  854, 1023,  447, 
          838,  844,   95,  645,   95,  929,  536,  537,  538,   88, 
          846,  848,  116,  539,  850,  126,  645,  122,  713,  851, 
          714,  715,  716,  717,   43,  589,   88,  871,   88,   92, 
          878,  106,  110,  106,   88,  106,  106,  106,  879, 1060, 
           92,  133,  106,  106,  519,  113,  993,  315,   80,  437, 
          895,  897,  106,  900,  904,  964,  865,  126,  351,  865, 
          102, 1017,  589,  865,  513,  645,  102,  984,  102,   95, 
          908,  447,  909,  910,  106,  916,  250,  106,  761,  589, 
          917,   88,  645,  645,  645,  922,  645,  645,  106,  106, 
          106,  288,  414,   78,  113,  589,   52,   52,   52,  415, 
          459,   52,   52,   52,  940,   52,  315,  943,  945,  947, 
          621,   95,  608,  315,  315,   52,  474,   95,  949,  620, 
          955,  106,  106,  958,  957,  609,  516,   52,   52,   52, 
           52,   52,   95,  102,  956,   91,  960,  989,  990,  292, 
          477,  478,  479,   95,  995,  481, 1035,  351, 1036,  292, 
          855,   95,  998,  999, 1037, 1001,   78, 1003,  106, 1002, 
         1005,   92, 1007,  865,  103,  865, 1013,  865, 1021,   91, 
          865,   92,  106,  106, 1028,  102,   52, 1029, 1031,   97, 
         1039,  102, 1047,  287,  106, 1000,   91,  536,  537,  541, 
         1004, 1006,   95, 1008,  539, 1009,  102,   52,  292, 1049, 
         1051, 1070, 1054,   95,  865,  460, 1061,  102,   96, 1069, 
         1072,  437,  493, 1074, 1084,  102,  513,  315, 1088,   92, 
          116,   52,  632,  126,  351,  122,  519,   43,   43,   43, 
          645,  634,  829,   43,   43,  633,   43,  634,  437,  713, 
          110,  714,  715,  716,  717,  930,  106,  933,  638,  640, 
          645,  315,  634,  616,  634,  518,  102,  638,   43,   43, 
           43,   43,   43,  106,  645,  106,  315,  102,  645,  645, 
          645,  106,  533,  830,   92,  645,   92,  480,   92,  375, 
          363,  519,  556,  620,  315,  444,  437,  630,  519,  519, 
          137,  315,  315,  437,  437,  217, 1033,  620,  315,  351, 
          698,  513,  113, 1073, 1075, 1076, 1077,   43,  513,  513, 
         1068,  769,  966,  106,  616,  617,  858,   92,  106,  936, 
          437,  931, 1016, 1015,   95,  885,  620,  279,   43,  493, 
         1067,   92,   92,   92,   95,  616,   98, 1082,  919,    0, 
            0,  678,  680,    0, 1014,  332, 1092,    0, 1018,    0, 
            0,    0,   43,  414,    0,    0,  493,    0,    0,    0, 
          415,  473,    0,  516,   92,   92,    0,  245,    0,    0, 
          516,  516,  493,  536,  537,  541,  617,  245,    0,    0, 
          539,  680,   95,  520,  332,    0,  493,    0,  102,  135, 
          677,    0,    0,  248,  248,  248,    0,  617,  102,  122, 
          437,  292,  292,  292,   91,  616, 1044,  292,  292,    0, 
          292,  245,    0,    0,  116,   92,   92,    0,   96,  206, 
            0,    0,    0,  981,    0,  245,  245,   92,    0,    0, 
          245,    0,  292,  292,  292,  292,  292,   95,    0,   95, 
            0,   95,  414,  115,    0,  493,  102,    0,  437,  415, 
          508,    0, 1079,  513,    0,  437,  437,    0,  621,    0, 
          513,  513,    0,  519,    0,    0,  616,   91,  206,    0, 
          519,  519,  752,  752,  770,    0,    0,    0,  135,    0, 
           95,  292,  437,    0,  866,    0,    0,  616,    0,   92, 
            0,    0,  518,    0,   95,   95,   95,    0,    0,  518, 
          518,  102,  292,  102,    0,  102,   92,  752,   92,  414, 
            0,  791,  437,    0,   92,  304,  415,  532,    0,  767, 
            0,    0,  414,    0,  767,    0,  292,   95,   95,  415, 
         1022,    0,    0,  414,    0,  315,  813,    0,  814,  426, 
          415, 1032,  315,  315,  102,  680,  332,    0,  620,  426, 
          536,  537,  543,    0,    0,    0,  767,  539,  102,  102, 
          102,   92,    0,  386,  793,  796,  315,  796,    0,  115, 
          543,  117,    0,    0,    0,  796,    0,    0,   95,   95, 
          287,  287,    0,    0,  287,    0,  982,    0,    0,   94, 
           95,  102,  102,    0,    0,  401,    0,    0,  426,  403, 
          404,  405,  406,  853,    0,    0,  311,  313,   96,    0, 
            0,    0,  287,  287,    0,  353,  355,    0,  115,  543, 
          520,    0,    0,   94,    0,    0,    0,  520,  520,  245, 
          245,  245,  889,    0,  245,  245,  245,    0,  245,    0, 
           94,    0,  102,  102,    0,  248,  248,   99,  245,  245, 
          518,    0,   95,    0,  102,    0,    0,  245,  245,  105, 
          245,  245,  245,  245,  245,    0,  911,    0,    0,   95, 
          516,   95,  245,    0,  104,  866,    0,   95,  866,    0, 
          866, 1024,  866,  714,  715,  716,  717,  497,    0,  499, 
          752,  245,  245,  105,  521,    0,  245,  245,  536,  537, 
          547,    0,    0,    0,  902,  539,  245,  906,  104,  245, 
          105,  520,  245,  939,  245,  206,  102,  206,  206,  206, 
          206,  206,    0,    0,   95,  104,  206,    0,  513,    0, 
          245,  124,    0,  102,    0,  102,    0,  519,    0,    0, 
          623,  102,  245,    0,    0,  519,  101,  963,    0,  206, 
          206,  100,    0,    0,  245,  619,  117,  206,  206,  206, 
          206,    0,  513,  248,  248,  248,  248,    0,  567,  568, 
            0,  536,  537,  555,    0,  332,    0,  513,  539,    0, 
          124,  102,  866,    0,  866,    0,  866,    0,  102,  866, 
          484,  484,  484,  523,    0,  206,    0,  484,  522,    0, 
            0,  426,  426,  426,    0,  117,  426,  426,  426,  513, 
          426,  713,  623,  714,  715,  716,  717,  718,    0,    0, 
          426,    0,  719,  866,  796,    0,  115,    0,  524,  426, 
          426,  646,  426,  426,  426,  426,  426,  520,   61,   62, 
           63,   64,  623,    0,    0,  720,  287,  287,  287,  355, 
            0,  287,  623,  722,  723,  724,  725,    0,   94,  617, 
            0,    0,  287,    0,  287,  287,  543,  426,  543,  543, 
          543,  543,  543,  505,  426,  426,    0,  543,  466,  466, 
          466,  426,  118,    0,    0,  466,  105,  518,  691,    0, 
            0,  726,    0,    0,  518,  518,    0,  472,  472,  472, 
          543,  623,  426,  124,  472,  623,    0,  516,  543,  543, 
          543,  543,    0,    0,  516,  516,  632,  632,  632,    0, 
          617,   94,  632,  632,    0,  632,  426,    0,  105,  623, 
            0,  521,    0,  623,    0,    0,    0,    0,  521,  521, 
            0,  617,    0,  104,  619,  493,  543,    0,  796,    0, 
            0,  569,  570,  571,  572,  573,  574,  575,  576,  577, 
          578,  579,  580,  581,  582,  583,  584,  585,  586,  587, 
          588,  589,  590,  591,  592,  593,  594,  623,    0,    0, 
            0,  120,  287,    0,  623,  623,  119,  614,  124,  618, 
          619,  105,  619,  287,  632,    0,  632,    0,  616,  619, 
          619,  623,    0,   93,    0,  619,  104,    0,  229,  287, 
            0,  623,    0,  117,    0,    0,  121,  632,  229,    0, 
            0,  248,    0,   91,    0,  287,  619,  614,  662,  632, 
          523,    0,  287,    0,    0,  522,    0,  523,  523,  287, 
            0,  632,  522,  522,    0,    0,  513,  287,  287,  623, 
          515,  287,  229,  513,  513,    0,  623,  623,    0,  616, 
          248,    0,  619,    0,    0,  524,  229,  229,    0,    0, 
          513,  229,  524,  524,  520,  694,  695,  513,    0,    0, 
          616,  520,  520,  623,    0,    0,    0,  287,    0,    0, 
          287,  199,  379,  380,  381,  382,  383,    0,  287,  619, 
          619,  619,    0,  619,  623,  623,  623,  619,  619,    0, 
          623,  623,  619,  623,  619,  619,  619,  619,  619,  619, 
          619,    0,  623,    0,    0,  619,  619,  619,  619,  619, 
          619,  619,  623,  623,  619,  623,  623,  623,  623,  623, 
          199,  619,   94,    0,  619,  619,  619,    0,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  623, 
          623,  623,  623,  623,  623,  623,  623,  623,  623,  623, 
          623,  623,  623,   92,    0,  623,  623,  623,  623,  619, 
          287,  248,  619,  619,  623,  619,  619,    0,  619,  516, 
          619,  623,  619,  623,  619,  623,  623,  623,  623,  623, 
          623,  623,  619,  623,  623,  623,    0,  619,  619,  619, 
          619,  619,  619,    0,    0,    0,  619,  287,  619,  619, 
          514,  619,  619,  619,  619,  513,  619,  619,  619,  623, 
            0,    0,    0,    0,    0,  513,    0,    0,  112,  633, 
          633,  633,  287,    0,  287,  633,  633,    0,  633,  944, 
            0,  287,  287,  948,    0,    0,    0,    0,  110,  616, 
            0,    0,    0,    0,  542,    0,  545,    0,  548,  513, 
          229,  229,  229,    0,  548,  229,  229,  229,    0,  229, 
            0,    0,    0,  513,  513,    0,  110,  515,  513,  229, 
          229,    0,    0,    0,  515,  515,    0,    0,  229,  229, 
            0,  229,  229,  229,  229,  229,    0,  513,    0,  287, 
            0,    0,    0,  229,  513,  513,  513,    0,    0,  633, 
          616,  874,  876,    0,    0,    0,    0,  881,  883,    0, 
            0,    0,    0,    0,    0,    0,  887,  662,  287,    0, 
          633,  616,  891,    0,    0,    0,    0,  229,    0,    0, 
          229,    0,    0,  229,    0,  229,    0,  516,    0,    0, 
            0,    0,    0,    0,  633,  874,  876,  516,  881,  883, 
            0,  229,  287,    0,    0,    0,    0,  113,  617,    0, 
            0,    0,  542,  229,    0,    0,    0,  199,    0,  199, 
          199,  199,  199,  199,    0,  229,    0,    0,  199,    0, 
            0,  516,    0, 1048,    0,    0, 1050, 1052,  111,  518, 
         1055, 1056,    0,    0,    0,  516,  516,  518,  113,  287, 
          516,  199,  199,    0,    0,    0,  516,    0,    0,  199, 
          199,  199,  199,  516,  516,    0,  959,    0,    0,  617, 
            0,    0,    0,  962,    0,    0,    0,    0,  516,    0, 
            0,  711,    0,  287,    0,    0,    0,  514,  711,    0, 
          617,    0,    0,    0,  514,  514,    0,  199,    0,    0, 
         1087, 1089, 1090, 1091,  962,    0,    0,  209,    0,    0, 
            0,  287,  616,  616,  616, 1094,  616,  513,  513,  513, 
          616,  616,  513,  513,  513,  616,  513,  616,  616,  616, 
          616,  616,  616,  616,    0,  616,  513,  513,  616,  616, 
          616,  616,  616,  616,  616,  513,  513,  616,  513,  513, 
          513,  513,  513,    0,  616,    0,  209,  616,  616,  616, 
          513,  616,  616,  616,  616,  616,  616,  616,  616,  616, 
          616,  616,  513,  513,  513,  513,  513,  513,  513,  513, 
          513,  513,  513,  513,  513,  513,    0,    0,  513,  513, 
          513,  513,  616,  525,  513,  616,  616,  513,  616,  616, 
          513,  616,  513,  616,  513,  616,  513,  616,  513,  513, 
          513,  513,  513,  513,  513,  616,  513,  616,  513,   95, 
          616,  616,  616,  616,  616,  616,    0,    0,  287,  616, 
          513,  616,  616,    0,  616,  616,  616,  616,  194,  616, 
          616,  616,  513,    0,  617,  617,  617,    0,  617,  516, 
          516,  516,  617,  617,  516,  516,  516,  617,  516,  617, 
          617,  617,  617,  617,  617,  617,  517,  617,  516,  516, 
          617,  617,  617,  617,  617,  617,  617,  516,  516,  617, 
          516,  516,  516,  516,  516,    0,  617,  194,    0,  617, 
          617,  617,  516,  617,  617,  617,  617,  617,  617,  617, 
          617,  617,  617,  617,  516,  516,  516,  516,  516,  516, 
          516,  516,  516,  516,  516,  516,  516,  516,  109,    0, 
          516,  516,  516,  516,  617,  528,  516,  617,  617,  516, 
          617,  617,  516,  617,  516,  617,  516,  617,  516,  617, 
          516,  516,  516,  516,  516,  516,  516,  617,  516,  617, 
          516,    0,  617,  617,  617,  617,  617,  617,    0,    0, 
          623,  617,  516,  617,  617,  305,  617,  617,  617,  617, 
          623,  617,  617,  617,  516,    0,    0,    0,  713,    0, 
          714,  715,  716,  717,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  713,    0,  714, 
          715,  716,  717,  209,  623,  209,  209,  209,  209,  209, 
            0,    0,  830,    0,  209,    0,    0,    0,  623,  623, 
          831,  124,    0,  623,  713,  520,  714,  715,  716,  717, 
          718,  830,    0,  520,    0,  719,    0,  209,  209,  996, 
            0,    0,    0,    0,    0,  209,  209,  209,  209,    0, 
            0,  623,    0,    0,  114,    0,    0,    0,  720,  721, 
            0,    0,    0,    0,    0,    0,  722,  723,  724,  725, 
          386,  713,    0,  714,  715,  716,  717,  718,    0,    0, 
            0,    0,  719,  209,    0,    0,  399,  400,    0,    0, 
            0,  619,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  619,  401,  517,  726,  720,  403,  404,  405,  406, 
          517,  517,    0,  722,  723,  724,  725,    0,    0,    0, 
            0,  713,    0,  714,  715,  716,  717,  718,    0,    0, 
            0,    0,  719,    0,  194,  619,  194,  194,  194,  194, 
          194,    0,    0,    0,    0,  194,    0,    0,    0,  619, 
          619,  726,  123,  128,  619,  720,  521,  386,    0,    0, 
            0,    0,    0,    0,  521,  724,  725,    0,  194,  194, 
            0,    0,    0,  399,  400,    0,  194,  194,  194,  194, 
            0,    0,  619,    0,    0,    0,    0,    0,    0,  401, 
            0,  402,    0,  403,  404,  405,  406,    0,    0,    0, 
            0,  726,  305,    0,    0,    0,    0,    0,    0,  305, 
          305,  176,    0,    0,  194,    0,    0,  619,  619,  619, 
            0,  619,  623,  623,  623,  619,  619,  623,  623,  623, 
          619,  623,  619,  619,  619,  619,  619,  619,  619,    0, 
          623,  623,  623,  619,  619,  619,  619,  619,  619,  619, 
          623,  623,  619,  623,  623,  623,  623,  623,    0,  619, 
          176,    0,  619,  619,  619,  623,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  623,  623,  623, 
          623,  623,  623,  623,  623,  623,  623,  623,  623,  623, 
          623,  108,    0,  623,  623,  623,  623,  619,    0,  623, 
          619,  619,  623,  619,  619,  623,  619,  623,  619,  623, 
          619,  623,  619,  623,  623,  623,  623,  623,  623,  623, 
          619,  623,  623,  623,    0,  619,  619,  619,  619,  619, 
          619,    0,    0,    0,  619,  623,  619,  619,  316,  619, 
          619,  619,  619,    0,  619,  619,  619,  623,  619,  619, 
          619,    0,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
            0,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,    0, 
          619,    0,    0,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,    0,    0,  619,  619,  619,  619,  619,    0, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,    0,  619,  619,  619,  619, 
          619,  619,    0,    0,  620,  619,  619,  619,  619,    0, 
          619,  619,  619,  619,  620,  619,  619,  619,  619,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  176,  620,  176, 
          176,  176,  176,  176,    0,    0,    0,    0,  176,    0, 
            0,    0,  620,  620,    0,  125,  127,  620,    0,  523, 
            0,    0,    0,    0,    0,    0,    0,  523,    0,    0, 
            0,  176,  176,    0,    0,    0,    0,    0,    0,  176, 
          176,  176,  176,    0,    0,  620,    0,    0,    0,    0, 
          386,  387,  388,  389,  390,  391,  392,  393,  394,  395, 
          396,    0,    0,    0,    0,  316,  399,  400,    0,    0, 
            0,    0,  316,  316,    0,    0,    0,  176,    0,    0, 
            0,    0,  401,    0,  402,  306,  403,  404,  405,  406, 
          407,  408,  409,    0,  410,  306,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  596,  597,    0, 
            0,  598,    0,    0,    0,  183,  184,    0,  185,  186, 
          187,  188,  189,  190,  191,    0,    0,  192,  193,  306, 
            0,    0,  194,  195,  196,  197,    0,    0,    0,    0, 
            0,    0,  300,  306,  306,    0,    0,  386,  306,  199, 
          200,    0,  201,  202,  203,  204,  205,  206,  207,  208, 
          209,  210,  211,  399,  400,  212,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  306,    0,    0,  401, 
            0,  402,    0,  403,  404,  405,  406,  213,    0,  409, 
            0,  410,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  177,    0,    0,    0,    0, 
            0,  620,  620,  620,    0,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,    0,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,    0,  620,  177,    0,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,    0,    0,  620,  620,  620, 
          620,  620,    0,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,    0,  620, 
          620,  620,  620,  620,  620,    0,    0,    0,  620,  620, 
          620,  620,    0,  620,  620,  620,  620,    0,  620,  620, 
          620,  620,  618,  618,  618,    0,  618,  306,  306,  306, 
          618,  618,  306,  306,  306,  618,  306,  618,  618,  618, 
          618,  618,  618,  618,    0,  618,  306,  306,  618,  618, 
          618,  618,  618,  618,  618,  306,  306,  618,  306,  306, 
          306,  306,  306,    0,  618,    0,    0,  618,  618,  618, 
          306,  618,  618,  618,  618,  618,  618,  618,  618,  618, 
          618,  618,  306,  306,  306,  306,  306,  306,  306,  306, 
          306,  306,  306,  306,  306,  306,    0,    0,  306,  306, 
          306,  306,  618,    0,  306,  618,  618,  306,  618,  618, 
          306,  618,  306,  618,  306,  618,  306,  618,  306,  306, 
          306,  306,  306,  306,  306,  618,  306,  618,  306,    0, 
          618,  618,  618,  618,  618,  618,    0,    0,  624,  618, 
          306,  618,  618,    0,  618,  618,  618,  618,  624,  618, 
          618,  618,  306,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  177,  624,  177,  177,  177,  177,  177,    0,    0, 
            0,    0,  177,    0,    0,    0,  624,  624,    0,    0, 
            0,  624,    0,  522,  386,  387,  388,  389,  390,  391, 
          392,  522,    0,  395,  396,  177,  177,    0,    0,    0, 
          399,  400,    0,  177,  177,  177,  177,    0,    0,  624, 
            0,    0,    0,    0,    0,    0,  401,    0,  402,    0, 
          403,  404,  405,  406,  407,  408,  409,    0,  410,    0, 
            0,    0,    0,    0,    0,    4,    5,    6,    0,    8, 
            0,  177,    0,    9,   10,    0,    0,    0,   11,  625, 
           12,   13,   14,  108,  109,   17,   18,    0,    0,  625, 
            0,  110,  111,  112,  113,  114,  115,  116,    0,    0, 
          117,    0,    0,    0,    0,    0,    0,  118,    0,    0, 
          119,  120,  121,    0,  122,   35,  123,  124,  125,  126, 
            0,   40,    0,  625,  127,    0,    0,  386,    0,    0, 
            0,    0,  391,  392,    0,    0,    0,  625,  625,    0, 
            0,  128,  625,  399,  400,  129,    0,    0,  130,    0, 
            0,   46,   47,    0,   48,    0,    0,    0,    0,  401, 
            0,  402,    0,  403,  404,  405,  406,  407,  408,  409, 
          625,  410,    0,   52,   53,   54,   55,   56,   57,    0, 
            0,    0,   58,    0,   59,   60,    0,   61,   62,   63, 
           64,    0,   65,   66,    0,    0,    0,    0,    0,  178, 
            0,    0,    0,    0,    0,  621,  621,  621,    0,  621, 
          624,  624,  624,  621,  621,  624,  624,  624,  621,  624, 
          621,  621,  621,  621,  621,  621,  621,    0,  624,  624, 
          624,  621,  621,  621,  621,  621,  621,  621,  624,  624, 
          621,  624,  624,  624,  624,  624,    0,  621,  178,    0, 
          621,  621,  621,  624,  621,  621,  621,  621,  621,  621, 
          621,  621,  621,  621,  621,  624,  624,  624,  624,  624, 
          624,  624,  624,  624,  624,  624,  624,  624,  624,    0, 
            0,  624,  624,  624,  624,  621,    0,  624,  621,  621, 
          624,  621,  621,  624,  621,  624,  621,  624,  621,  624, 
          621,  624,  624,  624,  624,  624,  624,  624,  621,  624, 
          624,  624,    0,  621,  621,  621,  621,  621,  621,    0, 
            0,    0,  621,  624,  621,  621,    0,  621,  621,  621, 
          621,    0,  621,  621,  621,  624,  622,  622,  622,    0, 
          622,  625,  625,  625,  622,  622,  625,  625,  625,  622, 
          625,  622,  622,  622,  622,  622,  622,  622,    0,  625, 
          625,  625,  622,  622,  622,  622,  622,  622,  622,  625, 
          625,  622,  625,  625,  625,  625,  625,  411,  622,    0, 
            0,  622,  622,  622,  625,  622,  622,  622,  622,  622, 
          622,  622,  622,  622,  622,  622,  625,  625,  625,  625, 
          625,  625,  625,  625,  625,  625,  625,  625,  625,  625, 
            0,    0,  625,  625,  625,  625,  622,    0,  625,  622, 
          622,  625,  622,  622,  625,  622,  625,  622,  625,  622, 
          625,  622,  625,  625,  625,  625,  625,  625,  625,  622, 
          625,  625,  625,    0,  622,  622,  622,  622,  622,  622, 
            0,    0,  315,  622,  625,  622,  622,    0,  622,  622, 
          622,  622,  315,  622,  622,  622,  625,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  178,  315,  178,  178,  178, 
          178,  178,    0,    0,    0,    0,  178,    0,    0,    0, 
          315,  315,    0,  126,    0,  315,    0,  524,    0,    0, 
            0,    0,    0,    0,    0,  524,    0,    0,    0,  178, 
          178,    0,    0,    0,    0,    0,    0,  178,  178,  178, 
          178,    0,    0,  315,    0,    0,    0,    0,  386,  387, 
          388,  389,  390,  391,  392,  393,    0,  395,  396,    0, 
            0,    0,    0,    0,  399,  400,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  178,    0,    0,    0,    0, 
          401,    0,  402,  436,  403,  404,  405,  406,  407,  408, 
          409,    0,  410,  436,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  771,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  436,    0,    0, 
            0,  386,  387,  388,  389,  390,  391,  392,  393,  394, 
          395,  396,  436,  397,  398,    0,  436,  399,  400,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  401,    0,  402,    0,  403,  404,  405, 
          406,  407,  408,  409,  436,  410,    0,    0,    0,    0, 
            0,    0,  134,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  620, 
          620,  620,    0,  620,  315,  315,  315,  620,  620,  315, 
          315,  315,  620,  315,  620,  620,  620,  620,  620,  620, 
          620,  134,    0,  315,  315,  620,  620,  620,  620,  620, 
          620,  620,  315,  315,  620,  315,  315,  315,  315,  315, 
            0,  620,    0,    0,  620,  620,  620,  315,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  315, 
          315,  315,  315,  315,  315,  315,  315,  315,  315,  315, 
          315,  315,  315,  105,    0,  315,  315,  315,  315,  620, 
            0,  315,  620,  620,  315,  620,  620,  315,  620,  315, 
          620,  315,  620,  315,  620,  315,  315,  315,  315,  315, 
          315,  315,  620,  315,    0,  315,    0,  620,  620,  620, 
          620,  620,  620,    0,    0,    0,  620,  315,  620,  620, 
          623,  620,  620,  620,  620,    0,  620,  620,  620,  315, 
          282,  282,  282,    0,  282,  436,  436,  436,  282,  282, 
          436,  436,  436,  282,  436,  282,  282,  282,  282,  282, 
          282,  282,  411,  436,  436,  436,  282,  282,  282,  282, 
          282,  282,  282,  436,  436,  282,  436,  436,  436,  436, 
          436,    0,  282,    0,    0,  282,  282,  282,    0,  282, 
          282,  282,  282,  282,  282,  282,  282,  282,  282,  282, 
          436,  436,  436,  436,  436,  436,  436,  436,  436,  436, 
          436,  436,  436,  436,  645,    0,  436,  436,  436,  436, 
          282,    0,  436,  282,  645,  436,  282,  282,  436,  282, 
          436,  282,  436,  282,  436,  282,  436,  436,  436,  436, 
          436,  436,  436,  282,  436,  436,  436,    0,  282,  282, 
          282,  282,  282,  282,    0,    0,    0,  282,  645,  282, 
          282,    0,  282,  282,  282,  282,    0,  282,  282,  282, 
          436,    0,    0,  645,    0,    0,    0,  645,  134,    0, 
          134,  134,  134,  134,  134,    0,    0,    0,    0,  134, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          513,    0,    0,    0,    0,  645,    0,    0,  513,    0, 
            0,    0,  134,  134,    0,    0,    0,    0,  124,  623, 
          134,  134,  134,  134,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  322,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  322,    0,    0,  134,    0, 
            0,    0,    0,    0,    0,    0,    0,  623,    0,    0, 
            0,    0,    0,    0,  623,  623,    0,    0,    0,    0, 
          619,    0,    0,    0,    0,    0,    0,    0,    0,  322, 
            0,    0,    0,    0,    0,    0,  777,    0,    0,    0, 
            0,  623,    0,    0,  322,    0,    0,    0,  322,    0, 
            0,    0,    0,    0,    0,    0,  386,  387,  388,  389, 
          390,  391,  392,  393,  394,  395,  396,    0,  397,  398, 
            0,    0,  399,  400,    0,    0,  322,    0,    0,    0, 
            0,    0,    0,    0,  135,    0,    0,    0,  401,    0, 
          402,    0,  403,  404,  405,  406,  407,  408,  409,    0, 
          410,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  282,  282,  282,    0,  282,  645,  645,  645,  282, 
          282,  645,  645,  645,  282,  645,  282,  282,  282,  282, 
          282,  282,  282,  135,  645,  645,  645,  282,  282,  282, 
          282,  282,  282,  282,  645,  645,  282,  645,  645,  645, 
          645,  645,    0,  282,    0,    0,  282,  282,  282,    0, 
          282,  282,  282,  282,  282,  282,  282,  282,  282,  282, 
          282,  645,  645,  645,  645,  645,  645,  645,  645,  645, 
          645,  645,  645,  645,  645,  104,    0,  645,  645,  645, 
          645,  282,    0,  645,  282,    0,  645,  282,  282,  645, 
          282,  645,  282,  645,  282,  645,  282,  645,  645,  645, 
          645,  645,  645,  645,  282,  645,  645,  645,    0,  282, 
          282,  282,  282,  282,  282,    0,    0,    0,  282,    0, 
          282,  282,  619,  282,  282,  282,  282,    0,  282,  282, 
          282,  645,  282,  282,  282,    0,  282,  322,  322,  322, 
          282,  282,  322,  322,  322,  282,  322,  282,  282,  282, 
          282,  282,  282,  282,  411,    0,  322,  322,  282,  282, 
          282,  282,  282,  282,  282,  322,  322,  282,  322,  322, 
          322,  322,  322,    0,  282,    0,    0,  282,  282,  282, 
            0,  282,  282,  282,  282,  282,  282,  282,  282,  282, 
          282,  282,  322,  322,  322,  322,  322,  322,  322,  322, 
          322,  322,  322,  322,  322,  322,  513,    0,  322,  322, 
          322,  322,  282,    0,  322,  282,  513,  322,  282,  282, 
          322,  282,  322,  282,  322,  282,  322,  282,  322,  322, 
          322,  322,  322,  322,  322,  282,  322,    0,  322,    0, 
          282,  282,  282,  282,  282,  282,    0,    0,    0,  282, 
           91,  282,  282,    0,  282,  282,  282,  282,    0,  282, 
          282,  282,  322,    0,    0,  513,    0,  110,    0,  513, 
          135,    0,  135,  135,  135,  135,  135,    0,    0,    0, 
            0,  135,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  516,    0,    0,    0,    0,  513,    0,    0, 
          516,    0,    0,    0,  135,  135,    0,    0,    0,    0, 
          123,  619,  135,  135,  135,  135,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  516,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  516,    0,    0, 
          135,    0,    0,    0,    0,    0,    0,    0,    0,  619, 
            0,    0,    0,    0,    0,    0,  619,  619,    0,    0, 
            0,    0,  619,    0,    0,    0,    0,    0,    0,    0, 
            0,   94,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  619,    0,    0,  516,    0,  113,    0, 
          516,    0,    0,    0,    0,    0,    0,  107,  386,  387, 
          388,  389,  390,  391,  392,  393,  394,  395,  396,    0, 
          397,  398,    0,    0,  399,  400,    0,    0,  516,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          401,  107,  402,    0,  403,  404,  405,  406,  407,  408, 
          409,    0,  410,    0,    0,    0,    0,    0,  107,    0, 
            0,    0,    0,  616,  616,  616,    0,  616,  513,  513, 
          513,  616,  616,    0,  513,  513,  616,  513,  616,  616, 
          616,  616,  616,  616,  616,    0,  616,    0,  315,  616, 
          616,  616,  616,  616,  616,  616,  513,  513,  616,  513, 
          513,  513,  513,  513,    0,  616,  106,    0,  616,  616, 
          616,    0,  616,  616,  616,  616,  616,  616,  616,  616, 
          616,  616,  616,  513,  513,  513,  513,  513,  513,  513, 
          513,  513,  513,  513,  513,  513,  513,    0,    0,  513, 
          513,  513,  513,  616,  525,    0,  616,  616,  513,  616, 
          616,    0,  616,  620,  616,  513,  616,  513,  616,  513, 
          513,  513,  513,  513,  513,  513,  616,  513,  616,  513, 
            0,  616,  616,  616,  616,  616,  616,    0,    0,    0, 
          616,    0,  616,  616,    0,  616,  616,  616,  616,    0, 
          616,  616,  616,  513,  617,  617,  617,    0,  617,  516, 
          516,  516,  617,  617,    0,  516,  516,  617,  516,  617, 
          617,  617,  617,  617,  617,  617,    0,  617,  857,    0, 
          617,  617,  617,  617,  617,  617,  617,  516,  516,  617, 
          516,  516,  516,  516,  516,    0,  617,    0,    0,  617, 
          617,  617,    0,  617,  617,  617,  617,  617,  617,  617, 
          617,  617,  617,  617,  516,  516,  516,  516,  516,  516, 
          516,  516,  516,  516,  516,  516,  516,  516,    0,    0, 
          516,  516,  516,  516,  617,  528,  107,  617,  617,  516, 
          617,  617,    0,  617,    0,  617,  516,  617,  516,  617, 
          516,  516,  516,  516,  516,  516,  516,  617,  516,  617, 
          516,    0,  617,  617,  617,  617,  617,  617,  925,    0, 
          619,  617,    0,  617,  617,  935,  617,  617,  617,  617, 
          619,  617,  617,  617,  516,  315,    0,    0,    0,    0, 
            0,    0,  315,  315,    0,    0,    0,    0,  620,  107, 
            0,  125,  620,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  104,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  619, 
            0,  123,    0,  619,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          620,    0,    0,    0,    0,    0,    0,  620,  620,    0, 
            0,  619,    0,  620,    0,    0,    0,    0,    0, 1011, 
            0,    0,    0,    0,    0,    0, 1019,    0,    0,    0, 
            0,    0,    0,    0,  620,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  620,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  620,    0,    0,   96,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0, 1043,    0, 1045,    0, 1046,    0,    0,    0, 
            0,    0,    0,    0,    0,  106,    0,    0,   96,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          620,    0,  125,    0,  620,   96,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0, 1078,    0, 
         1080,    0,    0, 1081,    0,    0,    0,    0,    0,    0, 
            0,    0,  620,    0,    0,  518,    0,    0,    0,    0, 
            0,    0,    0, 1086,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0, 1093,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  619,  619,  619, 
            0,  619,  619,  619,  619,  619,  619,    0,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,    0, 
          619,    0,    0,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,    0,  619, 
            0,    0,  619,  619,  619,    0,  619,  619,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
      };
   }

   private static final short[] yyTable2() {
      return new short[] {

          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,    0,    0,  619,  619,  619,  619,  619,    0,    0, 
          619,  619,  619,  619,  619,    0,  619,    0,  619,  619, 
          619,  619,  619,  619,  619,  619,  619,  619,  619,  619, 
          619,  619,  619,  619,    0,  619,  619,  619,  619,  619, 
          619,    0,    0,  645,  619,    0,  619,  619,    0,  619, 
          619,  619,  619,  645,  619,  619,  619,  619,  620,  620, 
          620,    0,  620,  620,  620,  620,  620,  620,    0,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
            0,  620,    0,   96,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,    0, 
          620,    0,  645,  620,  620,  620,    0,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  518,    0,  620,  620,  620,  620,  620,  518, 
          518,  620,  620,  620,  620,  620,   96,  620,    0,  620, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  620,  620,  620,  620,    0,  620,  620,  620,  620, 
          620,  620,    0,    0,  315,  620,    0,  620,  620,    0, 
          620,  620,  620,  620,  315,  620,  620,  620,  620,    0, 
            0,    0,    0,    0,    0,    0,  604,  605,    0,    0, 
          606,    0,    0,    0,  183,  184,    0,  185,  186,  187, 
          188,  189,  190,  191,    0,    0,  192,  193,  107,    0, 
            0,  194,  195,  196,  197,    0,    0,    0,    0,    0, 
            0,  300,    0,  315,    0,  126,    0,  315,  199,  200, 
            0,  201,  202,  203,  204,  205,  206,  207,  208,  209, 
          210,  211,    0,    0,  212,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  315,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  213,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  123,    0,    0,  125,    0,    0,    0,    0, 
          282,  282,  282,    0,  282,  645,  645,  645,  282,  282, 
          645,  645,  645,  282,  645,  282,  282,  282,  282,  282, 
          282,  282,    0,  645,  645,    0,  282,  282,  282,  282, 
          282,  282,  282,  645,  645,  282,  645,  645,  645,  645, 
          645,  123,  282,    0,  125,  282,  282,  282,    0,  282, 
          282,  282,  282,  282,  282,  282,  282,  282,  282,  282, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  645,    0,  619,    0,    0,  620,    0,  645,  645, 
          282,    0,    0,  282,    0,  645,  282,  282,    0,  282, 
            0,  282,    0,  282,    0,  282,    0,  261,    0,    0, 
            0,    0,    0,  282,    0,  645,  645,  261,  282,  282, 
          282,  282,  282,  282,    0,    0,    0,  282,    0,  282, 
          282,    0,  282,  282,  282,  282,    0,  282,  282,  282, 
          645,  620,  620,  620,    0,  620,  315,  315,  315,  620, 
          620,  261,  315,  315,  620,  315,  620,  620,  620,  620, 
          620,  620,  620,    0,    0,  261,  261,  620,  620,  620, 
          620,  620,  620,  620,  315,  315,  620,  315,  315,  315, 
          315,  315,    0,  620,    0,    0,  620,  620,  620,    0, 
          620,  620,  620,  620,  620,  620,  620,  620,  620,  620, 
          620,  315,  315,  315,  315,  315,  315,  315,  315,  315, 
          315,  315,  315,  315,  315,    0,    0,  315,  315,  315, 
          315,  620,    0,    0,  620,  620,  315,  620,  620,    0, 
          620,    0,  620,  315,  620,  315,  620,  315,  315,  315, 
          315,  315,  315,  315,  620,  315,  437,  315,    0,  620, 
          620,  620,  620,  620,  620,    0,  437,  862,  620,  123, 
          620,  620,  125,  620,  620,  620,  620,    0,  620,  620, 
          620,  315,  619,    0,    0,  620,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          437,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  437,  437,    0,  122,    0,  437, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          619,    0,    0,  620,    0,    0,    0,  619,  619,    0, 
          620,  620,  118,  619,    0,    0,  620,  437,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  436,    0, 
            0,    0,    0,    0,  619,    0,  937,  620,  436,  937, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  261, 
          261,  261,    0,    0,  261,  261,  261,    0,  261,    0, 
            0,  118,    0,    0,    0,    0,    0,    0,  261,  261, 
            0,    0,  436,    0,    0,    0,    0,  261,  261,    0, 
          261,  261,  261,  261,  261,    0,  436,  436,    0,    0, 
            0,  436,  261,  521,    0,    0,    0,    0,    0,    0, 
            0,    0,  103,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  436, 
            0,    0,    0,    0,    0,    0,  261,    0,  862,  261, 
            0,  862,  261,  937,  261,  862,  103,    0,    0,    0, 
            0,    0,  519,    0,    0,    0,    0,    0,    0,    0, 
          261,    0,  519,  103,    0,    0,    0,    0,    0,    0, 
            0,    0,  261,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  261,    0,    0,    0,    0,    0, 
            0,    0,    0,  437,    0,    0,  519,    0,  437,  437, 
          437,    0,    0,  437,  437,  437,    0,  437,    0,    0, 
          519,  519,    0,  116,    0,  519,  437,  437,  437,    0, 
            0,    0,    0,    0,    0,    0,  437,  437,    0,  437, 
          437,  437,  437,  437,    0,    0,    0,    0,    0,    0, 
            0,  437,    0,  519,    0,  862,    0,  862,    0,  862, 
            0,    0,  862,  437,  437,  437,  437,  437,  437,  437, 
          437,  437,  437,  437,  437,  437,  437,    0,  120,  437, 
          437,  437,  437,    0,    0,  437,    0,    0,  437,  118, 
            0,  437,    0,  437,    0,  437,  862,  437,    0,  437, 
          437,  437,  437,  437,  437,  437,    0,  437,  437,  437, 
          436,  436,  436,    0,    0,  436,  436,  436,    0,  436, 
            0,  437,    0,    0,    0,    0,    0,  120,  436,  436, 
          436,    0,    0,  437,    0,    0,    0,  119,  436,  436, 
          121,  436,  436,  436,  436,  436,    0,    0,    0,    0, 
          521,    0,    0,  436,  518,    0,    0,  521,  521,  523, 
            0,    0,    0,    0,  518,  436,  436,  436,  436,  436, 
          436,  436,  436,  436,  436,  436,  436,  436,  436,    0, 
            0,  436,  436,  436,  436,    0,  119,  436,    0,  121, 
          436,  103,  437,  436,    0,  436,    0,  436,  518,  436, 
            0,  436,  436,  436,  436,  436,  436,  436,    0,  436, 
          436,  436,  518,  518,    0,  115,    0,  518,  522,    0, 
            0,  524,    0,  436,  519,  519,  519,    0,    0,  519, 
          519,  519,    0,  519,    0,  436,    0,    0,    0,    0, 
          437,    0,    0,  519,  519,  518,    0,  437,  437,    0, 
            0,    0,  519,  519,  103,  519,  519,  519,  519,  519, 
            0,    0,    0,    0,    0,    0,    0,  519,  520,    0, 
            0,    0,    0,    0,  437,    0,    0,    0,  520,  519, 
          519,  519,  519,  519,  519,  519,  519,  519,  519,  519, 
          519,  519,  519,    0,    0,  519,  519,  519,  519,    0, 
          531,  519,    0,    0,  519,    0,    0,  519,    0,  519, 
           98,  519,  520,  519,    0,  519,  519,  519,  519,  519, 
          519,  519,    0,  519,    0,  519,  520,  520,    0,  117, 
            0,  520,    0,    0,    0,  120,    0,  519,    0,    0, 
            0,    0,    0,    0,   98,    0,    0,    0,    0,  519, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  520, 
            0,   98,    0,    0,    0,    0,    0,    0,    0,    0, 
          521,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          521,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  520,    0,    0,  119,    0,  523,  121,    0,    0, 
            0,    0,    0,  523,  523,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  521,    0,  518,  518,  518,    0, 
            0,  518,  518,  518,    0,  518,    0,    0,  521,  521, 
            0,  118,    0,  521,    0,  518,  518,    0,    0,    0, 
            0,    0,    0,    0,  518,  518,    0,  518,  518,  518, 
          518,  518,    0,    0,    0,  522,    0,  112,  524,  518, 
            0,  521,  522,  522,    0,  524,  524,    0,    0,    0, 
            0,  518,  518,  518,  518,  518,  518,  518,  518,  518, 
          518,  518,  518,  518,  518,  111,    0,  518,  518,  518, 
          518,    0,  530,  518,    0,    0,  518,    0,    0,  518, 
            0,  518,    0,  518,    0,  518,  112,  518,  518,  518, 
          518,  518,  518,  518,    0,  518,    0,  518,    0,    0, 
            0,    0,    0,  228,    0,    0,    0,  114,    0,  518, 
          520,  520,  520,  228,  111,  520,  520,  520,  515,  520, 
            0,  518,    0,    0,    0,    0,    0,    0,    0,  520, 
          520,    0,    0,    0,    0,    0,    0,    0,  520,  520, 
            0,  520,  520,  520,  520,  520,  514,  228,    0,  128, 
            0,    0,    0,  520,    0,    0,  114,    0,    0,   98, 
            0,  228,  228,    0,    0,  520,  520,  520,  520,  520, 
          520,  520,  520,  520,  520,  520,  520,  520,  520,  127, 
            0,  520,  520,  520,  520,    0,  532,  520,  517,    0, 
          520,    0,    0,  520,    0,  520,    0,  520,  128,  520, 
            0,  520,  520,  520,  520,  520,  520,  520,  520,  520, 
            0,  520,  521,  521,  521,  520,  520,  521,  521,  521, 
            0,  521,   98,  520,    0,    0,    0,    0,  127,    0, 
          305,  521,  521,    0,    0,  520,    0,    0,    0,    0, 
          521,  521,    0,  521,  521,  521,  521,  521,    0,    0, 
            0,    0,    0,    0,    0,  521,  523,    0,    0,    0, 
          316,    0,    0,    0,    0,    0,  523,  521,  521,  521, 
          521,  521,  521,  521,  521,  521,  521,  521,  521,  521, 
          521,    0,    0,  521,  521,  521,  521,    0,  533,  521, 
            0,    0,  521,    0,  112,  521,    0,  521,   99,  521, 
          523,  521,    0,  521,  521,  521,  521,  521,  521,  521, 
            0,  521,    0,  521,  523,  523,    0,  120,    0,  523, 
            0,    0,  111,    0,    0,  521,    0,    0,    0,    0, 
            0,    0,   99,    0,    0,    0,    0,  521,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  523,    0,   99, 
            0,    0,    0,    0,    0,  515,    0,    0,  522,    0, 
            0,    0,  515,  515,  114,  228,  228,  228,  522,    0, 
          228,  228,  228,    0,  228,    0,    0,    0,    0,  521, 
            0,    0,    0,  514,  228,  228,    0,    0,    0,    0, 
          514,  514,    0,  228,  228,    0,  228,  228,  228,  228, 
          228,    0,  522,    0,    0,    0,  128,    0,  228,    0, 
            0,    0,    0,    0,    0,    0,  522,  522,    0,  119, 
            0,  522,    0,    0,    0,  517,  101,    0,    0,    0, 
            0,    0,  517,  517,    0,    0,  127,    0,    0,    0, 
            0,    0,  228,    0,    0,  228,    0,    0,  228,  522, 
          228,    0,    0,    0,    0,  100,    0,    0,    0,    0, 
          101,    0,    0,    0,    0,    0,  228,  305,    0,    0, 
            0,    0,  524,    0,  305,  305,    0,  101,  228,    0, 
            0,    0,  524,    0,    0,    0,    0,    0,    0,  100, 
          228,    0,    0,    0,    0,    0,    0,  316,    0,    0, 
            0,    0,    0,    0,  316,  316,  100,  523,    0,    0, 
            0,    0,    0,    0,    0,    0,  524,    0,  523,  523, 
          523,    0,    0,  523,  523,  523,    0,  523,    0,    0, 
          524,  524,    0,  121,    0,  524,  522,  523,  523,    0, 
            0,    0,    0,    0,    0,    0,  523,  523,    0,  523, 
          523,  523,  523,  523,    0,    0,    0,    0,    0,    0, 
            0,  523,    0,  524,    0,    0,    0,   99,    0,    0, 
            0,    0,    0,  523,  523,  523,  523,  523,  523,  523, 
          523,  523,  523,  523,  523,  523,  523,    0,    0,  523, 
          523,  523,  523,    0,  535,  523,    0,    0,  523,    0, 
            0,  523,    0,  523,    0,  523,    0,  523,    0,  523, 
          523,  523,  523,  523,  523,  523,  521,  523,    0,  523, 
          522,  522,  522,  521,  521,  522,  522,  522,    0,  522, 
           99,  523,    0,    0,    0,    0,    0,    0,    0,  522, 
          522,    0,    0,  523,    0,    0,    0,    0,  522,  522, 
            0,  522,  522,  522,  522,  522,    0,    0,    0,    0, 
            0,    0,    0,  522,  515,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  515,  522,  522,  522,  522,  522, 
          522,  522,  522,  522,  522,  522,  522,  522,  522,    0, 
            0,  522,  522,  522,  522,  101,  534,  522,    0,    0, 
          522,    0,    0,  522,    0,  522,    0,  522,  515,  522, 
            0,  522,  522,  522,  522,  522,  522,  522,    0,  522, 
            0,  522,  515,  515,  100,  112,    0,  515,    0,    0, 
            0,    0,    0,  522,  524,  524,  524,    0,    0,  524, 
          524,  524,    0,  524,  523,  522,    0,    0,    0,    0, 
            0,  523,  523,  524,  524,  515,    0,    0,  101,    0, 
            0,    0,  524,  524,    0,  524,  524,  524,  524,  524, 
            0,    0,    0,  522,    0,    0,    0,  524,  514,    0, 
          522,  522,    0,    0,    0,    0,    0,  100,  514,  524, 
          524,  524,  524,  524,  524,  524,  524,  524,  524,  524, 
          524,  524,  524,    0,    0,  524,  524,  524,  524,    0, 
          536,  524,    0,    0,  524,    0,    0,  524,    0,  524, 
          102,  524,  514,  524,    0,  524,  524,  524,  524,  524, 
          524,  524,    0,  524,    0,  524,  514,  514,    0,  111, 
            0,  514,    0,    0,    0,    0,    0,  524,    0,    0, 
            0,    0,    0,    0,  102,    0,    0,    0,    0,  524, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  514, 
            0,  102,    0,    0,    0,    0,    0,    0,    0,    0, 
          517,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          517,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  524,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  517,    0,  515,  515,  515,    0, 
            0,  515,  515,  515,    0,  515,    0,    0,  517,  517, 
            0,  114,    0,  517,    0,  515,  515,    0,    0,    0, 
            0,    0,    0,    0,  515,  515,    0,  515,  515,  515, 
          515,  515,    0,    0,    0,    0,    0,    0,    0,  515, 
            0,  517,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  515,  515,  515,  515,  515,  515,  515,  515,  515, 
          515,  515,  515,  515,  515,    0,    0,  515,  515,  515, 
          515,    0,  527,  515,    0,    0,  515,    0,    0,  515, 
            0,  515,    0,  515,    0,  515,    0,  515,  515,  515, 
          515,  515,  515,  515,    0,  515,    0,  515,    0,    0, 
            0,    0,    0,  227,    0,    0,    0,    0,    0,  515, 
          514,  514,  514,  227,    0,  514,  514,  514,    0,  514, 
            0,  515,    0,    0,    0,    0,    0,    0,    0,  514, 
          514,    0,    0,    0,    0,    0,    0,    0,  514,  514, 
            0,  514,  514,  514,  514,  514,    0,  227,    0,    0, 
            0,    0,    0,  514,    0,    0,    0,    0,    0,  102, 
            0,  227,  227,    0,    0,  514,  514,  514,  514,  514, 
          514,  514,  514,  514,  514,  514,  514,  514,  514,    0, 
            0,  514,  514,  514,  514,    0,  526,  514,    0,    0, 
          514,    0,    0,  514,    0,  514,    0,  514,    0,  514, 
            0,  514,  514,  514,  514,  514,  514,  514,  524,  514, 
            0,  514,  517,  517,  517,  524,  524,  517,  517,  517, 
            0,  517,  102,  514,    0,    0,    0,    0,    0,    0, 
            0,  517,  517,    0,    0,  514,    0,    0,    0,    0, 
          517,  517,    0,  517,  517,  517,  517,  517,    0,    0, 
            0,    0,    0,    0,    0,  517,  645,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  645,  517,  517,  517, 
          517,  517,  517,  517,  517,  517,  517,  517,  517,  517, 
          517,    0,    0,  517,  517,  517,  517,    0,  529,  517, 
            0,    0,  517,    0,    0,  517,    0,  517,   93,  517, 
          645,  517,    0,  517,  517,  517,  517,  517,  517,  517, 
            0,  517,    0,  517,  645,  645,    0,    0,    0,  645, 
            0,    0,    0,    0,    0,  517,    0,    0,    0,    0, 
            0,    0,   93,    0,    0,    0,    0,  517,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  645,    0,   93, 
            0,    0,    0,    0,    0,    0,    0,    0,  305,    0, 
            0,    0,    0,    0,    0,  227,  227,  227,  305,    0, 
          227,  227,  227,    0,  227,    0,    0,    0,    0,  515, 
            0,    0,    0,    0,  227,  227,    0,    0,    0,    0, 
            0,    0,    0,  227,  227,    0,  227,  227,  227,  227, 
          227,    0,  305,    0,    0,    0,    0,    0,  227,    0, 
            0,    0,    0,    0,    0,    0,  305,  305,    0,  128, 
            0,  305,    0,    0,    0,    0,   92,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  227,    0,    0,  227,    0,    0,  227,  305, 
          227,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
           92,    0,    0,    0,    0,    0,  227,    0,    0,    0, 
            0,    0,  316,    0,    0,    0,    0,   92,  227,    0, 
            0,    0,  316,    0,    0,    0,    0,    0,    0,    0, 
          227,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  514,    0,    0, 
            0,    0,    0,    0,    0,    0,  316,    0,  645,  645, 
          645,    0,    0,  645,  645,  645,    0,  645,    0,    0, 
          316,  316,    0,  127,    0,  316,  645,  645,  645,    0, 
            0,    0,    0,    0,    0,    0,  645,  645,    0,  645, 
          645,  645,  645,  645,    0,    0,    0,    0,    0,    0, 
            0,  645,    0,  316,    0,    0,    0,   93,    0,    0, 
            0,    0,    0,  645,  645,  645,  645,  645,  645,  645, 
          645,  645,  645,  645,  645,  645,  645,    0,    0,  645, 
          645,  645,  645,    0,    0,  645,    0,    0,  645,    0, 
            0,  645,    0,  645,    0,  645,    0,  645,    0,  645, 
          645,  645,  645,  645,  645,  645,  515,  645,  645,  645, 
          305,  305,  305,  515,  515,  305,  305,  305,    0,  305, 
           93,  645,    0,    0,    0,    0,    0,    0,    0,  305, 
          305,    0,    0,  645,    0,    0,    0,    0,  305,  305, 
            0,  305,  305,  305,  305,  305,    0,    0,    0,    0, 
            0,    0,    0,  305,  457,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  457,  305,  305,  305,  305,  305, 
          305,  305,  305,  305,  305,  305,  305,  305,  305,    0, 
            0,  305,  305,  305,  305,   92,    0,  305,    0,    0, 
          305,    0,    0,  305,    0,  305,    0,  305,  457,  305, 
            0,  305,  305,  305,  305,  305,  305,  305,    0,  305, 
            0,  305,  457,  457,    0,    0,    0,  457,    0,    0, 
            0,    0,    0,  305,  316,  316,  316,    0,    0,  316, 
          316,  316,    0,  316,  514,  305,    0,    0,    0,    0, 
            0,  514,  514,  316,  316,  457,    0,    0,   92,    0, 
            0,    0,  316,  316,    0,  316,  316,  316,  316,  316, 
            0,    0,    0,    0,    0,    0,    0,  316,  327,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  327,  316, 
          316,  316,  316,  316,  316,  316,  316,  316,  316,  316, 
          316,  316,  316,    0,    0,  316,  316,  316,  316,    0, 
            0,  316,    0,    0,  316,    0,    0,  316,    0,  316, 
            0,  316,  327,  316,    0,  316,  316,  316,  316,  316, 
          316,  316,    0,  316,    0,  316,  327,  327,    0,    0, 
            0,  327,    0,    0,    0,    0,    0,  316,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  316, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  327, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          263,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          263,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  263,    0,  457,  457,  457,    0, 
            0,  457,  457,  457,    0,  457,    0,    0,  263,  263, 
            0,    0,    0,  263,    0,  457,  457,    0,    0,    0, 
            0,    0,    0,    0,  457,  457,    0,  457,  457,  457, 
          457,  457,    0,    0,    0,    0,    0,    0,    0,  457, 
            0,  359,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  457,  457,  457,  457,  457,  457,  457,  457,  457, 
          457,  457,  457,  457,  457,    0,    0,  457,  457,  457, 
          457,    0,    0,  457,    0,    0,  457,    0,    0,  457, 
            0,  457,    0,  457,    0,  457,    0,  457,  457,  457, 
          457,  457,  457,  457,    0,  457,    0,  457,    0,    0, 
            0,    0,    0,  225,    0,    0,    0,    0,    0,  457, 
          327,  327,  327,  225,    0,  327,  327,  327,    0,  327, 
            0,  457,    0,    0,    0,    0,    0,    0,    0,  327, 
          327,    0,    0,    0,    0,    0,    0,    0,  327,  327, 
            0,  327,  327,  327,  327,  327,    0,  225,    0,    0, 
            0,    0,    0,  327,    0,    0,    0,  106,    0,    0, 
            0,  225,  225,    0,    0,  327,  327,  327,  327,  327, 
          327,  327,  327,  327,  327,  327,  327,  327,  327,    0, 
            0,  327,  327,  327,  327,    0,    0,  327,    0,    0, 
          327,  106,    0,  327,    0,  327,    0,  327,    0,  327, 
           25,  327,  327,  327,  327,  327,  327,  327,  106,  327, 
           25,  327,  263,  263,  263,    0,    0,  263,  263,  263, 
            0,  263,    0,  327,    0,    0,    0,    0,    0,    0, 
            0,  263,  263,    0,    0,  327,    0,    0,  620,    0, 
          263,  263,    0,  263,  263,  263,  263,  263,    0,    0, 
            0,    0,    0,    0,    0,  263,  322,    0,    0,   25, 
            0,    0,    0,    0,    0,    0,  322,  263,  263,  263, 
          263,  263,  263,  263,  263,  263,  263,  263,  359,  263, 
          263,    0,    0,  263,  263,  359,  359,    0,    0,  263, 
            0,    0,  263,    0,    0,  263,    0,  263,   95,  263, 
          322,  263,    0,  263,  263,  263,  263,  263,  263,  263, 
            0,  263,    0,  263,  322,  322,    0,    0,    0,  322, 
            0,    0,    0,    0,    0,  263,    0,    0,    0,    0, 
            0,    0,   95,    0,    0,    0,    0,  263,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  322,    0,   95, 
            0,    0,    0,    0,    0,    0,    0,    0,  508,    0, 
            0,    0,    0,    0,    0,  225,  225,  225,  508,    0, 
          225,  225,  225,    0,  225,    0,    0,    0,    0,  517, 
            0,    0,    0,    0,  225,  225,    0,    0,    0,    0, 
            0,    0,    0,  225,  225,    0,  225,  225,  225,  225, 
          225,    0,  508,    0,    0,    0,    0,    0,  225,    0, 
            0,    0,    0,    0,    0,    0,  508,  508,    0,    0, 
            0,  508,    0,    0,    0,  516,  106,  620,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  225,    0,    0,  225,    0,    0,  225,  508, 
          225,    0,    0,    0,    0,    0,    0,    0,    0,  516, 
            0,    0,   25,   25,   25,    0,  225,  355,   25,   25, 
            0,   25,    0,    0,  516,  620,    0,  355,  225,    0, 
            0,    0,  620,  620,    0,    0,    0,    0,  620,  106, 
          225,    0,    0,   25,   25,   25,   25,   25,    0,    0, 
            0,    0,    0,    0,    0,    0,  516,    0,    0,  620, 
            0,  355,    0,    0,    0,    0,    0,    0,  322,  322, 
          322,    0,    0,  322,  322,  322,  355,  322,    0,    0, 
          355,    0,    0,    0,    0,    0,    0,  322,  322,    0, 
            0,    0,   25,    0,    0,    0,  322,  322,    0,  322, 
          322,  322,  322,  322,    0,    0,    0,    0,  355,    0, 
            0,  322,    0,   25,    0,    0,    0,   95,    0,    0, 
            0,    0,    0,  322,  322,  322,  322,  322,  322,  322, 
          322,  322,  322,  322,  322,  322,  322,   25,    0,  322, 
          322,  322,  322,    0,    0,  322,    0,    0,  322,    0, 
            0,  322,    0,  322,    0,  322,    0,  322,   51,  322, 
          322,  322,  322,  322,  322,  322,  517,  322,   51,  322, 
          508,  508,  508,  517,  517,  508,  508,  508,    0,  508, 
           95,  322,    0,    0,    0,    0,    0,    0,    0,  508, 
          508,    0,    0,  322,    0,    0,    0,    0,  508,  508, 
            0,  508,  508,  508,  508,  508,    0,    0,    0,    0, 
            0,    0,    0,  508,  356,    0,    0,   51,    0,    0, 
            0,    0,    0,    0,  356,    0,  508,  508,  508,  508, 
          508,  508,  508,  508,  508,  508,  508,  508,  508,    0, 
            0,  508,  508,  508,  508,  617,    0,  508,    0,    0, 
          508,    0,    0,  508,    0,  508,    0,  508,  356,  508, 
            0,  508,  508,  508,  508,  508,  508,  508,    0,  508, 
            0,  508,    0,  356,    0,    0,    0,  356,    0,  355, 
          355,  355,    0,  508,  355,  355,  355,    0,  355,    0, 
            0,    0,    0,  516,    0,  508,    0,    0,  355,  355, 
          516,  516,    0,    0,    0,  356,  617,  355,  355,    0, 
          355,  355,  355,  355,  355,    0,    0,    0,    0,    0, 
            0,    0,    0,  319,  516,    0,    0,  617,    0,    0, 
            0,    0,    0,  319,  355,  355,  355,  355,  355,  355, 
          355,  355,  355,  355,  355,  355,  355,  355,    0,    0, 
          355,  355,  355,  355,    0,    0,  355,    0,    0,  355, 
            0,    0,  355,    0,  355,    0,  355,  319,  355,    0, 
          355,  355,  355,  355,  355,  355,  355,    0,  355,    0, 
          355,    0,  319,    0,    0,    0,  319,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  355,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  319,    0,    0,    0,    0,    0, 
           51,   51,   51,    0,    0,   51,   51,   51,    0,   51, 
          230,    0,    0,    0,    0,    0,    0,    0,    0,   51, 
          230,    0,    0,    0,    0,    0,    0,    0,   51,   51, 
            0,   51,   51,   51,   51,   51,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  230,    0,  356,  356,  356,    0, 
            0,  356,  356,  356,    0,  356,    0,    0,  230,  230, 
            0,    0,    0,  230,    0,  356,  356,    0,    0,    0, 
           51,    0,    0,    0,  356,  356,    0,  356,  356,  356, 
          356,  356,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,   51,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  356,  356,  356,  356,  356,  356,  356,  356,  356, 
          356,  356,  356,  356,  356,   51,    0,  356,  356,  356, 
          356,    0,  232,  356,    0,    0,  356,    0,    0,  356, 
            0,  356,  232,  356,    0,  356,    0,  356,  356,  356, 
          356,  356,  356,  356,    0,  356,    0,  356,    0,    0, 
            0,    0,    0,    0,    0,  319,  319,  319,    0,    0, 
          319,  319,  319,    0,  319,    0,  232,    0,    0,    0, 
            0,  356,    0,    0,  319,  319,    0,    0,    0,    0, 
          232,  232,    0,  319,  319,  232,  319,  319,  319,  319, 
          319,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          319,  319,  319,  319,  319,  319,  319,  319,  319,  319, 
          319,  319,  319,  319,    0,    0,  319,  319,  319,  319, 
            0,    0,  319,    0,    0,  319,    0,    0,  319,    0, 
          319,    0,  319,  241,  319,    0,  319,  319,  319,  319, 
          319,  319,  319,  241,  319,    0,  319,    0,    0,    0, 
            0,    0,  230,  230,  230,    0,    0,  230,  230,  230, 
            0,  230,    0,    0,    0,    0,    0,    0,    0,    0, 
          319,  230,  230,    0,    0,    0,    0,  241,    0,    0, 
          230,  230,    0,  230,  230,  230,  230,  230,    0,    0, 
            0,  241,  241,    0,    0,  230,  241,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  230,  230,  230, 
          230,  230,  230,  230,  230,  230,  230,  230,    0,  230, 
          230,    0,    0,  230,  230,    0,    0,    0,    0,  230, 
            0,    0,  230,    0,    0,  230,    0,  230,    0,  230, 
            0,  230,    0,  230,  230,  230,  230,  230,  230,  230, 
            0,  230,    0,  230,    0,    0,    0,    0,    0,    0, 
          238,    0,    0,    0,    0,  230,    0,    0,    0,    0, 
          238,    0,    0,    0,  232,  232,  232,  230,    0,  232, 
          232,  232,    0,  232,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  232,  232,    0,    0,    0,    0,    0, 
            0,    0,  232,  232,  238,  232,  232,  232,  232,  232, 
            0,    0,    0,    0,    0,    0,    0,  232,  238,  238, 
            0,    0,    0,  238,    0,    0,    0,    0,    0,  232, 
          232,  232,  232,  232,  232,  232,  232,  232,  232,  232, 
            0,  232,  232,    0,    0,  232,  232,    0,    0,    0, 
            0,  232,    0,    0,  232,    0,    0,  232,    0,  232, 
            0,  232,  235,  232,    0,  232,  232,  232,  232,  232, 
          232,  232,  235,  232,    0,  232,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  232,    0,    0, 
            0,    0,    0,    0,    0,  241,  241,  241,    0,  232, 
          241,  241,  241,    0,  241,    0,  235,    0,    0,    0, 
            0,    0,    0,    0,  241,  241,    0,    0,    0,    0, 
          235,  235,    0,  241,  241,  235,  241,  241,  241,  241, 
          241,    0,    0,    0,  109,    0,    0,    0,  241,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  241,  241,  241,  241,  241,  241,  241,  241,  241, 
          241,    0,  241,  241,    0,    0,  241,  241,  109,    0, 
            0,    0,  241,    0,    0,  241,    0,    0,  241,    0, 
          241,    0,  241,  623,  241,  109,  241,  241,  241,  241, 
          241,  241,  241,    0,  241,    0,  241,    0,    0,    0, 
          237,    0,    0,    0,    0,    0,    0,    0,  241,    0, 
          237,    0,    0,    0,    0,  305,    0,  623,    0,    0, 
          241,    0,  238,  238,  238,    0,    0,  238,  238,  238, 
            0,  238,  623,    0,    0,    0,    0,    0,    0,    0, 
            0,  238,  238,    0,  237,    0,    0,    0,    0,    0, 
          238,  238,    0,  238,  238,  238,  238,  238,  237,  237, 
            0,    0,    0,  237,  623,  238,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  238,  238, 
          238,  238,  238,  238,  238,  238,  238,  238,    0,  238, 
          238,    0,    0,  238,  238,    0,    0,    0,    0,  238, 
            0,    0,  238,    0,    0,  238,    0,  238,    0,  238, 
          236,  238,    0,  238,  238,  238,  238,  238,  238,  238, 
          236,  238,    0,  238,  235,  235,  235,    0,    0,  235, 
          235,  235,    0,  235,    0,  238,    0,    0,    0,    0, 
            0,    0,    0,  235,  235,    0,    0,  238,    0,    0, 
            0,    0,  235,  235,  236,  235,  235,  235,  235,  235, 
            0,    0,    0,    0,    0,    0,    0,  235,  236,  236, 
            0,    0,    0,  236,    0,    0,    0,    0,    0,    0, 
          235,  235,  235,  235,  235,  235,  235,  235,  235,  235, 
            0,  235,  235,    0,    0,  235,  235,    0,    0,    0, 
            0,  235,    0,  109,  235,    0,    0,  235,    0,  235, 
            0,  235,    0,  235,    0,  235,  235,  235,  235,  235, 
          235,  235,    0,  235,    0,  235,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  235,    0,    0, 
          239,    0,    0,    0,    0,    0,    0,    0,    0,  235, 
          239,    0,  305,  623,    0,    0,    0,    0,    0,  305, 
          305,    0,  237,  237,  237,    0,  109,  237,  237,  237, 
            0,  237,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  237,  237,    0,  239,    0,    0,    0,    0,    0, 
          237,  237,    0,  237,  237,  237,  237,  237,  239,  239, 
            0,  623,    0,  239,    0,  237,    0,    0,  623,  623, 
            0,    0,    0,    0,  619,    0,    0,    0,  237,  237, 
          237,  237,  237,  237,  237,  237,  237,  237,    0,  237, 
          237,    0,  623,  237,  237,  623,    0,    0,    0,  237, 
            0,    0,  237,    0,    0,  237,    0,  237,    0,  237, 
          437,  237,    0,  237,  237,  237,  237,  237,  237,  237, 
          437,  237,    0,  237,    0,    0,    0,    0,    0,    0, 
            0,    0,  236,  236,  236,  237,    0,  236,  236,  236, 
            0,  236,    0,    0,    0,    0,    0,  237,    0,    0, 
            0,  236,  236,    0,  103,    0,    0,    0,    0,    0, 
          236,  236,    0,  236,  236,  236,  236,  236,    0,  437, 
            0,  122,    0,  437,    0,  236,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  236,  236, 
          236,  236,  236,  236,  236,  236,  236,  236,    0,  236, 
          236,  437,    0,  236,  236,    0,    0,    0,    0,  236, 
            0,    0,  236,    0,    0,  236,    0,  236,    0,  236, 
            0,  236,    0,  236,  236,  236,  236,  236,  236,  236, 
            0,  236,    0,  236,    0,    0,    0,    0,    0,    0, 
          519,    0,    0,    0,    0,  236,    0,    0,    0,    0, 
          519,    0,    0,    0,    0,    0,    0,  236,    0,    0, 
            0,    0,  239,  239,  239,    0,    0,  239,  239,  239, 
            0,  239,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  239,  239,    0,   97,    0,    0,    0,    0,    0, 
          239,  239,    0,  239,  239,  239,  239,  239,    0,  519, 
            0,  116,    0,  519,    0,  239,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  239,  239, 
          239,  239,  239,  239,  239,  239,  239,  239,    0,  239, 
          239,  519,    0,  239,  239,    0,    0,    0,    0,  239, 
            0,    0,  239,    0,    0,  239,    0,  239,    0,  239, 
          518,  239,    0,  239,  239,  239,  239,  239,  239,  239, 
          518,  239,    0,  239,    0,    0,    0,    0,    0,    0, 
            0,    0,  437,  437,  437,  239,    0,    0,  437,  437, 
            0,  437,    0,    0,    0,    0,    0,  239,    0,    0, 
          437,    0,    0,    0,   96,    0,    0,    0,    0,    0, 
          437,  437,    0,  437,  437,  437,  437,  437,    0,  518, 
            0,  115,    0,  518,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  437,  437,  437, 
          437,  437,  437,  437,  437,  437,  437,  437,  437,  437, 
          437,  518,    0,  437,  437,  437,  437,    0,    0,    0, 
            0,    0,  437,    0,    0,    0,    0,    0,    0,  437, 
            0,  437,    0,  437,  437,  437,  437,  437,  437,  437, 
            0,  437,  437,  437,    0,    0,    0,    0,    0,    0, 
          520,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          520,    0,    0,    0,    0,    0,    0,  437,    0,    0, 
            0,    0,  519,  519,  519,    0,    0,    0,  519,  519, 
            0,  519,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,   98,    0,    0,    0,    0,    0, 
          519,  519,    0,  519,  519,  519,  519,  519,    0,  520, 
            0,  117,    0,  520,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  519,  519,  519, 
          519,  519,  519,  519,  519,  519,  519,  519,  519,  519, 
          519,  520,    0,  519,  519,  519,  519,    0,  531,    0, 
            0,    0,  519,    0,    0,    0,    0,    0,    0,  519, 
          521,  519,    0,  519,  519,  519,  519,  519,  519,  519, 
          521,  519,    0,  519,    0,    0,    0,    0,    0,    0, 
            0,    0,  518,  518,  518,    0,    0,    0,  518,  518, 
            0,  518,    0,    0,    0,    0,    0,  519,    0,    0, 
            0,    0,    0,    0,   99,    0,    0,    0,    0,    0, 
          518,  518,    0,  518,  518,  518,  518,  518,    0,  521, 
            0,  118,    0,  521,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  518,  518,  518, 
          518,  518,  518,  518,  518,  518,  518,  518,  518,  518, 
          518,  521,    0,  518,  518,  518,  518,    0,  530,    0, 
            0,    0,  518,    0,    0,    0,    0,    0,    0,  518, 
            0,  518,  523,  518,  518,  518,  518,  518,  518,  518, 
            0,  518,  523,  518,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  518,    0,    0, 
            0,    0,  520,  520,  520,    0,  101,    0,  520,  520, 
            0,  520,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  523,    0,  120,    0,  523,    0,    0,    0,    0, 
          520,  520,    0,  520,  520,  520,  520,  520,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  523,    0,    0,    0,  520,  520,  520, 
          520,  520,  520,  520,  520,  520,  520,  520,  520,  520, 
          520,  522,    0,  520,  520,  520,  520,    0,  532,    0, 
            0,  522,  520,    0,    0,    0,    0,    0,    0,  520, 
            0,  520,    0,  520,  520,  520,  520,  520,  520,  520, 
            0,  520,    0,  520,    0,    0,    0,    0,    0,    0, 
            0,    0,  521,  521,  521,  100,    0,    0,  521,  521, 
            0,  521,    0,    0,    0,    0,    0,  520,    0,    0, 
          522,    0,  119,    0,  522,    0,    0,    0,    0,    0, 
          521,  521,    0,  521,  521,  521,  521,  521,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  522,    0,    0,    0,    0,  521,  521,  521, 
          521,  521,  521,  521,  521,  521,  521,  521,  521,  521, 
          521,    0,    0,  521,  521,  521,  521,    0,  533,    0, 
            0,    0,  521,    0,    0,    0,    0,    0,    0,  521, 
            0,  521,  524,  521,  521,  521,  521,  521,  521,  521, 
            0,  521,  524,  521,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  523,  523,  523,    0,    0,    0, 
          523,  523,    0,  523,    0,    0,    0,  521,    0,    0, 
            0,    0,    0,    0,    0,    0,  102,    0,    0,    0, 
            0,    0,  523,  523,    0,  523,  523,  523,  523,  523, 
            0,  524,    0,  121,    0,  524,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  523, 
          523,  523,  523,  523,  523,  523,  523,  523,  523,  523, 
          523,  523,  523,  524,    0,  523,  523,  523,  523,    0, 
          535,    0,    0,    0,  523,    0,    0,    0,    0,    0, 
            0,  523,  515,  523,    0,  523,  523,  523,  523,  523, 
          523,  523,  515,  523,    0,  523,    0,    0,    0,    0, 
            0,    0,    0,  522,  522,  522,    0,    0,    0,  522, 
          522,    0,  522,    0,    0,    0,    0,    0,    0,  523, 
            0,    0,    0,    0,    0,    0,   93,    0,    0,    0, 
            0,  522,  522,    0,  522,  522,  522,  522,  522,    0, 
            0,  515,    0,  112,    0,  515,    0,    0,    0,    0, 
          108,    0,    0,    0,    0,    0,    0,    0,  522,  522, 
          522,  522,  522,  522,  522,  522,  522,  522,  522,  522, 
      };
   }

   private static final short[] yyTable3() {
      return new short[] {

          522,  522,    0,  515,  522,  522,  522,  522,    0,  534, 
            0,    0,    0,  522,  108,    0,    0,    0,    0,    0, 
          522,    0,  522,  514,  522,  522,  522,  522,  522,  522, 
          522,  108,  522,  514,  522,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  522,    0, 
            0,  316,    0,    0,  524,  524,  524,   92,    0,    0, 
          524,  524,    0,  524,    0,    0,    0,    0,    0,    0, 
            0,    0,  514,    0,  111,    0,  514,    0,    0,    0, 
            0,    0,  524,  524,    0,  524,  524,  524,  524,  524, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  514,    0,    0,    0,    0,  524, 
          524,  524,  524,  524,  524,  524,  524,  524,  524,  524, 
          524,  524,  524,  517,    0,  524,  524,  524,  524,    0, 
          536,    0,    0,  517,  524,    0,    0,    0,    0,    0, 
            0,  524,    0,  524,    0,  524,  524,  524,  524,  524, 
          524,  524,    0,  524,    0,  524,    0,    0,    0,    0, 
            0,    0,    0,    0,  515,  515,  515,   95,    0,    0, 
          515,  515,    0,  515,    0,    0,    0,    0,    0,  524, 
            0,    0,  517,    0,  114,    0,  517,    0,    0,    0, 
            0,    0,  515,  515,    0,  515,  515,  515,  515,  515, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  517,    0,    0,    0,    0,  515, 
          515,  515,  515,  515,  515,  515,  515,  515,  515,  515, 
          515,  515,  515,    0,    0,  515,  515,  515,  515,  108, 
          527,    0,    0,    0,  515,    0,    0,    0,    0,    0, 
            0,  515,    0,  515,  305,  515,  515,  515,  515,  515, 
          515,  515,    0,  515,  305,  515,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  514,  514,  514,    0,    0, 
            0,  514,  514,    0,  514,    0,    0,    0,  316,  515, 
            0,    0,    0,    0,    0,  316,  316,    0,  109,    0, 
            0,    0,  108,  514,  514,    0,  514,  514,  514,  514, 
          514,    0,    0,  305,    0,  128,    0,  305,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          514,  514,  514,  514,  514,  514,  514,  514,  514,  514, 
          514,  514,  514,  514,    0,  305,  514,  514,  514,  514, 
            0,  526,    0,    0,    0,  514,    0,    0,    0,    0, 
            0,    0,  514,  316,  514,    0,  514,  514,  514,  514, 
          514,  514,  514,  316,  514,    0,  514,    0,    0,    0, 
            0,    0,    0,    0,    0,  517,  517,  517,    0,    0, 
            0,  517,  517,    0,  517,    0,    0,    0,    0,    0, 
          514,    0,    0,    0,    0,    0,    0,  108,    0,    0, 
            0,    0,    0,  517,  517,    0,  517,  517,  517,  517, 
          517,    0,  316,    0,  127,    0,  316,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          517,  517,  517,  517,  517,  517,  517,  517,  517,  517, 
          517,  517,  517,  517,  316,    0,  517,  517,  517,  517, 
            0,  529,    0,    0,    0,  517,    0,    0,    0,    0, 
            0,    0,  517,    0,  517,  233,  517,  517,  517,  517, 
          517,  517,  517,    0,  517,  233,  517,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          517,    0,    0,    0,    0,    0,  305,  305,  305,  233, 
            0,    0,  305,  305,    0,  305,    0,    0,    0,    0, 
            0,    0,    0,  233,  233,    0,    0,    0,  233,    0, 
            0,    0,    0,    0,  305,  305,    0,  305,  305,  305, 
          305,  305,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  305,  305,  305,  305,  305,  305,  305,  305,  305, 
          305,  305,  305,  305,  305,  234,    0,  305,  305,  305, 
          305,    0,    0,    0,    0,  234,  305,    0,    0,    0, 
            0,    0,    0,  305,    0,  305,    0,  305,  305,  305, 
          305,  305,  305,  305,    0,  305,    0,  305,    0,    0, 
            0,    0,    0,    0,    0,  316,  316,  316,    0,  234, 
            0,  316,  316,    0,  316,    0,    0,    0,    0,    0, 
            0,  305,    0,  234,  234,    0,    0,    0,  234,    0, 
            0,    0,    0,  316,  316,    0,  316,  316,  316,  316, 
          316,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          316,  316,  316,  316,  316,  316,  316,  316,  316,  316, 
          316,  316,  316,  316,    0,    0,  316,  316,  316,  316, 
            0,    0,    0,    0,    0,  316,    0,    0,    0,    0, 
            0,    0,  316,    0,  316,  257,  316,  316,  316,  316, 
          316,  316,  316,    0,  316,  257,  316,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  233,  233,  233, 
            0,    0,  233,  233,  233,    0,  233,    0,    0,    0, 
          316,    0,    0,    0,    0,    0,  233,  233,    0,  257, 
            0,    0,    0,    0,    0,  233,  233,    0,  233,  233, 
          233,  233,  233,  257,  257,    0,    0,    0,  257,    0, 
          233,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  233,  233,  233,  233,  233,  233,  233, 
          233,  233,  233,    0,  233,  233,    0,    0,  233,  233, 
            0,    0,    0,    0,  233,    0,    0,  233,    0,    0, 
          233,    0,  233,    0,    0,  258,  233,    0,    0,    0, 
          233,  233,  233,  233,  233,  258,  233,    0,  233,    0, 
            0,    0,    0,    0,    0,    0,    0,  234,  234,  234, 
          233,    0,  234,  234,  234,    0,  234,    0,    0,    0, 
            0,    0,  233,    0,    0,    0,  234,  234,    0,  258, 
            0,    0,    0,    0,    0,  234,  234,    0,  234,  234, 
          234,  234,  234,  258,  258,    0,    0,    0,  258,    0, 
          234,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  234,  234,  234,  234,  234,  234,  234, 
          234,  234,  234,    0,  234,  234,    0,    0,  234,  234, 
            0,    0,    0,    0,  234,    0,    0,  234,    0,    0, 
          234,    0,  234,    0,    0,    0,  234,    0,    0,    0, 
          234,  234,  234,  234,  234,    0,  234,    0,  234,    0, 
            0,    0,    0,    0,    0,  244,    0,    0,    0,    0, 
          234,    0,    0,    0,    0,  244,    0,    0,    0,    0, 
            0,    0,  234,    0,    0,    0,    0,  257,  257,  257, 
            0,    0,  257,  257,  257,    0,  257,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  257,  257,    0,  244, 
            0,    0,    0,    0,    0,  257,  257,    0,  257,  257, 
          257,  257,  257,  244,  244,    0,    0,    0,  244,    0, 
          257,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  257,  257,  257,  257,  257,  257,  257, 
          257,  257,  257,    0,  257,  257,    0,    0,  257,  257, 
            0,    0,    0,    0,  257,    0,    0,  257,    0,    0, 
          257,    0,  257,    0,    0,  242,  257,    0,    0,    0, 
            0,    0,  257,  257,  257,  242,  257,    0,  257,    0, 
            0,    0,    0,    0,    0,    0,    0,  258,  258,  258, 
          257,    0,  258,  258,  258,    0,  258,    0,    0,    0, 
            0,    0,  257,    0,    0,    0,  258,  258,    0,  242, 
            0,    0,    0,    0,    0,  258,  258,    0,  258,  258, 
          258,  258,  258,  242,  242,    0,    0,    0,  242,    0, 
          258,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  258,  258,  258,  258,  258,  258,  258, 
          258,  258,  258,    0,  258,  258,    0,    0,  258,  258, 
            0,    0,    0,    0,  258,    0,    0,  258,    0,    0, 
          258,    0,  258,    0,    0,    0,  258,    0,    0,    0, 
            0,    0,  258,  258,  258,    0,  258,    0,  258,    0, 
            0,    0,    0,    0,    0,  243,    0,    0,    0,    0, 
          258,    0,    0,    0,    0,  243,    0,    0,    0,    0, 
            0,    0,  258,    0,    0,    0,    0,  244,  244,  244, 
            0,    0,  244,  244,  244,    0,  244,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  244,  244,    0,  243, 
            0,    0,    0,    0,    0,  244,  244,    0,  244,  244, 
          244,  244,  244,  243,  243,    0,    0,    0,  243,    0, 
          244,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  244,  244,  244,  244,  244,  244,  244, 
          244,  244,  244,    0,  244,  244,    0,    0,    0,    0, 
            0,    0,    0,    0,  244,    0,    0,  244,    0,    0, 
          244,    0,  244,    0,    0,  247,  244,    0,    0,    0, 
            0,    0,  244,  244,  244,  247,  244,    0,  244,    0, 
            0,    0,    0,    0,    0,    0,    0,  242,  242,  242, 
          244,    0,  242,  242,  242,    0,  242,    0,    0,    0, 
            0,    0,  244,    0,    0,    0,  242,  242,    0,  247, 
            0,    0,    0,    0,    0,  242,  242,    0,  242,  242, 
          242,  242,  242,  247,  247,    0,    0,    0,  247,    0, 
          242,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  242,  242,  242,  242,  242,  242,  242, 
          242,  242,  242,    0,  242,  242,    0,    0,    0,    0, 
            0,    0,    0,    0,  242,    0,    0,  242,    0,    0, 
          242,    0,  242,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  242,  242,  242,    0,  242,    0,  242,    0, 
            0,    0,    0,    0,    0,  249,    0,    0,    0,    0, 
          242,    0,    0,    0,    0,  249,    0,    0,    0,    0, 
            0,    0,  242,    0,    0,    0,    0,  243,  243,  243, 
            0,    0,  243,  243,  243,    0,  243,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  243,  243,    0,  249, 
            0,    0,    0,    0,    0,  243,  243,    0,  243,  243, 
          243,  243,  243,  249,  249,    0,    0,    0,  249,    0, 
          243,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  243,  243,  243,  243,  243,  243,  243, 
          243,  243,  243,    0,  243,  243,    0,    0,    0,    0, 
            0,    0,    0,    0,  243,    0,    0,  243,    0,    0, 
          243,    0,  243,    0,    0,  248,    0,    0,    0,    0, 
            0,    0,  243,  243,  243,  248,  243,    0,  243,    0, 
            0,    0,    0,    0,    0,    0,    0,  247,  247,  247, 
          243,    0,  247,  247,  247,    0,  247,    0,    0,    0, 
            0,    0,  243,    0,    0,    0,  247,  247,    0,  248, 
            0,    0,    0,    0,    0,  247,  247,    0,  247,  247, 
          247,  247,  247,  248,  248,    0,    0,    0,  248,    0, 
          247,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  247,  247,  247,  247,  247,  247,  247, 
          247,  247,  247,    0,  247,  247,    0,    0,    0,    0, 
            0,    0,    0,    0,  247,    0,    0,  247,    0,    0, 
          247,    0,  247,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  247,  247,    0,    0,    0,    0,  247,    0, 
            0,    0,    0,    0,    0,  246,    0,    0,    0,    0, 
          247,    0,    0,    0,    0,  246,    0,    0,    0,    0, 
            0,    0,  247,    0,    0,    0,    0,  249,  249,  249, 
            0,    0,  249,  249,  249,    0,  249,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  249,  249,    0,  246, 
            0,    0,    0,    0,    0,  249,  249,    0,  249,  249, 
          249,  249,  249,  246,  246,    0,    0,    0,  246,    0, 
          249,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  249,  249,  249,  249,  249,  249,  249, 
          249,  249,  249,    0,  249,  249,    0,    0,    0,    0, 
            0,    0,    0,    0,  249,    0,    0,  249,    0,    0, 
          249,    0,  249,    0,    0,  250,    0,    0,    0,    0, 
            0,    0,  249,  249,    0,  250,    0,    0,  249,    0, 
            0,    0,    0,    0,    0,    0,    0,  248,  248,  248, 
          249,    0,  248,  248,  248,    0,  248,    0,    0,    0, 
            0,    0,  249,    0,    0,    0,  248,  248,    0,  250, 
            0,    0,    0,    0,    0,  248,  248,    0,  248,  248, 
          248,  248,  248,  250,  250,    0,    0,    0,  250,    0, 
          248,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  248,  248,  248,  248,  248,  248,  248, 
          248,  248,  248,    0,  248,  248,    0,    0,    0,    0, 
            0,    0,    0,    0,  248,    0,    0,  248,    0,    0, 
          248,    0,  248,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  248,  248,    0,    0,    0,    0,  248,    0, 
            0,    0,    0,    0,    0,  251,    0,    0,    0,    0, 
          248,    0,    0,    0,    0,  251,    0,    0,    0,    0, 
            0,    0,  248,    0,    0,    0,    0,  246,  246,  246, 
            0,    0,  246,  246,  246,    0,  246,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  246,  246,    0,  251, 
            0,    0,    0,    0,    0,  246,  246,    0,  246,  246, 
          246,  246,  246,  251,  251,    0,    0,    0,  251,    0, 
          246,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  246,  246,  246,  246,  246,  246,  246, 
          246,  246,  246,    0,  246,  246,    0,    0,    0,    0, 
            0,    0,    0,    0,  246,    0,    0,  246,    0,    0, 
          246,    0,  246,    0,    0,  252,    0,    0,    0,    0, 
            0,    0,  246,  246,    0,  252,    0,    0,  246,    0, 
            0,    0,    0,    0,    0,    0,    0,  250,  250,  250, 
          246,    0,  250,  250,  250,    0,  250,    0,    0,    0, 
            0,    0,  246,    0,    0,    0,  250,  250,    0,  252, 
            0,    0,    0,    0,    0,  250,  250,    0,  250,  250, 
          250,  250,  250,  252,  252,    0,    0,    0,  252,    0, 
          250,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  250, 
          250,  259,    0,    0,  250,  250,    0,    0,    0,    0, 
            0,  259,    0,    0,  250,    0,    0,  250,    0,    0, 
          250,    0,  250,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  250,    0, 
            0,    0,    0,    0,    0,  259,    0,    0,    0,    0, 
          250,    0,    0,    0,    0,    0,    0,    0,    0,  259, 
          259,    0,  250,    0,  259,    0,    0,  251,  251,  251, 
            0,    0,  251,  251,  251,    0,  251,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  251,  251,    0,    0, 
            0,    0,    0,    0,    0,  251,  251,    0,  251,  251, 
          251,  251,  251,    0,  253,    0,    0,    0,    0,    0, 
          251,    0,    0,    0,  253,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  251, 
          251,    0,    0,    0,  251,  251,    0,    0,    0,    0, 
            0,    0,    0,    0,  251,    0,    0,  251,  253,    0, 
          251,    0,  251,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  253,  253,    0,    0,    0,  253,  251,    0, 
            0,    0,    0,    0,    0,    0,    0,  252,  252,  252, 
          251,    0,  252,  252,  252,    0,  252,    0,    0,    0, 
          254,    0,  251,    0,    0,    0,  252,  252,    0,    0, 
          254,    0,    0,    0,    0,  252,  252,    0,  252,  252, 
          252,  252,  252,    0,    0,    0,    0,    0,    0,    0, 
          252,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  254,    0,    0,    0,    0,  252, 
          252,    0,    0,    0,  252,  252,    0,    0,  254,  254, 
            0,    0,    0,  254,  252,    0,    0,  252,    0,    0, 
          252,    0,  252,  259,  259,  259,    0,    0,  259,  259, 
          259,    0,  259,    0,    0,    0,  260,    0,  252,    0, 
            0,    0,  259,  259,    0,    0,  260,    0,    0,    0, 
          252,  259,  259,    0,  259,  259,  259,  259,  259,    0, 
            0,    0,  252,    0,    0,    0,  259,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          260,    0,    0,    0,    0,  259,  259,    0,    0,    0, 
          259,  259,    0,    0,  260,  260,    0,    0,    0,  260, 
          259,    0,    0,  259,    0,    0,  259,    0,  259,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  259,    0,  253,  253,  253,    0, 
            0,  253,  253,  253,    0,  253,  259,    0,    0,  231, 
            0,    0,    0,    0,    0,  253,  253,    0,  259,  231, 
            0,    0,    0,    0,  253,  253,    0,  253,  253,  253, 
          253,  253,    0,    0,    0,    0,    0,    0,    0,  253, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  231,    0,    0,    0,    0,  253,  253, 
            0,    0,    0,  253,  253,    0,    0,  231,  231,    0, 
            0,    0,  231,  253,    0,    0,  253,    0,    0,  253, 
            0,  253,  254,  254,  254,    0,    0,  254,  254,  254, 
            0,  254,    0,    0,    0,  226,    0,  253,    0,    0, 
            0,  254,  254,    0,    0,  226,    0,    0,    0,  253, 
          254,  254,    0,  254,  254,  254,  254,  254,    0,    0, 
            0,  253,    0,    0,    0,  254,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  226, 
            0,    0,    0,    0,  254,  254,    0,    0,    0,  254, 
          254,    0,    0,  226,  226,    0,    0,    0,    0,  254, 
            0,    0,  254,    0,    0,  254,    0,  254,  260,  260, 
          260,    0,    0,  260,  260,  260,    0,  260,    0,    0, 
            0,  223,    0,  254,    0,    0,    0,  260,  260,    0, 
            0,  223,    0,    0,    0,  254,  260,  260,    0,  260, 
          260,  260,  260,  260,    0,    0,    0,  254,    0,    0, 
            0,  260,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  223,    0,    0,    0,    0, 
            0,  260,    0,    0,    0,  260,  260,    0,    0,  223, 
          223,    0,    0,    0,    0,  260,    0,    0,  260,    0, 
            0,  260,    0,  260,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  260, 
            0,  231,  231,  231,    0,    0,  231,  231,  231,  224, 
          231,  260,    0,    0,    0,    0,    0,    0,    0,  224, 
          231,  231,    0,  260,    0,    0,    0,    0,    0,  231, 
          231,    0,  231,  231,  231,  231,  231,    0,    0,    0, 
            0,    0,    0,    0,  231,    0,    0,    0,    0,    0, 
            0,    0,    0,  224,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  224,  224,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  231,    0, 
            0,  231,    0,    0,  231,    0,  231,  226,  226,  226, 
            0,    0,  226,  226,  226,  219,  226,    0,    0,    0, 
            0,    0,  231,    0,    0,  219,  226,  226,    0,    0, 
            0,    0,    0,    0,  231,  226,  226,    0,  226,  226, 
          226,  226,  226,    0,    0,    0,  231,    0,    0,    0, 
          226,    0,    0,    0,    0,    0,    0,    0,    0,  219, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  219,  219,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  226,    0,    0,  226,    0,    0, 
          226,    0,  226,  223,  223,  223,    0,    0,  223,  223, 
          223,  221,  223,    0,    0,    0,    0,    0,  226,    0, 
            0,  221,  223,  223,    0,    0,    0,    0,    0,    0, 
          226,  223,  223,    0,  223,  223,  223,  223,  223,    0, 
            0,    0,  226,    0,    0,    0,  223,    0,    0,    0, 
            0,    0,    0,    0,    0,  221,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  221, 
          221,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          223,    0,    0,  223,    0,    0,  223,    0,  223,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  262, 
            0,  224,  224,  224,  223,    0,  224,  224,  224,  262, 
          224,    0,    0,    0,    0,    0,  223,    0,    0,    0, 
          224,  224,    0,    0,    0,    0,    0,    0,  223,  224, 
          224,    0,  224,  224,  224,  224,  224,    0,    0,    0, 
            0,    0,    0,  262,  224,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  262,  262,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  224,    0, 
            0,  224,    0,    0,  224,  222,  224,  219,  219,  219, 
            0,    0,  219,  219,  219,  222,  219,    0,    0,    0, 
            0,    0,  224,    0,    0,    0,  219,  219,    0,    0, 
            0,    0,    0,    0,  224,  219,  219,    0,  219,  219, 
          219,  219,  219,    0,    0,    0,  224,    0,    0,  222, 
          219,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  222,  222,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  219,    0,    0,  219,    0,    0, 
          219,  264,  219,  221,  221,  221,    0,    0,  221,  221, 
          221,  264,  221,    0,    0,    0,    0,    0,  219,    0, 
            0,    0,  221,  221,    0,    0,    0,    0,    0,    0, 
          219,  221,  221,    0,  221,  221,  221,  221,  221,    0, 
            0,    0,  219,    0,    0,  264,  221,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          264,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          221,    0,    0,  221,    0,    0,  221,    0,  221,  220, 
            0,  262,  262,  262,    0,    0,  262,  262,  262,  220, 
          262,    0,    0,    0,  221,    0,    0,    0,    0,    0, 
          262,  262,    0,    0,    0,    0,  221,    0,    0,  262, 
          262,    0,  262,  262,  262,  262,  262,    0,  221,    0, 
            0,    0,    0,  220,  262,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  220,  220,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  262,    0, 
            0,  262,    0,    0,  262,  218,  262,  222,  222,  222, 
            0,    0,  222,  222,  222,  218,  222,    0,    0,    0, 
            0,    0,  262,    0,    0,    0,  222,  222,    0,    0, 
            0,    0,    0,    0,  262,  222,  222,    0,  222,  222, 
          222,  222,  222,    0,    0,    0,  262,    0,    0,  218, 
          222,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  218,  218,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  222,    0,    0,  222,    0,    0, 
          222,  287,  222,  264,  264,  264,    0,    0,  264,  264, 
          264,  287,  264,    0,    0,    0,    0,    0,  222,    0, 
            0,    0,  264,  264,    0,    0,    0,    0,    0,    0, 
          222,  264,  264,    0,  264,  264,  264,  264,  264,    0, 
            0,    0,  222,    0,    0,  287,  264,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  289, 
          287,    0,    0,    0,    0,    0,    0,    0,    0,  289, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          264,    0,    0,  264,    0,    0,  264,    0,  264,    0, 
            0,  220,  220,  220,    0,    0,  220,  220,  220,    0, 
          220,    0,    0,  289,  264,    0,    0,    0,    0,    0, 
          220,  220,    0,    0,    0,    0,  264,    0,  289,  220, 
          220,    0,  220,  220,  220,  220,    0,    0,  264,    0, 
            0,    0,    0,    0,  220,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  646,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  646,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  220,    0, 
            0,  220,    0,    0,  220,    0,  220,  218,  218,  218, 
            0,    0,  218,  218,  218,   60,  218,    0,    0,    0, 
            0,    0,  220,    0,    0,   60,  218,  218,    0,    0, 
            0,    0,    0,    0,  220,  218,  218,  646,  218,  218, 
          218,  218,    0,    0,    0,    0,  220,    0,    0,    0, 
          218,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,   60,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  218,    0,    0,  218,    0,    0, 
          218,    0,  218,  287,  287,  287,    0,    0,  287,  287, 
          287,    0,  287,    0,   64,    0,    0,    0,  218,    0, 
            0,    0,  287,  287,   64,    0,    0,    0,    0,    0, 
          218,  287,  287,    0,  287,  287,  287,  287,  287,    0, 
            0,    0,  218,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,   62,    0,    0,    0,    0,    0, 
            0,  289,  289,  289,   62,    0,  289,  289,  289,    0, 
          289,    0,    0,   64,    0,    0,    0,    0,    0,    0, 
          289,  289,    0,  287,    0,    0,  287,    0,  287,  289, 
          289,    0,  289,  289,  289,  289,  289,    0,    0,    0, 
            0,    0,    0,    0,  287,    0,    0,    0,    0,    0, 
            0,    0,   53,   62,    0,    0,    0,    0,    0,    0, 
            0,    0,   53,    0,    0,    0,    0,    0,  287,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  289,    0,    0,  289,    0,  289,    0,    0,    0, 
          646,  646,  646,    0,    0,  646,  646,  646,    0,  646, 
            0,    0,  289,    0,    0,   55,    0,    0,    0,  646, 
          646,   53,    0,    0,    0,   55,    0,    0,  646,  646, 
            0,  646,  646,  646,  646,  646,  289,   60,   60,   60, 
            0,    0,   60,   60,   60,    0,   60,    0,    0,    0, 
            0,   56,    0,    0,    0,    0,   60,   60,    0,    0, 
            0,   56,    0,    0,    0,   60,   60,    0,   60,   60, 
           60,   60,   60,    0,   55,    0,    0,    0,    0,    0, 
          646,    0,    0,  646,    0,  646,    0,    0,    0,    0, 
            0,    0,  293,    0,    0,    0,    0,    0,    0,    0, 
            0,  646,  293,    0,  294,    0,    0,    0,    0,    0, 
           56,    0,    0,    0,  294,    0,    0,   60,    0,    0, 
           60,    0,    0,    0,    0,  646,   64,   64,   64,    0, 
            0,   64,   64,   64,    0,   64,  289,    0,   60,    0, 
            0,    0,    0,    0,    0,   64,   64,    0,  290,  304, 
            0,  293,    0,    0,   64,   64,    0,   64,   64,   64, 
           64,   64,   60,  294,    0,    0,   62,   62,   62,    0, 
            0,   62,   62,   62,    0,   62,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,   62,   62,    0,    0,    0, 
            0,    0,    0,    0,   62,   62,    0,   62,   62,   62, 
           62,   62,  411,    0,    0,    0,   64,    0,    0,   64, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,   53,   53,   53,   64,    0,   53, 
           53,   53,    0,   53,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,   53,  218,    0,   62,    0,    0,   62, 
            0,   64,   53,   53,  218,   53,   53,   53,   53,   53, 
            0,    0,    0,    0,    0,    0,    0,   62,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,   55,   55,   55, 
            0,    0,   55,   55,   55,    0,   55,    0,  264,    0, 
            0,   62,    0,    0,    0,    0,   55,    0,    0,  304, 
            0,    0,    0,  218,   53,   55,   55,    0,   55,   55, 
           55,   55,   55,   56,   56,   56,    0,    0,   56,   56, 
           56,    0,   56,    0,    0,   53,    0,    0,    0,    0, 
            0,    0,   56,    0,    0,    0,    0,    0,    0,    0, 
            0,   56,   56,    0,   56,   56,   56,   56,   56,   53, 
            0,    0,    0,    0,  293,  293,  293,   55,    0,  293, 
          293,  293,    0,  293,    0,    0,  294,  294,  294,    0, 
            0,  294,  294,  294,    0,  294,    0,    0,   55,    0, 
            0,    0,    0,    0,    0,  293,  293,  293,  293,  293, 
            0,    0,    0,   56,    0,    0,    0,  294,  294,  294, 
          294,  294,   55,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,   56,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  293,    0,  129,  293,    0,    0,    0,   56,    0, 
            0,    0,    0,  294,    0,    0,  294,    0,    0,    0, 
            0,    0,    0,    0,    0,  293,  386,  387,  388,  389, 
          390,  391,  392,  393,  394,  395,  396,  294,  397,  398, 
            0,    0,  399,  400,    0,    0,    0,    0,    0,  293, 
            0,    0,  129,    0,    0,    0,    0,    0,  401,    0, 
          402,  294,  403,  404,  405,  406,  407,  408,  409,    0, 
          410,    0,    0,    0,    0,    0,  218,  218,  218,    0, 
            0,    0,  218,  218,  513,  218,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  218,  218,    0,  218,  218,  218, 
          218,    0,    0,    0,    0,    0,    4,    5,    6,    0, 
            8,    0,    0,    0,    9,   10,    0,    0,    0,   11, 
            0,   12,   13,   14,   15,   16,   17,   18,    0,  130, 
            0,    0,   19,   20,   21,  235,  236,  237,  238,    0, 
            0,  117,    0,    0,    0,    0,  218,    0,   28,    0, 
            0,  239,  240,  241,    0,  242,   35,  243,  244,  245, 
          246,  281,   40,   41,   42,   43,    0,  218,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  130,    0, 
            0,    0,    0,    0,    0,    0,  247,    0,    0,  130, 
            0,  218,   46,   47,    0,   48,    0,  282,    0,  283, 
            0,   50,    0,    0,    0,    0,    0,    0,    0,  284, 
          516,    0,    0,    0,   52,  285,   54,   55,   56,   57, 
            0,    0,    0,   58,    0,   59,   60,    0,   61,   62, 
           63,   64,    0,   65,   66,  286,    0,    0,    0,  129, 
          129,  129,  129,  129,  129,  129,  129,  129,  129,  129, 
            0,    0,  129,  129,    0,  129,  129,  129,  129,  129, 
          129,  129,    0,  616,    0,  129,  129,  129,  129,  129, 
          129,  129,  129,    0,    0,  129,    0,    0,    0,    0, 
            0,  129,  129,    0,  129,  129,  129,  129,    0,  129, 
          129,  129,  129,  129,  129,    0,  129,  129,  129,  129, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  513,    0,    0,  129,    0,    0,    0,  513,  513, 
          129,    0,    0,  129,  616,    0,  129,  129,    0,  129, 
            0,  129,    0,    0,    0,  129,    0,    0,    0,    0, 
          129,    0,    0,  129,    0,  616,  623,    0,  129,  129, 
          129,  129,  129,  129,    0,    0,    0,  129,    0,  129, 
          129,    0,  129,  129,  129,  129,    0,  129,  129,    0, 
            0,    0,    0,    0,    0,  130,  130,  130,  130,  130, 
          130,  130,  130,  130,  130,  130,    0,    0,  130,  130, 
            0,  130,  130,  130,  130,  130,  130,  130,    0,  617, 
            0,  130,  130,  130,  130,  130,  130,  130,  130,    0, 
            0,  130,    0,    0,    0,    0,    0,  130,  130,    0, 
          130,  130,  130,  130,    0,  130,  130,  130,  130,  130, 
          130,    0,  130,  130,  130,  130,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  516,    0,    0, 
          130,    0,    0,    0,  516,  516,  130,    0,    0,  130, 
          617,    0,  130,  130,    0,  130,    0,  130,    0,    0, 
            0,  130,    0,    0,    0,    0,  130,    0,    0,  130, 
            0,  617,  315,    0,  130,  130,  130,  130,  130,  130, 
            0,    0,    0,  130,    0,  130,  130,    0,  130,  130, 
          130,  130,    0,  130,  130,    0,    0,    0,    0,    0, 
            0,  129,  129,  129,  129,  129,  129,  129,  129,  129, 
          129,  129,    0,    0,  129,  129,    0,  129,  129,  129, 
          129,  129,  129,  129,    0,  623,    0,  130,  129,  129, 
          129,  129,  129,  129,  129,    0,    0,  129,    0,    0, 
            0,    0,    0,  129,  129,    0,  129,  129,  129,  129, 
            0,  129,  129,  129,  129,  129,  129,    0,  129,  129, 
          129,  129,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  623,    0,    0,  130,    0,    0,    0, 
          623,  623,  129,    0,    0,  129,  619,    0,  129,  129, 
            0,  129,    0,  129,    0,    0,    0,  129,    0,    0, 
            0,    0,  129,    0,    0,  129,    0,  623,  316,    0, 
          129,  129,  129,  129,  129,  129,    0,    0,    0,  129, 
            0,  129,  129,    0,  129,  129,  129,  129,    0,  129, 
          129,    0,    0,    0,    0,    0,    0,  130,  130,  130, 
          130,  130,  130,  130,  130,  130,  130,  130,    0,    0, 
          130,  130,    0,  130,  130,  130,  130,  130,  130,  130, 
            0,  360,    0,    0,  130,  130,  130,  130,  130,  130, 
          130,    0,    0,  130,    0,    0,    0,    0,    0,  130, 
          130,    0,  130,  130,  130,  130,    0,  130,  130,  130, 
          130,  130,  130,    0,  130,  130,  130,  130,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  315, 
          360,    0,    0,    0,    0,    0,  315,  315,  130,    0, 
            0,  130,  620,    0,  130,  130,    0,  130,    0,  130, 
            0,    0,    0,  130,    0,    0,    0,    0,  130,    0, 
            0,  130,    0,    0,    0,    0,  130,  130,  130,  130, 
          130,  130,    0,    0,    0,  130,    0,  130,  130,    0, 
          130,  130,  130,  130,    0,  130,  130,    0,    0,    0, 
            0,    0,    0,  130,  130,  130,  130,  130,  130,  130, 
          130,  130,  130,  130,    0,    0,  130,  130,    0,  130, 
          130,  130,  130,  130,  130,  130,    0,  307,    0,    0, 
          130,  130,  130,  130,  130,  130,  130,    0,    0,  130, 
            0,    0,    0,    0,    0,  130,  130,    0,  130,  130, 
          130,  130,    0,  130,  130,  130,  130,  130,  130,    0, 
          130,  130,  130,  130,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  316,  307,    0,    0,    0, 
            0,    0,  316,  316,  130,    0,    0,  130,    0,    0, 
          130,  130,    0,  130,    0,  130,    0,    0,    0,  130, 
            0,    0,    0,    0,  130,    0,    0,  130,    0,    0, 
            0,    0,  130,  130,  130,  130,  130,  130,    0,    0, 
            0,  130,    0,  130,  130,    0,  130,  130,  130,  130, 
            0,  130,  130,    0,    0,    0,    0,  360,  360,  360, 
          360,  360,  360,  360,  360,  360,  360,  360,    0,  360, 
          360,  360,  360,  360,  360,  360,  360,  360,  360,  360, 
          541,    0,    0,    0,  360,  360,  360,  360,  360,  360, 
          360,    0,    0,  360,    0,    0,    0,    0,    0,  360, 
          360,    0,  360,  360,  360,  360,    0,  360,  360,  360, 
          360,  360,  360,    0,  360,  360,  360,  360,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  541, 
            0,    0,    0,    0,    0,    0,    0,    0,  360,    0, 
            0,  360,    0,    0,  360,  360,    0,  360,    0,  360, 
            0,    0,    0,  360,    0,    0,    0,    0,    0,    0, 
            0,  360,    0,    0,    0,    0,  360,  360,  360,  360, 
          360,  360,    0,    0,    0,  360,    0,  360,  360,    0, 
          360,  360,  360,  360,    0,  360,  360,    0,    0,    0, 
            0,    0,    0,  307,  307,  307,  307,  307,  307,  307, 
          307,  307,  307,  307,    0,    0,  307,  307,    0,  307, 
          307,  307,  307,  307,  307,  307,  645,    0,    0,    0, 
          307,  307,  307,  307,  307,  307,  307,    0,    0,  307, 
            0,    0,    0,    0,    0,  307,  307,    0,  307,  307, 
          307,  307,    0,  307,  307,  307,  307,  307,  307,    0, 
          307,  307,  307,  307,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  645,    0,    0,    0,    0, 
            0,    0,    0,    0,  307,    0,    0,  307,    0,    0, 
          307,  307,    0,  307,    0,  307,    0,    0,    0,  307, 
            0,    0,    0,    0,    0,    0,    0,  307,    0,   18, 
            0,    0,  307,  307,  307,  307,  307,  307,    0,    0, 
            0,  307,    0,  307,  307,    0,  307,  307,  307,  307, 
            0,  307,  307,    0,    0,    0,  541,  541,  541,  541, 
          541,  541,  541,  541,  541,  541,  541,    0,    0,  541, 
          541,    0,  541,  541,  541,  541,  541,  541,  541,  645, 
            0,    0,    0,  541,  541,  541,  541,  541,  541,  541, 
            0,    0,  541,    0,    0,    0,    0,    0,  541,  541, 
            0,  541,  541,  541,  541,    0,  541,  541,  541,  541, 
          541,  541,    0,  541,  541,  541,  541,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  645,    0, 
            0,    0,    0,    0,    0,    0,    0,  541,    0,    0, 
          541,    0,    0,  541,  541,    0,  541,    0,  541,    0, 
            0,    0,  541,    0,    0,    0,    0,    0,    0,    0, 
          541,    0,    0,    0,    0,  541,  541,  541,  541,  541, 
          541,    0,    0,    0,  541,    0,  541,  541,    0,  541, 
          541,  541,  541,    0,  541,  541,    0,    0,    0,    0, 
            0,    0,  645,  645,  645,  645,  645,  645,  645,  645, 
          645,  645,  645,    0,    0,  645,  645,    0,  645,  645, 
          645,  645,  645,  645,  645,  256,    0,    0,    0,  645, 
          645,  645,  645,  645,  645,  645,    0,    0,  645,    0, 
            0,    0,    0,    0,  645,  645,    0,  645,  645,  645, 
          645,    0,  645,  645,  645,  645,  645,  645,    0,  645, 
          645,  645,  645,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  255,    0,    0,    0,    0,    0, 
            0,    0,    0,  645,    0,    0,  645,    0,    0,  645, 
          645,    0,  645,    0,  645,    0,    0,    0,  645,    0, 
            0,    0,    0,    0,    0,    0,  645,    0,    0,    0, 
            0,  645,  645,  645,  645,  645,  645,    0,    0,    0, 
          645,    0,  645,  645,    0,  645,  645,  645,  645,    0, 
          645,  645,    0,    0,    0,  645,  645,  645,  645,  645, 
          645,    0,    0,    0,  645,  645,    0,    0,    0,  645, 
            0,  645,  645,  645,  645,  645,  645,  645,  309,    0, 
            0,    0,  645,  645,  645,  645,  645,  645,  645,    0, 
            0,  645,    0,    0,    0,    0,    0,  645,  645,    0, 
          645,  645,  645,  645,    0,  645,  645,  645,  645,  645, 
          645,    0,  645,  645,  645,  645,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  645,    0,    0,  645, 
            0,    0,  645,  645,    0,  645,    0,  645,    0,    0, 
            0,  645,    0,    0,    0,    0,    0,    0,    0,  645, 
            0,    0,  645,    0,  645,  645,  645,  645,  645,  645, 
            0,    0,    0,  645,    0,  645,  645,    0,  645,  645, 
          645,  645,    0,  645,  645,    0,    0,    0,    0,    0, 
            0,    0,    4,    5,    6,    0,    8,    0,    0,    0, 
            9,   10,    0,    0,    0,   11,    0,   12,   13,   14, 
           15,   16,   17,   18,  304,    0,    0,    0,   19,   20, 
           21,  235,  236,  237,  238,    0,    0,   26,    0,    0, 
            0,    0,    0,    0,   28,    0,    0,  239,  240,  241, 
      };
   }

   private static final short[] yyTable4() {
      return new short[] {

            0,  242,   35,  243,  244,  245,  246,    0,   40,   41, 
           42,   43,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  247,    0,    0,  130,    0,    0,   46,   47, 
            0,   48,    0,    0,    0,    0,    0,   50,    0,    0, 
            0,    0,    0,    0,    0,   51,    0,    0,    0,    0, 
           52,   53,   54,   55,   56,   57,    0,    0,    0,   58, 
            0,   59,   60,    0,   61,   62,   63,   64,    0,   65, 
           66,    0,    0,    0,    0,  311,  311,  311,    0,  311, 
            0,    0,    0,  311,  311,    0,    0,    0,  311,    0, 
          311,  311,  311,  311,  311,  311,  311,    0,    0,    0, 
            0,  311,  311,  311,  311,  311,  311,  311,    0,    0, 
          311,    0,    0,    0,    0,    0,    0,  311,    0,    0, 
          311,  311,  311,    0,  311,  311,  311,  311,  311,  311, 
          349,  311,  311,  311,  311,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  311,    0,    0,  311,    0, 
          309,  311,  311,    0,  311,    0,    0,    0,    0,    0, 
          311,    0,    0,    0,    0,    0,    0,    0,  311,    0, 
            0,    0,    0,  311,  311,  311,  311,  311,  311,    0, 
            0,    0,  311,    0,  311,  311,    0,  311,  311,  311, 
          311,    0,  311,  311,    0,    0,    0,    0,    0,    0, 
            0,    4,    5,    6,    0,    8,    0,    0,    0,    9, 
           10,    0,    0,    0,   11,    0,   12,   13,   14,   15, 
           16,   17,   18,    0,    0,    0,    0,   19,   20,   21, 
          235,  236,  237,  238,    0,    0,   26,    0,    0,    0, 
            0,    0,    0,   28,    0,    0,  239,  240,  241,  624, 
          242,   35,  243,  244,  245,  246,    0,   40,   41,   42, 
           43,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  247,    0,    0,  130,    0,    0,   46,   47,    0, 
           48,    0,    0,    0,    0,    0,   50,    0,    0,    0, 
            0,    0,    0,    0,   51,    0,    0,    0,    0,   52, 
           53,   54,   55,   56,   57,    0,    0,    0,   58,    0, 
           59,   60,    0,   61,   62,   63,   64,    0,   65,   66, 
            0,    0,    0,    4,    5,    6,    0,    8,    0,    0, 
            0,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,  108,  109,   17,   18,    0,    0,    0,    0,  110, 
          111,  112,  336,  337,  338,  339,    0,    0,  117,    0, 
            0,    0,    0,    0,    0,  118,    0,    0,  340,  341, 
          342,    0,  343,   35,  344,  345,  346,  347,    0,   40, 
            0,    0,  127,  469,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  348,    0,    0,  130,    0,    0,   46, 
           47,    0,   48,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,   52,   53,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
           65,   66,    4,    5,    6,    0,    8,    0,    0,    0, 
            9,   10,    0,    0,    0,   11,    0,   12,   13,   14, 
          108,  109,   17,   18,    0,    0,    0,    0,  110,  111, 
          112,  336,  337,  338,  339,    0,    0,  117,    0,    0, 
            0,    0,    0,    0,  118,    0,    0,  340,  341,  342, 
            0,  343,   35,  344,  345,  346,  347,    0,   40,    0, 
            0,  127,  469,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  348,    0,    0,  130,    0,    0,   46,   47, 
            0,   48,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
           52,   53,   54,   55,   56,   57,    0,    0,    0,   58, 
            0,   59,   60,    0,   61,   62,   63,   64,    0,   65, 
           66,    4,    5,    6,    7,    8,    0,    0,    0,    9, 
           10,    0,    0,    0,   11,    0,   12,   13,   14,   15, 
           16,   17,   18,    0,    0,    0,    0,   19,   20,   21, 
           22,   23,   24,   25,    0,    0,   26,    0,    0,    0, 
            0,    0,   27,   28,   29,   30,   31,   32,   33,    0, 
           34,   35,   36,   37,   38,   39,    0,   40,   41,   42, 
           43,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,   44,    0,    0,   45,    0,    0,   46,   47,    0, 
           48,    0,   49,    0,    0,    0,   50,    0,    0,    0, 
            0,    0,    0,    0,   51,    0,    0,    0,    0,   52, 
           53,   54,   55,   56,   57,    0,    0,    0,   58,    0, 
           59,   60,    0,   61,   62,   63,   64,    0,   65,   66, 
            4,    5,    6,    7,  317,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,   15,   16, 
           17,   18,    0,    0,    0,    0,   19,   20,   21,   22, 
           23,   24,   25,    0,    0,   26,    0,    0,    0,    0, 
            0,   27,   28,    0,   30,   31,   32,   33,    0,   34, 
           35,   36,   37,   38,   39,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
           44,    0,    0,   45,    0,    0,   46,   47,    0,   48, 
            0,   49,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,   51,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,   65,   66,  138, 
          139,  140,  141,  142,  143,  144,  145,  146,  147,  148, 
          149,  150,  151,  152,  153,  154,  155,  156,  157,  158, 
          159,  160,  161,    0,    0,  162,  163,  164,  165,  166, 
          167,  168,  169,  170,  171,    0,    0,    0,    0,  172, 
          173,  174,  175,  176,  177,  178,  179,    0,  180,  181, 
          123,  124,  182,  126,    0,    0,  183,  184,    0,  185, 
          186,  187,  188,  189,  190,  191,    0,    0,  192,  193, 
            0,    0,    0,  194,  195,  196,  197,    0,    0,    0, 
            0,    0,    0,  198,    0,    0,    0,    0,    0,    0, 
          199,  200,    0,  201,  202,  203,  204,  205,  206,  207, 
          208,  209,  210,  211,    0,    0,  212,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  213,  138, 
          139,  140,  141,  142,  143,  144,  145,  146,  147,  148, 
          149,  150,  151,  152,  153,  154,  155,  156,  157,  158, 
          159,  160,  161,    0,    0,  162,  163,  164,  219,  220, 
          221,  222,  169,  170,  171,    0,    0,    0,    0,  172, 
          173,  174,  175,  176,  223,  224,  225,    0,  226,  181, 
          356,  357,  227,  358,    0,    0,  183,  184,    0,  185, 
          186,  187,  188,  189,  190,  191,    0,    0,  192,  193, 
            0,    0,    0,  194,  195,  196,  197,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          199,  200,    0,  201,  202,  203,  204,  205,  206,  207, 
          208,  209,  210,  211,    0,    0,  212,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  213,  138, 
          139,  140,  141,  142,  143,  144,  145,  146,  147,  148, 
          149,  150,  151,  152,  153,  154,  155,  156,  157,  158, 
          159,  160,  161,    0,    0,  162,  163,  164,  219,  220, 
          221,  222,  169,  170,  171,    0,    0,    0,    0,  172, 
          173,  174,  175,  176,  223,  224,  225,    0,  226,  181, 
          306,    0,  227,    0,    0,    0,  183,  184,    0,  185, 
          186,  187,  188,  189,  190,  191,    0,    0,  192,  193, 
            0,    0,    0,  194,  195,  196,  197,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          199,  200,    0,  201,  202,  203,  204,  205,  206,  207, 
          208,  209,  210,  211,    0,    0,  212,   52,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  213,  138, 
          139,  140,  141,  142,  143,  144,  145,  146,  147,  148, 
          149,  150,  151,  152,  153,  154,  155,  156,  157,  158, 
          159,  160,  161,    0,    0,  162,  163,  164,  219,  220, 
          221,  222,  169,  170,  171,    0,    0,    0,    0,  172, 
          173,  174,  175,  176,  223,  224,  225,    0,  226,  181, 
            0,    0,  227,    0,    0,    0,  183,  184,    0,  185, 
          186,  187,  188,  189,  190,  191,    0,    0,  192,  193, 
            0,    0,    0,  194,  195,  196,  197,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          199,  200,    0,  201,  202,  203,  204,  205,  206,  207, 
          208,  209,  210,  211,    0,    0,  212,   52,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  213,  138, 
          139,  140,  141,  142,  143,  144,  145,  146,  147,  148, 
          149,  150,  151,  152,  153,  154,  155,  156,  157,  158, 
          159,  160,  161,    0,    0,  162,  163,  164,  219,  220, 
          221,  222,  169,  170,  171,    0,    0,    0,    0,  172, 
          173,  174,  175,  176,  223,  224,  225,    0,  226,  181, 
            0,    0,  227,    0,    0,    0,  183,  184,    0,  185, 
          186,  187,  188,  189,  190,  191,    0,    0,  192,  193, 
            0,    0,    0,  194,  195,  196,  197,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          199,  200,    0,  201,  202,  203,  204,  205,  206,  207, 
          208,  209,  210,  211,    0,    0,  212,    0,    0,    0, 
            0,    0,    0,    3,    4,    5,    6,    7,    8,    0, 
            0,    0,    9,   10,    0,    0,    0,   11,  213,   12, 
           13,   14,   15,   16,   17,   18,    0,    0,    0,    0, 
           19,   20,   21,   22,   23,   24,   25,    0,    0,   26, 
            0,    0,    0,    0,    0,   27,   28,   29,   30,   31, 
           32,   33,    0,   34,   35,   36,   37,   38,   39,    0, 
           40,   41,   42,   43,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,   44,    0,    0,   45,    0,    0, 
           46,   47,    0,   48,    0,   49,    0,    0,    0,   50, 
            0,    0,    0,    0,    0,    0,    0,   51,    0,    0, 
            0,    0,   52,   53,   54,   55,   56,   57,    0,    0, 
            0,   58,    0,   59,   60,    0,   61,   62,   63,   64, 
            0,   65,   66,  316,    4,    5,    6,    7,  317,    0, 
            0,    0,    9,   10,    0,    0,    0,   11,    0,   12, 
           13,   14,   15,   16,   17,   18,    0,    0,    0,    0, 
           19,   20,   21,   22,   23,   24,   25,    0,    0,   26, 
            0,    0,    0,    0,    0,   27,   28,    0,   30,   31, 
           32,   33,    0,   34,   35,   36,   37,   38,   39,    0, 
           40,   41,   42,   43,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,   44,    0,    0,  318,    0,    0, 
           46,   47,    0,   48,    0,   49,    0,    0,    0,   50, 
            0,    0,    0,    0,    0,    0,    0,   51,    0,    0, 
            0,    0,   52,   53,   54,   55,   56,   57,    0,    0, 
            0,   58,    0,   59,   60,    0,   61,   62,   63,   64, 
            0,   65,   66,  316,    4,    5,    6,    7,  317,    0, 
            0,    0,    9,   10,    0,    0,    0,   11,    0,   12, 
           13,   14,   15,   16,   17,   18,    0,    0,    0,    0, 
           19,   20,   21,   22,   23,   24,   25,    0,    0,   26, 
            0,    0,    0,    0,    0,   27,   28,    0,   30,   31, 
           32,   33,    0,   34,   35,   36,   37,   38,   39,    0, 
           40,   41,   42,   43,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,   44,    0,    0,   45,    0,    0, 
           46,   47,    0,   48,    0,   49,    0,    0,    0,   50, 
            0,    0,    0,    0,    0,    0,    0,   51,    0,    0, 
            0,    0,   52,   53,   54,   55,   56,   57,    0,    0, 
            0,   58,    0,   59,   60,    0,   61,   62,   63,   64, 
            0,   65,   66,    4,    5,    6,    0,    8,    0,    0, 
            0,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,   15,   16,   17,   18,    0,    0,    0,    0,   19, 
           20,   21,  235,  236,  237,  238,    0,    0,  117,    0, 
            0,    0,    0,    0,    0,   28,    0,    0,  239,  240, 
          241,    0,  242,   35,  243,  244,  245,  246,  281,   40, 
           41,   42,   43,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  247,    0,    0,  130,    0,    0,   46, 
           47,    0,   48,    0,  282,    0,  283,    0,   50,    0, 
            0,    0,    0,    0,    0,    0,  284,    0,    0,    0, 
            0,   52,  285,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
           65,   66,  286,    4,    5,    6,    0,    8,    0,    0, 
            0,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,  108,  109,   17,   18,    0,    0,    0,    0,  110, 
          111,  112,  235,  236,  237,  238,    0,    0,  117,    0, 
            0,    0,    0,    0,    0,   28,    0,    0,  239,  240, 
          241,    0,  242,   35,  243,  244,  245,  246,  281,   40, 
           41,   42,   43,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  247,    0,    0,  130,    0,    0,   46, 
           47,    0,   48,    0,  679,    0,  283,    0,   50,    0, 
            0,    0,    0,    0,    0,    0,  284,    0,    0,    0, 
            0,   52,  285,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
           65,   66,  286,  282,  282,  282,    0,  282,    0,    0, 
            0,  282,  282,    0,    0,    0,  282,    0,  282,  282, 
          282,  282,  282,  282,  282,    0,    0,    0,    0,  282, 
          282,  282,  282,  282,  282,  282,    0,    0,  282,    0, 
            0,    0,    0,    0,    0,  282,    0,    0,  282,  282, 
          282,    0,  282,  282,  282,  282,  282,  282,  282,  282, 
          282,  282,  282,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  282,    0,    0,  282,    0,    0,  282, 
          282,    0,  282,    0,  282,    0,  282,    0,  282,    0, 
            0,    0,    0,    0,    0,    0,  282,    0,    0,    0, 
            0,  282,  282,  282,  282,  282,  282,    0,    0,    0, 
          282,    0,  282,  282,    0,  282,  282,  282,  282,    0, 
          282,  282,  282,    4,    5,    6,    0,    8,    0,    0, 
            0,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,  108,  109,   17,   18,    0,    0,    0,    0,  110, 
          111,  112,  235,  236,  237,  238,    0,    0,  117,    0, 
            0,    0,    0,    0,    0,   28,    0,    0,  239,  240, 
          241,    0,  242,   35,  243,  244,  245,  246,  281,   40, 
           41,   42,   43,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  247,    0,    0,  130,    0,    0,   46, 
           47,    0,   48,    0,  282,    0,    0,    0,   50,    0, 
            0,    0,    0,    0,    0,    0,  284,    0,    0,    0, 
            0,   52,  285,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
           65,   66,  286,    4,    5,    6,    0,    8,    0,    0, 
            0,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,  108,  109,   17,   18,    0,    0,    0,    0,  110, 
          111,  112,  235,  236,  237,  238,    0,    0,  117,    0, 
            0,    0,    0,    0,    0,   28,    0,    0,  239,  240, 
          241,    0,  242,   35,  243,  244,  245,  246,  281,   40, 
           41,   42,   43,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  247,    0,    0,  130,    0,    0,   46, 
           47,    0,   48,    0,    0,    0,  283,    0,   50,    0, 
            0,    0,    0,    0,    0,    0,  284,    0,    0,    0, 
            0,   52,  285,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
           65,   66,  286,    4,    5,    6,    0,    8,    0,    0, 
            0,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,  108,  109,   17,   18,    0,    0,    0,    0,  110, 
          111,  112,  235,  236,  237,  238,    0,    0,  117,    0, 
            0,    0,    0,    0,    0,   28,    0,    0,  239,  240, 
          241,    0,  242,   35,  243,  244,  245,  246,  281,   40, 
           41,   42,   43,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  247,    0,    0,  130,    0,    0,   46, 
           47,    0,   48,    0,  679,    0,    0,    0,   50,    0, 
            0,    0,    0,    0,    0,    0,  284,    0,    0,    0, 
            0,   52,  285,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
           65,   66,  286,    4,    5,    6,    0,    8,    0,    0, 
            0,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,  108,  109,   17,   18,    0,    0,    0,    0,  110, 
          111,  112,  235,  236,  237,  238,    0,    0,  117,    0, 
            0,    0,    0,    0,    0,   28,    0,    0,  239,  240, 
          241,    0,  242,   35,  243,  244,  245,  246,  281,   40, 
           41,   42,   43,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  247,    0,    0,  130,    0,    0,   46, 
           47,    0,   48,    0,    0,    0,    0,    0,   50,    0, 
            0,    0,    0,    0,    0,    0,  284,    0,    0,    0, 
            0,   52,  285,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
           65,   66,  286,    4,    5,    6,    7,    8,    0,    0, 
            0,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,   15,   16,   17,   18,    0,    0,    0,    0,   19, 
           20,   21,   22,   23,   24,   25,    0,    0,   26,    0, 
            0,    0,    0,    0,   27,   28,   29,   30,   31,   32, 
           33,    0,   34,   35,   36,   37,   38,   39,    0,   40, 
           41,   42,   43,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,   44,    0,    0,   45,    0,    0,   46, 
           47,    0,   48,    0,   49,    0,    0,    0,   50,    0, 
            0,    0,    0,    0,    0,    0,   51,    0,    0,    0, 
            0,   52,   53,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
           65,   66,    4,    5,    6,    7,    8,    0,    0,    0, 
            9,   10,    0,    0,    0,   11,    0,   12,   13,   14, 
           15,   16,   17,   18,    0,    0,    0,    0,   19,   20, 
           21,   22,   23,   24,   25,    0,    0,   26,    0,    0, 
            0,    0,    0,   27,   28,    0,   30,   31,   32,   33, 
            0,   34,   35,   36,   37,   38,   39,    0,   40,   41, 
           42,   43,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,   44,    0,    0,   45,    0,    0,   46,   47, 
            0,   48,    0,   49,    0,    0,    0,   50,    0,    0, 
            0,    0,    0,    0,    0,   51,    0,    0,    0,    0, 
           52,   53,   54,   55,   56,   57,    0,    0,    0,   58, 
            0,   59,   60,    0,   61,   62,   63,   64,    0,   65, 
           66,    4,    5,    6,    0,    8,    0,    0,    0,    9, 
           10,    0,    0,    0,   11,    0,   12,   13,   14,  108, 
          109,   17,   18,    0,    0,    0,    0,  110,  111,  112, 
          235,  236,  237,  238,    0,    0,  117,    0,    0,    0, 
            0,    0,    0,   28,    0,    0,  239,  240,  241,    0, 
          242,   35,  243,  244,  245,  246,    0,   40,   41,   42, 
           43,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  247,    0,    0,  130,  504,    0,   46,   47,    0, 
           48,    0,    0,    0,    0,    0,   50,    0,    0,    0, 
            0,    0,    0,    0,  284,    0,    0,    0,    0,   52, 
           53,   54,   55,   56,   57,    0,    0,    0,   58,    0, 
           59,   60,    0,   61,   62,   63,   64,    0,   65,   66, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,   15,   16, 
           17,   18,    0,    0,    0,    0,   19,   20,   21,  235, 
          236,  237,  238,    0,    0,  117,    0,    0,    0,    0, 
            0,    0,   28,    0,    0,  239,  240,  241,    0,  242, 
           35,  243,  244,  245,  246,    0,   40,   41,   42,   43, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          247,    0,    0,  130,    0,    0,   46,   47,    0,   48, 
            0,  615,    0,    0,    0,   50,    0,    0,    0,    0, 
            0,    0,    0,  284,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,   65,   66,    4, 
            5,    6,    0,    8,    0,    0,    0,    9,   10,    0, 
            0,    0,   11,    0,   12,   13,   14,  108,  109,   17, 
           18,    0,    0,    0,    0,  110,  111,  112,  235,  236, 
          237,  238,    0,    0,  117,    0,    0,    0,    0,    0, 
            0,   28,    0,    0,  239,  240,  241,    0,  242,   35, 
          243,  244,  245,  246,    0,   40,   41,   42,   43,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  247, 
            0,    0,  130,    0,    0,   46,   47,    0,   48,    0, 
          282,    0,    0,    0,   50,    0,    0,    0,    0,    0, 
            0,    0,  284,    0,    0,    0,    0,   52,   53,   54, 
           55,   56,   57,    0,    0,    0,   58,    0,   59,   60, 
            0,   61,   62,   63,   64,    0,   65,   66,    4,    5, 
            6,    0,    8,    0,    0,    0,    9,   10,    0,    0, 
            0,   11,    0,   12,   13,   14,  108,  109,   17,   18, 
            0,    0,    0,    0,  110,  111,  112,  235,  236,  237, 
          238,    0,    0,  117,    0,    0,    0,    0,    0,    0, 
           28,    0,    0,  239,  240,  241,    0,  242,   35,  243, 
          244,  245,  246,    0,   40,   41,   42,   43,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  247,    0, 
            0,  130,    0,    0,   46,   47,    0,   48,    0,  615, 
            0,    0,    0,   50,    0,    0,    0,    0,    0,    0, 
            0,  284,    0,    0,    0,    0,   52,   53,   54,   55, 
           56,   57,    0,    0,    0,   58,    0,   59,   60,    0, 
           61,   62,   63,   64,    0,   65,   66,    4,    5,    6, 
            0,    8,    0,    0,    0,    9,   10,    0,    0,    0, 
           11,    0,   12,   13,   14,  108,  109,   17,   18,    0, 
            0,    0,    0,  110,  111,  112,  235,  236,  237,  238, 
            0,    0,  117,    0,    0,    0,    0,    0,    0,   28, 
            0,    0,  239,  240,  241,    0,  242,   35,  243,  244, 
          245,  246,    0,   40,   41,   42,   43,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  247,    0,    0, 
          130,    0,    0,   46,   47,    0,   48,    0,  888,    0, 
            0,    0,   50,    0,    0,    0,    0,    0,    0,    0, 
          284,    0,    0,    0,    0,   52,   53,   54,   55,   56, 
           57,    0,    0,    0,   58,    0,   59,   60,    0,   61, 
           62,   63,   64,    0,   65,   66,    4,    5,    6,    0, 
            8,    0,    0,    0,    9,   10,    0,    0,    0,   11, 
            0,   12,   13,   14,  108,  109,   17,   18,    0,    0, 
            0,    0,  110,  111,  112,  235,  236,  237,  238,    0, 
            0,  117,    0,    0,    0,    0,    0,    0,   28,    0, 
            0,  239,  240,  241,    0,  242,   35,  243,  244,  245, 
          246,    0,   40,   41,   42,   43,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  247,    0,    0,  130, 
            0,    0,   46,   47,    0,   48,    0,  679,    0,    0, 
            0,   50,    0,    0,    0,    0,    0,    0,    0,  284, 
            0,    0,    0,    0,   52,   53,   54,   55,   56,   57, 
            0,    0,    0,   58,    0,   59,   60,    0,   61,   62, 
           63,   64,    0,   65,   66,  634,  634,  634,    0,  634, 
            0,    0,    0,  634,  634,    0,    0,    0,  634,    0, 
          634,  634,  634,  634,  634,  634,  634,    0,    0,    0, 
            0,  634,  634,  634,  634,  634,  634,  634,    0,    0, 
          634,    0,    0,    0,    0,    0,    0,  634,    0,    0, 
          634,  634,  634,    0,  634,  634,  634,  634,  634,  634, 
            0,  634,  634,  634,  634,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  634,    0,    0,  634,  634, 
            0,  634,  634,    0,  634,    0,    0,    0,    0,    0, 
          634,    0,    0,    0,    0,    0,    0,    0,  634,    0, 
            0,    0,    0,  634,  634,  634,  634,  634,  634,    0, 
            0,    0,  634,    0,  634,  634,    0,  634,  634,  634, 
          634,    0,  634,  634,    4,    5,    6,    0,    8,    0, 
            0,    0,    9,   10,    0,    0,    0,   11,    0,   12, 
           13,   14,   15,   16,   17,   18,    0,    0,    0,    0, 
           19,   20,   21,  235,  236,  237,  238,    0,    0,   26, 
            0,    0,    0,    0,    0,    0,   28,    0,    0,  239, 
          240,  241,    0,  242,   35,  243,  244,  245,  246,    0, 
           40,   41,   42,   43,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,  247,    0,    0,  130,    0,    0, 
           46,   47,    0,   48,    0,    0,    0,    0,    0,   50, 
            0,    0,    0,    0,    0,    0,    0,   51,    0,    0, 
            0,    0,   52,   53,   54,   55,   56,   57,    0,    0, 
            0,   58,    0,   59,   60,    0,   61,   62,   63,   64, 
            0,   65,   66,    4,    5,    6,    0,    8,    0,    0, 
            0,    9,   10,    0,    0,    0,   11,    0,   12,   13, 
           14,  108,  109,   17,   18,    0,    0,    0,    0,  110, 
          111,  112,  235,  236,  237,  238,    0,    0,  117,    0, 
            0,    0,    0,    0,    0,   28,    0,    0,  239,  240, 
          241,    0,  242,   35,  243,  244,  245,  246,    0,   40, 
           41,   42,   43,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,  247,    0,    0,  130,    0,    0,   46, 
           47,    0,   48,    0,    0,    0,    0,    0,   50,    0, 
            0,    0,    0,    0,    0,    0,  284,    0,    0,    0, 
            0,   52,   53,   54,   55,   56,   57,    0,    0,    0, 
           58,    0,   59,   60,    0,   61,   62,   63,   64,    0, 
           65,   66,    4,    5,    6,    0,    8,    0,    0,    0, 
            9,   10,    0,    0,    0,   11,    0,   12,   13,   14, 
           15,   16,   17,   18,    0,    0,    0,    0,   19,   20, 
           21,  235,  236,  237,  238,    0,    0,  117,    0,    0, 
            0,    0,    0,    0,   28,    0,    0,  239,  240,  241, 
            0,  242,   35,  243,  244,  245,  246,    0,   40,   41, 
           42,   43,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  247,    0,    0,  130,    0,    0,   46,   47, 
            0,   48,    0,    0,    0,    0,    0,   50,    0,    0, 
            0,    0,    0,    0,    0,  284,    0,    0,    0,    0, 
           52,   53,   54,   55,   56,   57,    0,    0,    0,   58, 
            0,   59,   60,    0,   61,   62,   63,   64,    0,   65, 
           66,  634,  634,  634,    0,  634,    0,    0,    0,  634, 
          634,    0,    0,    0,  634,    0,  634,  634,  634,  634, 
          634,  634,  634,    0,    0,    0,    0,  634,  634,  634, 
          634,  634,  634,  634,    0,    0,  634,    0,    0,    0, 
            0,    0,    0,  634,    0,    0,  634,  634,  634,    0, 
          634,  634,  634,  634,  634,  634,    0,  634,  634,  634, 
          634,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,  634,    0,    0,  634,    0,    0,  634,  634,    0, 
          634,    0,    0,    0,    0,    0,  634,    0,    0,    0, 
            0,    0,    0,    0,  634,    0,    0,    0,    0,  634, 
          634,  634,  634,  634,  634,    0,    0,    0,  634,    0, 
          634,  634,    0,  634,  634,  634,  634,    0,  634,  634, 
            4,    5,    6,    0,    8,    0,    0,    0,    9,   10, 
            0,    0,    0,   11,    0,   12,   13,   14,  108,  109, 
           17,   18,    0,    0,    0,    0,  110,  111,  112,  263, 
          264,  265,  266,    0,    0,  117,    0,    0,    0,    0, 
            0,    0,  118,    0,    0,  267,  268,  269,    0,  270, 
           35,  271,  272,  273,  274,    0,   40,    0,    0,  127, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          275,    0,    0,   45,    0,    0,   46,   47,    0,   48, 
            0,   49,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,   52,   53, 
           54,   55,   56,   57,    0,    0,    0,   58,    0,   59, 
           60,    0,   61,   62,   63,   64,    0,   65,   66,    4, 
            5,    6,    0,    8,    0,    0,    0,    9,   10,    0, 
            0,    0,   11,    0,   12,   13,   14,  108,  109,   17, 
           18,    0,    0,    0,    0,  110,  111,  112,  336,  337, 
          338,  339,    0,    0,  117,    0,    0,    0,    0,    0, 
            0,  118,    0,    0,  340,  341,  342,    0,  343,   35, 
          344,  345,  346,  347,    0,   40,    0,    0,  127,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,  348, 
            0,    0,  425,    0,    0,   46,   47,    0,   48,    0, 
          426,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,   52,   53,   54, 
           55,   56,   57,    0,    0,    0,   58,    0,   59,   60, 
            0,   61,   62,   63,   64,    0,   65,   66,    4,    5, 
            6,    0,    8,    0,    0,    0,    9,   10,    0,    0, 
            0,   11,    0,   12,   13,   14,  108,  109,   17,   18, 
            0,    0,    0,    0,  110,  111,  112,  113,  114,  115, 
          116,    0,    0,  117,    0,    0,    0,    0,    0,    0, 
          118,    0,    0,  119,  120,  121,    0,  122,   35,  123, 
          124,  125,  126,    0,   40,    0,    0,  127,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,  129,    0, 
            0,  130,    0,    0,   46,   47,    0,   48,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,   52,   53,   54,   55, 
           56,   57,    0,    0,    0,   58,    0,   59,   60,    0, 
           61,   62,   63,   64,    0,   65,   66,    4,    5,    6, 
            0,    8,    0,    0,    0,    9,   10,    0,    0,    0, 
           11,    0,   12,   13,   14,  108,  109,   17,   18,    0, 
            0,    0,    0,  110,  111,  112,  336,  337,  338,  339, 
            0,    0,  117,    0,    0,    0,    0,    0,    0,  118, 
            0,    0,  340,  341,  342,    0,  343,   35,  344,  345, 
          346,  347,    0,   40,    0,    0,  127,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,  348,    0,    0, 
          425,    0,    0,   46,   47,    0,   48,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,   52,   53,   54,   55,   56, 
           57,    0,    0,    0,   58,    0,   59,   60,    0,   61, 
           62,   63,   64,    0,   65,   66,    4,    5,    6,    0, 
            8,    0,    0,    0,    9,   10,    0,    0,    0,   11, 
            0,   12,   13,   14,  108,  109,   17,   18,    0,    0, 
            0,    0,  110,  111,  112,  113,  114,  115,  116,    0, 
            0,  117,    0,    0,    0,    0,    0,    0,  118,    0, 
            0,  119,  120,  121,    0,  951,   35,  123,  124,  952, 
          126,    0,   40,    0,    0,  127,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,  953,    0,    0,  130, 
            0,    0,   46,   47,    0,   48,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,   52,   53,   54,   55,   56,   57, 
            0,    0,    0,   58,    0,   59,   60,    0,   61,   62, 
           63,   64,    0,   65,   66,    4,    5,    6,    0,    8, 
            0,    0,    0,    9,   10,    0,    0,    0,   11,    0, 
           12,   13,   14,  108,  109,   17,   18,    0,    0,    0, 
            0,  110,  111,  112,  967,  968,  969,  970,    0,    0, 
          117,    0,    0,    0,    0,    0,    0,  118,    0,    0, 
          971,  972,  973,    0,  974,   35,  975,  976,  977,  978, 
            0,   40,    0,    0,  127,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,  979,    0,    0,  130,    0, 
            0,   46,   47,    0,   48,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,   52,   53,   54,   55,   56,   57,    0, 
            0,    0,   58,    0,   59,   60,    0,   61,   62,   63, 
           64,    0,   65,   66,  639,  597,    0,    0,  640,    0, 
            0,    0,  183,  184,    0,  185,  186,  187,  188,  189, 
          190,  191,    0,    0,  192,  193,    0,    0,    0,  194, 
          195,  196,  197,    0,    0,    0,    0,    0,    0,  300, 
            0,    0,    0,    0,    0,    0,  199,  200,    0,  201, 
          202,  203,  204,  205,  206,  207,  208,  209,  210,  211, 
          610,  605,  212,    0,  611,    0,    0,    0,  183,  184, 
            0,  185,  186,  187,  188,  189,  190,  191,    0,    0, 
          192,  193,    0,    0,  213,  194,  195,  196,  197,    0, 
            0,    0,    0,    0,    0,  300,    0,    0,    0,    0, 
            0,    0,  199,  200,    0,  201,  202,  203,  204,  205, 
          206,  207,  208,  209,  210,  211,  657,  597,  212,    0, 
          658,    0,    0,    0,  183,  184,    0,  185,  186,  187, 
          188,  189,  190,  191,    0,    0,  192,  193,    0,    0, 
          213,  194,  195,  196,  197,    0,    0,    0,    0,    0, 
            0,  300,    0,    0,    0,    0,    0,    0,  199,  200, 
            0,  201,  202,  203,  204,  205,  206,  207,  208,  209, 
          210,  211,  660,  605,  212,    0,  661,    0,    0,    0, 
          183,  184,    0,  185,  186,  187,  188,  189,  190,  191, 
            0,    0,  192,  193,    0,    0,  213,  194,  195,  196, 
          197,    0,    0,    0,    0,    0,    0,  300,    0,    0, 
            0,    0,    0,    0,  199,  200,    0,  201,  202,  203, 
          204,  205,  206,  207,  208,  209,  210,  211,  671,  597, 
          212,    0,  672,    0,    0,    0,  183,  184,    0,  185, 
          186,  187,  188,  189,  190,  191,    0,    0,  192,  193, 
            0,    0,  213,  194,  195,  196,  197,    0,    0,    0, 
            0,    0,    0,  300,    0,    0,    0,    0,    0,    0, 
          199,  200,    0,  201,  202,  203,  204,  205,  206,  207, 
          208,  209,  210,  211,  674,  605,  212,    0,  675,    0, 
            0,    0,  183,  184,    0,  185,  186,  187,  188,  189, 
          190,  191,    0,    0,  192,  193,    0,    0,  213,  194, 
          195,  196,  197,    0,    0,    0,    0,    0,    0,  300, 
            0,    0,    0,    0,    0,    0,  199,  200,    0,  201, 
          202,  203,  204,  205,  206,  207,  208,  209,  210,  211, 
          702,  597,  212,    0,  703,    0,    0,    0,  183,  184, 
            0,  185,  186,  187,  188,  189,  190,  191,    0,    0, 
          192,  193,    0,    0,  213,  194,  195,  196,  197,    0, 
            0,    0,    0,    0,    0,  300,    0,    0,    0,    0, 
            0,    0,  199,  200,    0,  201,  202,  203,  204,  205, 
          206,  207,  208,  209,  210,  211,  705,  605,  212,    0, 
          706,    0,    0,    0,  183,  184,    0,  185,  186,  187, 
          188,  189,  190,  191,    0,    0,  192,  193,    0,    0, 
          213,  194,  195,  196,  197,    0,    0,    0,    0,    0, 
            0,  300,    0,    0,    0,    0,    0,    0,  199,  200, 
            0,  201,  202,  203,  204,  205,  206,  207,  208,  209, 
          210,  211, 1057,  597,  212,    0, 1058,    0,    0,    0, 
          183,  184,    0,  185,  186,  187,  188,  189,  190,  191, 
            0,    0,  192,  193,    0,    0,  213,  194,  195,  196, 
          197,    0,    0,    0,    0,    0,    0,  300,    0,    0, 
            0,    0,    0,    0,  199,  200,    0,  201,  202,  203, 
          204,  205,  206,  207,  208,  209,  210,  211, 1062,  597, 
          212,    0, 1063,    0,    0,    0,  183,  184,    0,  185, 
          186,  187,  188,  189,  190,  191,    0,    0,  192,  193, 
            0,    0,  213,  194,  195,  196,  197,    0,    0,    0, 
            0,    0,    0,  300,    0,    0,    0,    0,    0,    0, 
          199,  200,    0,  201,  202,  203,  204,  205,  206,  207, 
          208,  209,  210,  211, 1065,  605,  212,    0, 1066,    0, 
            0,    0,  183,  184,    0,  185,  186,  187,  188,  189, 
          190,  191,    0,    0,  192,  193,    0,    0,  213,  194, 
          195,  196,  197,    0,    0,    0,    0,    0,    0,  300, 
            0,    0,    0,    0,    0,    0,  199,  200,    0,  201, 
          202,  203,  204,  205,  206,  207,  208,  209,  210,  211, 
          610,  605,  212,    0,  611,    0,    0,    0,  183,  184, 
            0,  185,  186,  187,  188,  189,  190,  191,    0,    0, 
          192,  193,    0,    0,  213,  194,  195,  196,  197,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,  199,  200,    0,  201,  202,  203,  204,  205, 
          206,  207,  208,  209,  210,  211,    0,    0,  212,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
            0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 
          213, 
      };
   }

   private static final short[] yyCheck1() {
      return new short[] {

           26,   74,   28,  292,   27,    2,    3,    4,    5,   88, 
            6,   90,    9,   10,   11,    7,   21,   14,   15,   16, 
           51,  466,   19,    7,  446,    2,    3,  486,  254,   99, 
          417,   45,   49,   15,   16,   27,    3,   19,  471,   10, 
            0,    0,   10,   27,    6,    7,   10,  129,   45,   10, 
            4,    5,   49,  424,   51,   11,   52,  444,  483,    0, 
           14,  486,  104,  922,   10,   27,  525,   15,   16,   10, 
           10,   19,   21,  460,   10,   11,   59,   10,   10,    0, 
           44,   15,   16,   44,   10,   19,  535,  474,   59,   10, 
           52,  416,  118,  418,  561,   49,  421,   61,   59,   47, 
           48,  106,   99,   10,  331,   10,   10,  428,  335,  643, 
          846,   44,  318,   47,  553,   10,   10,   59,   59,   59, 
          445,  956,  506,   44,  104,  659,   59,   10,  595,   61, 
           10,  128,   10,  130,  459,  844,  461,   44,   59,  673, 
           44,  947,  341,   10,   10,   99,  533,   43,  104,  653, 
          475,   10,   59,  280,   59,   59,  341,  106,  647,  663, 
          649,   44,  280,  112,   59,   10,   10,   10,   45,   10, 
          704,  250,  362,   10,   10,  624,   59,  325,   32,   59, 
         1039,   59,  305,   10,  509,   44,  345,  136,  759,   10, 
           10,   10,   59,   59,  765,   44,  305,  276,  553,   10, 
           44,   44,   10,  342,  363,   10,  495,  310,  647,  534, 
          649,  359,   61,  366,   10,   59,  638,   44,   59,  425, 
          359,   10,   59,  341, 1059,   10,  648,   10,  687,   10, 
           32,  127,  721,  312,  943,  362,  282,  234,   10,   10, 
           61,   10,    2,    3,  362, 1051,   10,   10,   59,    9, 
           10,   11,  375,  130,   59,   15,   16,   32,   10,   19, 
           10,  686,  687,   10,  261,  262,  375,  308,   10, 1005, 
           59,  350,   44,   10,   59,  836,   59,  257,  320,  349, 
          328,  342,  721,   10,  312,   45,   10,   59,  336,  317, 
          999,   51,  647,   32,  649,  856,   32,    0,  359,   32, 
          297,  257,  299,  300,  318,  338,  303,   10,  305,  373, 
          749,  778,  696,  310,  378,  297,   44,  299,  300,  316, 
           10,  318,  350,  351,   61,  261,  262,  776,  306,  751, 
          327,  652,  310,  755,   61,  376,  377,   61, 1047,  316, 
          320,  262,  263,  264,  305,   10,  267,  268,  269,  297, 
          271,  299,  349,   15,   16,  373,   59,   19,  386,  342, 
          378,  443,   10,  297,  320,  299,  721,   61,  128,  264, 
          130,   61,  293,  294,  295,  296,  297,  345,  342,  812, 
           61,  342,  379,  380,  381,  382,  383,  384,  385,  341, 
          303,  422,  681,  424,   59,  349,  342,  428,  429,  345, 
          305,  772,  357,  363,  363,  844,  383,  846,  912,  426, 
          342,  425,  325,  326,  375,  341,  387,  359,  339,  841, 
          417,  342,  363,  363,  338,  422,  359,  424,  425,  426, 
          279,  428,  429,  310,  431,  417,   44,  342,   10,  665, 
          281,  318,  363,  379,  380,  381,  382,  444,  342,  446, 
          447,  345,  359,  720,  431,  359,  418,  916,  279,  421, 
          457,  305,  444,  460,  431,  535,  387,  912,  465,  418, 
          375,  893,  426,  700,  234,  342,  359,  474,  460,  359, 
          503,  359,  267,  445,  267,    2,    3,    4,    5,   61, 
          424,  916,  474,  359,  428,  267,  445,   14,  342,  461, 
          310,  261,  262,  338,  943,  342,  342,  504,  947,  506, 
           10,  503,  461,  475,  470,  471,  305,  371,  372,  503, 
          517,  342,  342,  342,  378,  345,  475,  816,   45,  506, 
         1064,  375,   49,  467,  342,  483,  533,  297,  535,  299, 
          300,  503,  279,  303,  966,  305,  342,  509,  425,  345, 
          310,  533,  279,  342,  624,  279,  316,   10,  318,   59, 
          509,  342,   10,  830,  831,   10, 1005,  327,  594,  371, 
          372,  342,  534,  342,  522,  488,  378,  603,  342,  342, 
          457,  535,   99,  496,  497,  534,  375,   10,  465,  279, 
          342,   44,  342,   61,  599,  342,   44,  262,  263,  264, 
          342,  514,  607,  268,  269,  342,  271,  612,   61,  338, 
           10,   59, 1051,  130,   91,  342,   61,  643,  342,  379, 
          380,  381,  382,  383,  384,  385,  743,  624,  651,  290, 
          291,  701,   59,  659,   10,  297,   59,  299,   91,  373, 
          637,  638,  290,  291,  378,   10, 1033,  673,    2,    3, 
          599,  648,  342,  920,  650,  922,  271,  417,  607,  651, 
           14,   61,  422,  612,  424,  425,   10,  651,  428,  429, 
          624,  431,  269,  310,  271,  901,   10,  342,  704,  676, 
          636,  907,  339,   59,  444,   44,  446,  447,  650,  651, 
           10,   45,  641,   91,   59,  644,  769,  457,  363,  696, 
          460,  637,   10,  373,  701,  465,  776,  279,  378, 1034, 
          623,  279,    2,    3,  474,   59,   44,  234,  652,  696, 
          268,  269,  387,  749,   44,  756,  757,   61,  995,  996, 
           44,  762,  763,  646,  920,   44,   44,  362,  686,   59, 
          676,  772,  317,  860,  504,  338,  506,  701,  865,  308, 
          309,   59,  311,  317,  751,   45,  753,  517,  755,  756, 
          757,  342,  262,  263,  264,  762,  763,  345,  268,  269, 
          342,  271, 1039,  533, 1041,  772,  130,  363,  691,  776, 
          371,  372,   44,  310,   10,   44,  699,  378,   10,  373, 
           10,  317,  383,  310,    0,  328,  373,  794,   44,  316, 
          797,  318,  335,  336,   10, 1072,  350,  351,  373,  995, 
          836,  808,  809,  810,  264,   91,  373,  376,  377,  263, 
          264,  938,  776,  349,  780,  749,  279,  353,  354,   91, 
          856,  787,  349,  347,  279,   61,  341,   59,  341,   59, 
          130,  264,  342,  262,  841,  842,  269,  878,  271,  267, 
          264,  328,  342,   59,   44, 1041,  812,  305,  335,  336, 
           44,  797,  363,  363,  263,  264,  383,   48,  362,  866, 
          269,   44,  785,   32,    2,  328,  753,  637,  638,  279, 
          234,  878,  335,  336,   32,   61, 1072,  387,  648,  342, 
           44,  267,  338,   44,  342,  892,  893,  342,  338,  264, 
         1017,  980,  338,  900,  269,  338,  264,  904,  425,  426, 
          338,  359,  866,  269,  431,  264,  676,  794,  338,   47, 
           48,  338,  835,   10,  848,    0,   10,  375,   10,  446, 
          328,  808,  809,  810,  338,   10,  696,  335,  336,  363, 
          457,   44,  342,   10,  234,  279,  900,  306,  465,  308, 
          309,  310,  311,  306,  734,  735,  310,   61,  955,  267, 
          306,   44,  316,  339,  318,  842,  371,  372,  373,  966, 
           44,   44,   59,  378,  306,   59,   10,   59,  306,   44, 
          308,  309,  310,  311,   59,  305,  983,   58,  985,  506, 
          338,  751,   59,  753,  991,  755,  756,  757,  345, 1025, 
          517,  955,  762,  763,   91,   10,  919,   91,  342,   91, 
          264,  264,  772,  339,  263,  892,  940,  279,  535,  943, 
          310,  945,  342,  947,   91,   59,  316,  904,  318,  383, 
          264,  339,  338,  264,  794,   44, 1033,  797, 1064,  359, 
          363, 1038,  264,  263,  264,   44,  268,  269,  808,  809, 
          810, 1033,  328,  279,   59,  375,  262,  263,  264,  335, 
          336,  267,  268,  269,   44,  271,  328,   44,   44,   44, 
          424,  425, 1034,  335,  336,  281,   91,  431,  359,  341, 
           61,  841,  842,  363,  264, 1034,   91,  293,  294,  295, 
          296,  297,  446,  383,   59,   10,  264,  264,  264,    0, 
          281,  282,  283,  457,   44,  286,  983,  624,  985,   10, 
          749,  465,  342,   44,  991,  264,  342,   44,  878,  363, 
           44,  638,   44, 1047,   44, 1049,  359, 1051,  310,   44, 
         1054,  648,  892,  893,  363,  425,  342,  264,  310,   44, 
           44,  431,   44,   44,  904,  925,   61,  371,  372,  373, 
          930,  931,  506,  933,  378,  935,  446,  363,   59,   44, 
           44, 1038,   44,  517, 1088,   91,  264,  457,   44,  264, 
           44,   91,  300,   44,  387,  465,   91,   10,   44,  696, 
          267,  387,  271,  267,  701,  267,   91,  262,  263,  264, 
          345,  341,  709,  268,  269,  271,  271,  342,  280,  306, 
          267,  308,  309,  310,  311,  844,  966,  846,  345,  345, 
          363,   44,  345,  280,   58,   91,  506,  363,  293,  294, 
          295,  296,  297,  983,  264,  985,   59,  517,  262,  263, 
          264,  991,   91,  340,  751,  269,  753,  285,  755,   70, 
           54,  328,  372,  424,  328,   91,  328,  428,  335,  336, 
            5,  335,  336,  335,  336,    6,   91,  341,   91,  776, 
          517,  328,  267, 1043, 1044, 1045, 1046,  342,  335,  336, 
         1036,  612,  899, 1033,  341,  280,  749,  794, 1038,  848, 
          362,  844,  945,  943,  638,  768,  467,   14,  363,  417, 
         1035,  808,  809,  810,  648,  362,   44, 1059,  830,   -1, 
           -1,  482,  483,   -1,  943,  486, 1086,   -1,  947,   -1, 
           -1,   -1,  387,  328,   -1,   -1,  444,   -1,   -1,   -1, 
          335,  336,   -1,  328,  841,  842,   -1,    0,   -1,   -1, 
          335,  336,  460,  371,  372,  373,  341,   10,   -1,   -1, 
          378,  522,  696,   91,  525,   -1,  474,   -1,  638,  866, 
          388,   -1,   -1,    9,   10,   11,   -1,  362,  648,  279, 
          280,  262,  263,  264,  279,  280, 1005,  268,  269,   -1, 
          271,   44,   -1,   -1,  279,  892,  893,   -1,  506,   10, 
           -1,   -1,   -1,  900,   -1,   58,   59,  904,   -1,   -1, 
           63,   -1,  293,  294,  295,  296,  297,  751,   -1,  753, 
           -1,  755,  328,  279,   -1,  533,  696,   -1,  328,  335, 
          336,   -1, 1051,  328,   -1,  335,  336,   -1,  772,   -1, 
          335,  336,   -1,  328,   -1,   -1,  341,  342,   59,   -1, 
          335,  336,  560,  561,  615,   -1,   -1,   -1,  955,   -1, 
          794,  342,  362,   -1,  749,   -1,   -1,  362,   -1,  966, 
           -1,   -1,  328,   -1,  808,  809,  810,   -1,   -1,  335, 
          336,  751,  363,  753,   -1,  755,  983,  595,  985,  328, 
           -1,  652,  128,   -1,  991,   10,  335,  336,   -1,  607, 
           -1,   -1,  328,   -1,  612,   -1,  387,  841,  842,  335, 
          336,   -1,   -1,  328,   -1,  328,  677,   -1,  679,    0, 
          335,  336,  335,  336,  794,  686,  687,   -1,  341,   10, 
          371,  372,  373,   -1,   -1,   -1,  644,  378,  808,  809, 
          810, 1038,   -1,  317,  652,  653,  359,  655,   -1,   10, 
           10,  279,   -1,   -1,   -1,  663,   -1,   -1,  892,  893, 
           15,   16,   -1,   -1,   19,   -1,  900,   -1,   -1,   10, 
          904,  841,  842,   -1,   -1,  349,   -1,   -1,   59,  353, 
          354,  355,  356,  744,   -1,   -1,   41,   42,  696,   -1, 
           -1,   -1,   47,   48,   -1,   50,   51,   -1,   59,   59, 
          328,   -1,   -1,   44,   -1,   -1,   -1,  335,  336,  262, 
          263,  264,  773,   -1,  267,  268,  269,   -1,  271,   -1, 
           61,   -1,  892,  893,   -1,  261,  262,   44,  281,  282, 
           91,   -1,  966,   -1,  904,   -1,   -1,  290,  291,   10, 
          293,  294,  295,  296,  297,   -1,  807,   -1,   -1,  983, 
           91,  985,  305,   -1,   10,  940,   -1,  991,  943,   -1, 
          945,  306,  947,  308,  309,  310,  311,  303,   -1,  305, 
          778,  324,  325,   44,   91,   -1,  329,  330,  371,  372, 
          373,   -1,   -1,   -1,  792,  378,  339,  795,   44,  342, 
           61,  327,  345,  854,  347,  306,  966,  308,  309,  310, 
          311,  312,   -1,   -1, 1038,   61,  317,   -1,   10,   -1, 
          363,   10,   -1,  983,   -1,  985,   -1,  328,   -1,   -1, 
           91,  991,  375,   -1,   -1,  336,   44,  888,   -1,  340, 
          341,   44,   -1,   -1,  387,   91,   10,  348,  349,  350, 
          351,   -1,   44,  379,  380,  381,  382,   -1,  384,  385, 
           -1,  371,  372,  373,   -1,  916,   -1,   59,  378,   -1, 
           59,   44, 1047,   -1, 1049,   -1, 1051,   -1, 1038, 1054, 
          371,  372,  373,   91,   -1,  386,   -1,  378,   91,   -1, 
           -1,  262,  263,  264,   -1,   59,  267,  268,  269,   91, 
          271,  306,   91,  308,  309,  310,  311,  312,   -1,   -1, 
          281,   -1,  317, 1088,  912,   -1,  267,   -1,   91,  290, 
          291,  447,  293,  294,  295,  296,  297,   91,  379,  380, 
          381,  382,    0,   -1,   -1,  340,  281,  282,  283,  284, 
           -1,  286,   10,  348,  349,  350,  351,   -1,  279,  280, 
           -1,   -1,  297,   -1,  299,  300,  306,  328,  308,  309, 
          310,  311,  312,  308,  335,  336,   -1,  317,  371,  372, 
          373,  342,  279,   -1,   -1,  378,   44,  328,  504,   -1, 
           -1,  386,   -1,   -1,  335,  336,   -1,  371,  372,  373, 
          340,   59,  363,   61,  378,   63,   -1,  328,  348,  349, 
          350,  351,   -1,   -1,  335,  336,  262,  263,  264,   -1, 
          341,  342,  268,  269,   -1,  271,  387,   -1,  279,  280, 
           -1,  328,   -1,   91,   -1,   -1,   -1,   -1,  335,  336, 
           -1,  362,   -1,  279,  280, 1033,  386,   -1, 1036,   -1, 
           -1,  386,  387,  388,  389,  390,  391,  392,  393,  394, 
          395,  396,  397,  398,  399,  400,  401,  402,  403,  404, 
          405,  406,  407,  408,  409,  410,  411,  328,   -1,   -1, 
           -1,  279,  417,   -1,  335,  336,  279,  422,  267,  424, 
          341,  342,  328,  428,  429,   -1,  342,   -1,  280,  335, 
          336,  280,   -1,   44,   -1,  341,  342,   -1,    0,  444, 
           -1,  362,   -1,  267,   -1,   -1,  279,  363,   10,   -1, 
           -1,  637,   -1,   44,   -1,  460,  362,  462,  463,  464, 
          328,   -1,  467,   -1,   -1,  328,   -1,  335,  336,  474, 
           -1,  387,  335,  336,   -1,   -1,  328,  482,  483,  328, 
           91,  486,   44,  335,  336,   -1,  335,  336,   -1,  341, 
          676,   -1,  341,   -1,   -1,  328,   58,   59,   -1,   -1, 
           91,   63,  335,  336,  328,  510,  511,  359,   -1,   -1, 
          362,  335,  336,  362,   -1,   -1,   -1,  522,   -1,   -1, 
          525,   10,  293,  294,  295,  296,  297,   -1,  533,  257, 
          258,  259,   -1,  261,  262,  263,  264,  265,  266,   -1, 
          268,  269,  270,  271,  272,  273,  274,  275,  276,  277, 
          278,   -1,  280,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,  290,  291,  292,  293,  294,  295,  296,  297, 
           59,  299,   44,   -1,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,  312,  313,  314,  315,  316,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   44,   -1,  333,  334,  335,  336,  337, 
          615,  797,  340,  341,  342,  343,  344,   -1,  346,   91, 
          348,  349,  350,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,  362,  363,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,  652,  376,  377, 
           91,  379,  380,  381,  382,    0,  384,  385,  386,  387, 
           -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,  279,  262, 
          263,  264,  677,   -1,  679,  268,  269,   -1,  271,  857, 
           -1,  686,  687,  861,   -1,   -1,   -1,   -1,  279,  280, 
           -1,   -1,   -1,   -1,  362,   -1,  364,   -1,  366,   44, 
          262,  263,  264,   -1,  372,  267,  268,  269,   -1,  271, 
           -1,   -1,   -1,   58,   59,   -1,   61,  328,   63,  281, 
          282,   -1,   -1,   -1,  335,  336,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,  328,   -1,  744, 
           -1,   -1,   -1,  305,  335,  336,   91,   -1,   -1,  342, 
          341,  756,  757,   -1,   -1,   -1,   -1,  762,  763,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  771,  772,  773,   -1, 
          363,  362,  777,   -1,   -1,   -1,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,    0,   -1,   -1, 
           -1,   -1,   -1,   -1,  387,  800,  801,   10,  803,  804, 
           -1,  363,  807,   -1,   -1,   -1,   -1,  279,  280,   -1, 
           -1,   -1,  480,  375,   -1,   -1,   -1,  306,   -1,  308, 
          309,  310,  311,  312,   -1,  387,   -1,   -1,  317,   -1, 
           -1,   44,   -1, 1011,   -1,   -1, 1014, 1015,  279,  328, 
         1018, 1019,   -1,   -1,   -1,   58,   59,  336,   61,  854, 
           63,  340,  341,   -1,   -1,   -1,  328,   -1,   -1,  348, 
          349,  350,  351,  335,  336,   -1,  871,   -1,   -1,  341, 
           -1,   -1,   -1,  878,   -1,   -1,   -1,   -1,   91,   -1, 
           -1,  549,   -1,  888,   -1,   -1,   -1,  328,  556,   -1, 
          362,   -1,   -1,   -1,  335,  336,   -1,  386,   -1,   -1, 
         1078, 1079, 1080, 1081,  909,   -1,   -1,   10,   -1,   -1, 
           -1,  916,  257,  258,  259, 1093,  261,  262,  263,  264, 
          265,  266,  267,  268,  269,  270,  271,  272,  273,  274, 
          275,  276,  277,  278,   -1,  280,  281,  282,  283,  284, 
          285,  286,  287,  288,  289,  290,  291,  292,  293,  294, 
          295,  296,  297,   -1,  299,   -1,   59,  302,  303,  304, 
          305,  306,  307,  308,  309,  310,  311,  312,  313,  314, 
          315,  316,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   -1,   -1,  333,  334, 
          335,  336,  337,  338,  339,  340,  341,  342,  343,  344, 
          345,  346,  347,  348,  349,  350,  351,  352,  353,  354, 
          355,  356,  357,  358,  359,  360,  361,  362,  363,   44, 
          365,  366,  367,  368,  369,  370,   -1,   -1, 1033,  374, 
          375,  376,  377,   -1,  379,  380,  381,  382,   10,  384, 
          385,  386,  387,   -1,  257,  258,  259,   -1,  261,  262, 
          263,  264,  265,  266,  267,  268,  269,  270,  271,  272, 
          273,  274,  275,  276,  277,  278,   91,  280,  281,  282, 
          283,  284,  285,  286,  287,  288,  289,  290,  291,  292, 
          293,  294,  295,  296,  297,   -1,  299,   59,   -1,  302, 
          303,  304,  305,  306,  307,  308,  309,  310,  311,  312, 
          313,  314,  315,  316,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   44,   -1, 
          333,  334,  335,  336,  337,  338,  339,  340,  341,  342, 
          343,  344,  345,  346,  347,  348,  349,  350,  351,  352, 
          353,  354,  355,  356,  357,  358,  359,  360,  361,  362, 
          363,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
            0,  374,  375,  376,  377,   91,  379,  380,  381,  382, 
           10,  384,  385,  386,  387,   -1,   -1,   -1,  306,   -1, 
          308,  309,  310,  311,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  306,   -1,  308, 
          309,  310,  311,  306,   44,  308,  309,  310,  311,  312, 
           -1,   -1,  340,   -1,  317,   -1,   -1,   -1,   58,   59, 
          348,   61,   -1,   63,  306,  328,  308,  309,  310,  311, 
          312,  340,   -1,  336,   -1,  317,   -1,  340,  341,  348, 
           -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,   -1, 
           -1,   91,   -1,   -1,  279,   -1,   -1,   -1,  340,  341, 
           -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351, 
          317,  306,   -1,  308,  309,  310,  311,  312,   -1,   -1, 
           -1,   -1,  317,  386,   -1,   -1,  333,  334,   -1,   -1, 
           -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   10,  349,  328,  386,  340,  353,  354,  355,  356, 
          335,  336,   -1,  348,  349,  350,  351,   -1,   -1,   -1, 
           -1,  306,   -1,  308,  309,  310,  311,  312,   -1,   -1, 
           -1,   -1,  317,   -1,  306,   44,  308,  309,  310,  311, 
          312,   -1,   -1,   -1,   -1,  317,   -1,   -1,   -1,   58, 
           59,  386,   61,  279,   63,  340,  328,  317,   -1,   -1, 
           -1,   -1,   -1,   -1,  336,  350,  351,   -1,  340,  341, 
           -1,   -1,   -1,  333,  334,   -1,  348,  349,  350,  351, 
           -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,  349, 
           -1,  351,   -1,  353,  354,  355,  356,   -1,   -1,   -1, 
           -1,  386,  328,   -1,   -1,   -1,   -1,   -1,   -1,  335, 
          336,   10,   -1,   -1,  386,   -1,   -1,  257,  258,  259, 
           -1,  261,  262,  263,  264,  265,  266,  267,  268,  269, 
          270,  271,  272,  273,  274,  275,  276,  277,  278,   -1, 
          280,  281,  282,  283,  284,  285,  286,  287,  288,  289, 
          290,  291,  292,  293,  294,  295,  296,  297,   -1,  299, 
           59,   -1,  302,  303,  304,  305,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   44,   -1,  333,  334,  335,  336,  337,   -1,  339, 
          340,  341,  342,  343,  344,  345,  346,  347,  348,  349, 
          350,  351,  352,  353,  354,  355,  356,  357,  358,  359, 
          360,  361,  362,  363,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,  375,  376,  377,   91,  379, 
          380,  381,  382,   -1,  384,  385,  386,  387,  257,  258, 
          259,   -1,  261,  262,  263,  264,  265,  266,  267,  268, 
          269,  270,  271,  272,  273,  274,  275,  276,  277,  278, 
           -1,  280,  281,  282,  283,  284,  285,  286,  287,  288, 
          289,  290,  291,  292,  293,  294,  295,  296,  297,   -1, 
          299,   -1,   -1,  302,  303,  304,  305,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,   -1,   -1,  333,  334,  335,  336,  337,   -1, 
          339,  340,  341,  342,  343,  344,  345,  346,  347,  348, 
          349,  350,  351,  352,  353,  354,  355,  356,  357,  358, 
          359,  360,  361,  362,  363,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,    0,  374,  375,  376,  377,   -1, 
          379,  380,  381,  382,   10,  384,  385,  386,  387,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  306,   44,  308, 
          309,  310,  311,  312,   -1,   -1,   -1,   -1,  317,   -1, 
           -1,   -1,   58,   59,   -1,   61,  279,   63,   -1,  328, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  336,   -1,   -1, 
           -1,  340,  341,   -1,   -1,   -1,   -1,   -1,   -1,  348, 
          349,  350,  351,   -1,   -1,   91,   -1,   -1,   -1,   -1, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,   -1,   -1,   -1,   -1,  328,  333,  334,   -1,   -1, 
           -1,   -1,  335,  336,   -1,   -1,   -1,  386,   -1,   -1, 
           -1,   -1,  349,   -1,  351,    0,  353,  354,  355,  356, 
          357,  358,  359,   -1,  361,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  306,  307,   -1, 
           -1,  310,   -1,   -1,   -1,  314,  315,   -1,  317,  318, 
          319,  320,  321,  322,  323,   -1,   -1,  326,  327,   44, 
           -1,   -1,  331,  332,  333,  334,   -1,   -1,   -1,   -1, 
           -1,   -1,  341,   58,   59,   -1,   -1,  317,   63,  348, 
          349,   -1,  351,  352,  353,  354,  355,  356,  357,  358, 
          359,  360,  361,  333,  334,  364,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,  349, 
           -1,  351,   -1,  353,  354,  355,  356,  386,   -1,  359, 
           -1,  361,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1, 
           -1,  257,  258,  259,   -1,  261,  262,  263,  264,  265, 
          266,  267,  268,  269,  270,  271,  272,  273,  274,  275, 
          276,  277,  278,   -1,  280,  281,  282,  283,  284,  285, 
          286,  287,  288,  289,  290,  291,  292,  293,  294,  295, 
          296,  297,   -1,  299,   59,   -1,  302,  303,  304,  305, 
          306,  307,  308,  309,  310,  311,  312,  313,  314,  315, 
          316,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   -1,   -1,  333,  334,  335, 
          336,  337,   -1,  339,  340,  341,  342,  343,  344,  345, 
          346,  347,  348,  349,  350,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,  362,  363,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,  375, 
          376,  377,   -1,  379,  380,  381,  382,   -1,  384,  385, 
          386,  387,  257,  258,  259,   -1,  261,  262,  263,  264, 
          265,  266,  267,  268,  269,  270,  271,  272,  273,  274, 
          275,  276,  277,  278,   -1,  280,  281,  282,  283,  284, 
          285,  286,  287,  288,  289,  290,  291,  292,  293,  294, 
          295,  296,  297,   -1,  299,   -1,   -1,  302,  303,  304, 
          305,  306,  307,  308,  309,  310,  311,  312,  313,  314, 
          315,  316,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,   -1,   -1,  333,  334, 
          335,  336,  337,   -1,  339,  340,  341,  342,  343,  344, 
          345,  346,  347,  348,  349,  350,  351,  352,  353,  354, 
          355,  356,  357,  358,  359,  360,  361,  362,  363,   -1, 
          365,  366,  367,  368,  369,  370,   -1,   -1,    0,  374, 
          375,  376,  377,   -1,  379,  380,  381,  382,   10,  384, 
          385,  386,  387,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  306,   44,  308,  309,  310,  311,  312,   -1,   -1, 
           -1,   -1,  317,   -1,   -1,   -1,   58,   59,   -1,   -1, 
           -1,   63,   -1,  328,  317,  318,  319,  320,  321,  322, 
          323,  336,   -1,  326,  327,  340,  341,   -1,   -1,   -1, 
          333,  334,   -1,  348,  349,  350,  351,   -1,   -1,   91, 
           -1,   -1,   -1,   -1,   -1,   -1,  349,   -1,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,   -1, 
           -1,   -1,   -1,   -1,   -1,  257,  258,  259,   -1,  261, 
           -1,  386,   -1,  265,  266,   -1,   -1,   -1,  270,    0, 
          272,  273,  274,  275,  276,  277,  278,   -1,   -1,   10, 
           -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
           -1,  313,   -1,   44,  316,   -1,   -1,  317,   -1,   -1, 
           -1,   -1,  322,  323,   -1,   -1,   -1,   58,   59,   -1, 
           -1,  333,   63,  333,  334,  337,   -1,   -1,  340,   -1, 
           -1,  343,  344,   -1,  346,   -1,   -1,   -1,   -1,  349, 
           -1,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           91,  361,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,  384,  385,   -1,   -1,   -1,   -1,   -1,   10, 
           -1,   -1,   -1,   -1,   -1,  257,  258,  259,   -1,  261, 
          262,  263,  264,  265,  266,  267,  268,  269,  270,  271, 
          272,  273,  274,  275,  276,  277,  278,   -1,  280,  281, 
          282,  283,  284,  285,  286,  287,  288,  289,  290,  291, 
          292,  293,  294,  295,  296,  297,   -1,  299,   59,   -1, 
          302,  303,  304,  305,  306,  307,  308,  309,  310,  311, 
          312,  313,  314,  315,  316,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,  337,   -1,  339,  340,  341, 
          342,  343,  344,  345,  346,  347,  348,  349,  350,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
          362,  363,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,  375,  376,  377,   -1,  379,  380,  381, 
          382,   -1,  384,  385,  386,  387,  257,  258,  259,   -1, 
          261,  262,  263,  264,  265,  266,  267,  268,  269,  270, 
          271,  272,  273,  274,  275,  276,  277,  278,   -1,  280, 
          281,  282,  283,  284,  285,  286,  287,  288,  289,  290, 
          291,  292,  293,  294,  295,  296,  297,   63,  299,   -1, 
           -1,  302,  303,  304,  305,  306,  307,  308,  309,  310, 
          311,  312,  313,  314,  315,  316,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  328,  329,  330, 
           -1,   -1,  333,  334,  335,  336,  337,   -1,  339,  340, 
          341,  342,  343,  344,  345,  346,  347,  348,  349,  350, 
          351,  352,  353,  354,  355,  356,  357,  358,  359,  360, 
          361,  362,  363,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,    0,  374,  375,  376,  377,   -1,  379,  380, 
          381,  382,   10,  384,  385,  386,  387,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  306,   44,  308,  309,  310, 
          311,  312,   -1,   -1,   -1,   -1,  317,   -1,   -1,   -1, 
           58,   59,   -1,   61,   -1,   63,   -1,  328,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  336,   -1,   -1,   -1,  340, 
          341,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350, 
          351,   -1,   -1,   91,   -1,   -1,   -1,   -1,  317,  318, 
          319,  320,  321,  322,  323,  324,   -1,  326,  327,   -1, 
           -1,   -1,   -1,   -1,  333,  334,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  386,   -1,   -1,   -1,   -1, 
          349,   -1,  351,    0,  353,  354,  355,  356,  357,  358, 
          359,   -1,  361,   10,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   44,   -1,   -1, 
           -1,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,   59,  329,  330,   -1,   63,  333,  334,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  349,   -1,  351,   -1,  353,  354,  355, 
          356,  357,  358,  359,   91,  361,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257, 
          258,  259,   -1,  261,  262,  263,  264,  265,  266,  267, 
          268,  269,  270,  271,  272,  273,  274,  275,  276,  277, 
          278,   59,   -1,  281,  282,  283,  284,  285,  286,  287, 
          288,  289,  290,  291,  292,  293,  294,  295,  296,  297, 
           -1,  299,   -1,   -1,  302,  303,  304,  305,  306,  307, 
          308,  309,  310,  311,  312,  313,  314,  315,  316,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   44,   -1,  333,  334,  335,  336,  337, 
           -1,  339,  340,  341,  342,  343,  344,  345,  346,  347, 
          348,  349,  350,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,   -1,  363,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,  375,  376,  377, 
           91,  379,  380,  381,  382,   -1,  384,  385,  386,  387, 
          257,  258,  259,   -1,  261,  262,  263,  264,  265,  266, 
          267,  268,  269,  270,  271,  272,  273,  274,  275,  276, 
          277,  278,   63,  280,  281,  282,  283,  284,  285,  286, 
          287,  288,  289,  290,  291,  292,  293,  294,  295,  296, 
          297,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,  312,  313,  314,  315,  316, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,    0,   -1,  333,  334,  335,  336, 
          337,   -1,  339,  340,   10,  342,  343,  344,  345,  346, 
          347,  348,  349,  350,  351,  352,  353,  354,  355,  356, 
          357,  358,  359,  360,  361,  362,  363,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   44,  376, 
          377,   -1,  379,  380,  381,  382,   -1,  384,  385,  386, 
          387,   -1,   -1,   59,   -1,   -1,   -1,   63,  306,   -1, 
          308,  309,  310,  311,  312,   -1,   -1,   -1,   -1,  317, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          328,   -1,   -1,   -1,   -1,   91,   -1,   -1,  336,   -1, 
           -1,   -1,  340,  341,   -1,   -1,   -1,   -1,  279,  280, 
          348,  349,  350,  351,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,  386,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  328,   -1,   -1, 
           -1,   -1,   -1,   -1,  335,  336,   -1,   -1,   -1,   -1, 
          341,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,   -1,   -1, 
           -1,  362,   -1,   -1,   59,   -1,   -1,   -1,   63,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,   -1,  329,  330, 
           -1,   -1,  333,  334,   -1,   -1,   91,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,  349,   -1, 
          351,   -1,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  257,  258,  259,   -1,  261,  262,  263,  264,  265, 
          266,  267,  268,  269,  270,  271,  272,  273,  274,  275, 
          276,  277,  278,   59,  280,  281,  282,  283,  284,  285, 
          286,  287,  288,  289,  290,  291,  292,  293,  294,  295, 
          296,  297,   -1,  299,   -1,   -1,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,  312,  313,  314,  315, 
          316,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   44,   -1,  333,  334,  335, 
          336,  337,   -1,  339,  340,   -1,  342,  343,  344,  345, 
          346,  347,  348,  349,  350,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,  362,  363,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   91,  379,  380,  381,  382,   -1,  384,  385, 
          386,  387,  257,  258,  259,   -1,  261,  262,  263,  264, 
          265,  266,  267,  268,  269,  270,  271,  272,  273,  274, 
          275,  276,  277,  278,   63,   -1,  281,  282,  283,  284, 
          285,  286,  287,  288,  289,  290,  291,  292,  293,  294, 
          295,  296,  297,   -1,  299,   -1,   -1,  302,  303,  304, 
           -1,  306,  307,  308,  309,  310,  311,  312,  313,  314, 
          315,  316,  317,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,  328,  329,  330,    0,   -1,  333,  334, 
          335,  336,  337,   -1,  339,  340,   10,  342,  343,  344, 
          345,  346,  347,  348,  349,  350,  351,  352,  353,  354, 
          355,  356,  357,  358,  359,  360,  361,   -1,  363,   -1, 
          365,  366,  367,  368,  369,  370,   -1,   -1,   -1,  374, 
           44,  376,  377,   -1,  379,  380,  381,  382,   -1,  384, 
          385,  386,  387,   -1,   -1,   59,   -1,   61,   -1,   63, 
          306,   -1,  308,  309,  310,  311,  312,   -1,   -1,   -1, 
           -1,  317,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  328,   -1,   -1,   -1,   -1,   91,   -1,   -1, 
          336,   -1,   -1,   -1,  340,  341,   -1,   -1,   -1,   -1, 
          279,  280,  348,  349,  350,  351,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1, 
          386,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  328, 
           -1,   -1,   -1,   -1,   -1,   -1,  335,  336,   -1,   -1, 
           -1,   -1,  341,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  362,   -1,   -1,   59,   -1,   61,   -1, 
           63,   -1,   -1,   -1,   -1,   -1,   -1,   10,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,   -1, 
          329,  330,   -1,   -1,  333,  334,   -1,   -1,   91,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          349,   44,  351,   -1,  353,  354,  355,  356,  357,  358, 
          359,   -1,  361,   -1,   -1,   -1,   -1,   -1,   61,   -1, 
           -1,   -1,   -1,  257,  258,  259,   -1,  261,  262,  263, 
          264,  265,  266,   -1,  268,  269,  270,  271,  272,  273, 
          274,  275,  276,  277,  278,   -1,  280,   -1,   91,  283, 
          284,  285,  286,  287,  288,  289,  290,  291,  292,  293, 
          294,  295,  296,  297,   -1,  299,   44,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,  312,  313, 
          314,  315,  316,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333, 
          334,  335,  336,  337,  338,   -1,  340,  341,  342,  343, 
          344,   -1,  346,   91,  348,  349,  350,  351,  352,  353, 
          354,  355,  356,  357,  358,  359,  360,  361,  362,  363, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
          384,  385,  386,  387,  257,  258,  259,   -1,  261,  262, 
          263,  264,  265,  266,   -1,  268,  269,  270,  271,  272, 
          273,  274,  275,  276,  277,  278,   -1,  280,  749,   -1, 
          283,  284,  285,  286,  287,  288,  289,  290,  291,  292, 
          293,  294,  295,  296,  297,   -1,  299,   -1,   -1,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,  312, 
          313,  314,  315,  316,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1, 
          333,  334,  335,  336,  337,  338,  279,  340,  341,  342, 
          343,  344,   -1,  346,   -1,  348,  349,  350,  351,  352, 
          353,  354,  355,  356,  357,  358,  359,  360,  361,  362, 
          363,   -1,  365,  366,  367,  368,  369,  370,  839,   -1, 
            0,  374,   -1,  376,  377,  846,  379,  380,  381,  382, 
           10,  384,  385,  386,  387,  328,   -1,   -1,   -1,   -1, 
           -1,   -1,  335,  336,   -1,   -1,   -1,   -1,  341,  342, 
           -1,  279,  280,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59, 
           -1,   61,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          328,   -1,   -1,   -1,   -1,   -1,   -1,  335,  336,   -1, 
           -1,   91,   -1,  341,   -1,   -1,   -1,   -1,   -1,  940, 
           -1,   -1,   -1,   -1,   -1,   -1,  947,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  362,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   10,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1, 1003,   -1, 1005,   -1, 1007,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   44,   -1,   -1,   44,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           59,   -1,   61,   -1,   63,   61,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 1049,   -1, 
         1051,   -1,   -1, 1054,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   91,   -1,   -1,   91,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1, 1074,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1, 1088,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,  258,  259, 
           -1,  261,  262,  263,  264,  265,  266,   -1,  268,  269, 
          270,  271,  272,  273,  274,  275,  276,  277,  278,   -1, 
          280,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
          290,  291,  292,  293,  294,  295,  296,  297,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,  312,  313,  314,  315,  316,  317,  318,  319, 
      };
   }

   private static final short[] yyCheck2() {
      return new short[] {

          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   -1,   -1,  333,  334,  335,  336,  337,   -1,   -1, 
          340,  341,  342,  343,  344,   -1,  346,   -1,  348,  349, 
          350,  351,  352,  353,  354,  355,  356,  357,  358,  359, 
          360,  361,  362,  363,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,    0,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   10,  384,  385,  386,  387,  257,  258, 
          259,   -1,  261,  262,  263,  264,  265,  266,   -1,  268, 
          269,  270,  271,  272,  273,  274,  275,  276,  277,  278, 
           -1,  280,   -1,  279,  283,  284,  285,  286,  287,  288, 
          289,  290,  291,  292,  293,  294,  295,  296,  297,   -1, 
          299,   -1,   59,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,  312,  313,  314,  315,  316,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
          329,  330,  328,   -1,  333,  334,  335,  336,  337,  335, 
          336,  340,  341,  342,  343,  344,  342,  346,   -1,  348, 
          349,  350,  351,  352,  353,  354,  355,  356,  357,  358, 
          359,  360,  361,  362,  363,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,    0,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   10,  384,  385,  386,  387,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  306,  307,   -1,   -1, 
          310,   -1,   -1,   -1,  314,  315,   -1,  317,  318,  319, 
          320,  321,  322,  323,   -1,   -1,  326,  327,   44,   -1, 
           -1,  331,  332,  333,  334,   -1,   -1,   -1,   -1,   -1, 
           -1,  341,   -1,   59,   -1,   61,   -1,   63,  348,  349, 
           -1,  351,  352,  353,  354,  355,  356,  357,  358,  359, 
          360,  361,   -1,   -1,  364,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  386,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,   -1,   -1,   10,   -1,   -1,   -1,   -1, 
          257,  258,  259,   -1,  261,  262,  263,  264,  265,  266, 
          267,  268,  269,  270,  271,  272,  273,  274,  275,  276, 
          277,  278,   -1,  280,  281,   -1,  283,  284,  285,  286, 
          287,  288,  289,  290,  291,  292,  293,  294,  295,  296, 
          297,   59,  299,   -1,   59,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,  312,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  328,   -1,   91,   -1,   -1,   91,   -1,  335,  336, 
          337,   -1,   -1,  340,   -1,  342,  343,  344,   -1,  346, 
           -1,  348,   -1,  350,   -1,  352,   -1,    0,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,  362,  363,   10,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,  384,  385,  386, 
          387,  257,  258,  259,   -1,  261,  262,  263,  264,  265, 
          266,   44,  268,  269,  270,  271,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   58,   59,  283,  284,  285, 
          286,  287,  288,  289,  290,  291,  292,  293,  294,  295, 
          296,  297,   -1,  299,   -1,   -1,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,  312,  313,  314,  315, 
          316,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   -1,   -1,  333,  334,  335, 
          336,  337,   -1,   -1,  340,  341,  342,  343,  344,   -1, 
          346,   -1,  348,  349,  350,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,    0,  363,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   10,  749,  374,  267, 
          376,  377,  267,  379,  380,  381,  382,   -1,  384,  385, 
          386,  387,  280,   -1,   -1,  280,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   58,   59,   -1,   61,   -1,   63, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          328,   -1,   -1,  328,   -1,   -1,   -1,  335,  336,   -1, 
          335,  336,   10,  341,   -1,   -1,  341,   91,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1, 
           -1,   -1,   -1,   -1,  362,   -1,  848,  362,   10,  851, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262, 
          263,  264,   -1,   -1,  267,  268,  269,   -1,  271,   -1, 
           -1,   59,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282, 
           -1,   -1,   44,   -1,   -1,   -1,   -1,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   58,   59,   -1,   -1, 
           -1,   63,  305,   91,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91, 
           -1,   -1,   -1,   -1,   -1,   -1,  339,   -1,  940,  342, 
           -1,  943,  345,  945,  347,  947,   44,   -1,   -1,   -1, 
           -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          363,   -1,   10,   61,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  375,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  387,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   91,   -1,   -1,   44,   -1,  262,  263, 
          264,   -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1, 
           58,   59,   -1,   61,   -1,   63,  280,  281,  282,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  305,   -1,   91,   -1, 1047,   -1, 1049,   -1, 1051, 
           -1,   -1, 1054,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   10,  333, 
          334,  335,  336,   -1,   -1,  339,   -1,   -1,  342,  267, 
           -1,  345,   -1,  347,   -1,  349, 1088,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,   -1,  361,  362,  363, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271, 
           -1,  375,   -1,   -1,   -1,   -1,   -1,   59,  280,  281, 
          282,   -1,   -1,  387,   -1,   -1,   -1,   10,  290,  291, 
           10,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
          328,   -1,   -1,  305,    0,   -1,   -1,  335,  336,   91, 
           -1,   -1,   -1,   -1,   10,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,   -1,   59,  339,   -1,   59, 
          342,  279,  280,  345,   -1,  347,   -1,  349,   44,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   -1,  361, 
          362,  363,   58,   59,   -1,   61,   -1,   63,   91,   -1, 
           -1,   91,   -1,  375,  262,  263,  264,   -1,   -1,  267, 
          268,  269,   -1,  271,   -1,  387,   -1,   -1,   -1,   -1, 
          328,   -1,   -1,  281,  282,   91,   -1,  335,  336,   -1, 
           -1,   -1,  290,  291,  342,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,    0,   -1, 
           -1,   -1,   -1,   -1,  362,   -1,   -1,   -1,   10,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   -1,   -1,  333,  334,  335,  336,   -1, 
          338,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           10,  349,   44,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,   -1,  363,   58,   59,   -1,   61, 
           -1,   63,   -1,   -1,   -1,  267,   -1,  375,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,  387, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91, 
           -1,   61,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
            0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   91,   -1,   -1,  267,   -1,  328,  267,   -1,   -1, 
           -1,   -1,   -1,  335,  336,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,   -1,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,   -1,   -1,   58,   59, 
           -1,   61,   -1,   63,   -1,  281,  282,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,  328,   -1,   10,  328,  305, 
           -1,   91,  335,  336,   -1,  335,  336,   -1,   -1,   -1, 
           -1,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   10,   -1,  333,  334,  335, 
          336,   -1,  338,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,   -1,  349,   -1,  351,   59,  353,  354,  355, 
          356,  357,  358,  359,   -1,  361,   -1,  363,   -1,   -1, 
           -1,   -1,   -1,    0,   -1,   -1,   -1,   10,   -1,  375, 
          262,  263,  264,   10,   59,  267,  268,  269,   91,  271, 
           -1,  387,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   91,   44,   -1,   10, 
           -1,   -1,   -1,  305,   -1,   -1,   59,   -1,   -1,  279, 
           -1,   58,   59,   -1,   -1,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   10, 
           -1,  333,  334,  335,  336,   -1,  338,  339,   91,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,  349,   59,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,  328,  361, 
           -1,  363,  262,  263,  264,  335,  336,  267,  268,  269, 
           -1,  271,  342,  375,   -1,   -1,   -1,   -1,   59,   -1, 
           91,  281,  282,   -1,   -1,  387,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  305,    0,   -1,   -1,   -1, 
           91,   -1,   -1,   -1,   -1,   -1,   10,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   -1,   -1,  333,  334,  335,  336,   -1,  338,  339, 
           -1,   -1,  342,   -1,  267,  345,   -1,  347,   10,  349, 
           44,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   -1,  363,   58,   59,   -1,   61,   -1,   63, 
           -1,   -1,  267,   -1,   -1,  375,   -1,   -1,   -1,   -1, 
           -1,   -1,   44,   -1,   -1,   -1,   -1,  387,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   61, 
           -1,   -1,   -1,   -1,   -1,  328,   -1,   -1,    0,   -1, 
           -1,   -1,  335,  336,  267,  262,  263,  264,   10,   -1, 
          267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   91, 
           -1,   -1,   -1,  328,  281,  282,   -1,   -1,   -1,   -1, 
          335,  336,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   44,   -1,   -1,   -1,  267,   -1,  305,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   -1,   61, 
           -1,   63,   -1,   -1,   -1,  328,   10,   -1,   -1,   -1, 
           -1,   -1,  335,  336,   -1,   -1,  267,   -1,   -1,   -1, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   91, 
          347,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1, 
           44,   -1,   -1,   -1,   -1,   -1,  363,  328,   -1,   -1, 
           -1,   -1,    0,   -1,  335,  336,   -1,   61,  375,   -1, 
           -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,   44, 
          387,   -1,   -1,   -1,   -1,   -1,   -1,  328,   -1,   -1, 
           -1,   -1,   -1,   -1,  335,  336,   61,   91,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   44,   -1,  262,  263, 
          264,   -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1, 
           58,   59,   -1,   61,   -1,   63,   91,  281,  282,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  305,   -1,   91,   -1,   -1,   -1,  279,   -1,   -1, 
           -1,   -1,   -1,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333, 
          334,  335,  336,   -1,  338,  339,   -1,   -1,  342,   -1, 
           -1,  345,   -1,  347,   -1,  349,   -1,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,  328,  361,   -1,  363, 
          262,  263,  264,  335,  336,  267,  268,  269,   -1,  271, 
          342,  375,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   -1,  387,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  305,    0,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   10,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,  279,  338,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,  349,   44,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   -1,  361, 
           -1,  363,   58,   59,  279,   61,   -1,   63,   -1,   -1, 
           -1,   -1,   -1,  375,  262,  263,  264,   -1,   -1,  267, 
          268,  269,   -1,  271,  328,  387,   -1,   -1,   -1,   -1, 
           -1,  335,  336,  281,  282,   91,   -1,   -1,  342,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,  328,   -1,   -1,   -1,  305,    0,   -1, 
          335,  336,   -1,   -1,   -1,   -1,   -1,  342,   10,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   -1,   -1,  333,  334,  335,  336,   -1, 
          338,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           10,  349,   44,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,   -1,  363,   58,   59,   -1,   61, 
           -1,   63,   -1,   -1,   -1,   -1,   -1,  375,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,  387, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91, 
           -1,   61,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
            0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,   -1,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,   -1,   -1,   58,   59, 
           -1,   61,   -1,   63,   -1,  281,  282,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305, 
           -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   -1,   -1,  333,  334,  335, 
          336,   -1,  338,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,   -1,  349,   -1,  351,   -1,  353,  354,  355, 
          356,  357,  358,  359,   -1,  361,   -1,  363,   -1,   -1, 
           -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,  375, 
          262,  263,  264,   10,   -1,  267,  268,  269,   -1,  271, 
           -1,  387,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   44,   -1,   -1, 
           -1,   -1,   -1,  305,   -1,   -1,   -1,   -1,   -1,  279, 
           -1,   58,   59,   -1,   -1,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,   -1,  338,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,  349,   -1,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,  328,  361, 
           -1,  363,  262,  263,  264,  335,  336,  267,  268,  269, 
           -1,  271,  342,  375,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  281,  282,   -1,   -1,  387,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  305,    0,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   10,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   -1,   -1,  333,  334,  335,  336,   -1,  338,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   10,  349, 
           44,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   -1,  363,   58,   59,   -1,   -1,   -1,   63, 
           -1,   -1,   -1,   -1,   -1,  375,   -1,   -1,   -1,   -1, 
           -1,   -1,   44,   -1,   -1,   -1,   -1,  387,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   61, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1, 
           -1,   -1,   -1,   -1,   -1,  262,  263,  264,   10,   -1, 
          267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   91, 
           -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   44,   -1,   -1,   -1,   -1,   -1,  305,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   -1,   61, 
           -1,   63,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   91, 
          347,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           44,   -1,   -1,   -1,   -1,   -1,  363,   -1,   -1,   -1, 
           -1,   -1,    0,   -1,   -1,   -1,   -1,   61,  375,   -1, 
           -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          387,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   44,   -1,  262,  263, 
          264,   -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1, 
           58,   59,   -1,   61,   -1,   63,  280,  281,  282,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  305,   -1,   91,   -1,   -1,   -1,  279,   -1,   -1, 
           -1,   -1,   -1,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333, 
          334,  335,  336,   -1,   -1,  339,   -1,   -1,  342,   -1, 
           -1,  345,   -1,  347,   -1,  349,   -1,  351,   -1,  353, 
          354,  355,  356,  357,  358,  359,  328,  361,  362,  363, 
          262,  263,  264,  335,  336,  267,  268,  269,   -1,  271, 
          342,  375,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   -1,  387,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  305,    0,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   10,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,  279,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,  349,   44,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   -1,  361, 
           -1,  363,   58,   59,   -1,   -1,   -1,   63,   -1,   -1, 
           -1,   -1,   -1,  375,  262,  263,  264,   -1,   -1,  267, 
          268,  269,   -1,  271,  328,  387,   -1,   -1,   -1,   -1, 
           -1,  335,  336,  281,  282,   91,   -1,   -1,  342,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,    0,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   -1,   -1,  333,  334,  335,  336,   -1, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           -1,  349,   44,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,   -1,  363,   58,   59,   -1,   -1, 
           -1,   63,   -1,   -1,   -1,   -1,   -1,  375,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  387, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
            0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,   -1,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,   -1,   -1,   58,   59, 
           -1,   -1,   -1,   63,   -1,  281,  282,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305, 
           -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,   -1,   -1,  333,  334,  335, 
          336,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,   -1,  349,   -1,  351,   -1,  353,  354,  355, 
          356,  357,  358,  359,   -1,  361,   -1,  363,   -1,   -1, 
           -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,  375, 
          262,  263,  264,   10,   -1,  267,  268,  269,   -1,  271, 
           -1,  387,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   44,   -1,   -1, 
           -1,   -1,   -1,  305,   -1,   -1,   -1,   10,   -1,   -1, 
           -1,   58,   59,   -1,   -1,  317,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,   -1,   -1,  339,   -1,   -1, 
          342,   44,   -1,  345,   -1,  347,   -1,  349,   -1,  351, 
            0,  353,  354,  355,  356,  357,  358,  359,   61,  361, 
           10,  363,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           -1,  271,   -1,  375,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  281,  282,   -1,   -1,  387,   -1,   -1,   91,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  305,    0,   -1,   -1,   59, 
           -1,   -1,   -1,   -1,   -1,   -1,   10,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   -1,   -1,  333,  334,  335,  336,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   10,  349, 
           44,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   -1,  363,   58,   59,   -1,   -1,   -1,   63, 
           -1,   -1,   -1,   -1,   -1,  375,   -1,   -1,   -1,   -1, 
           -1,   -1,   44,   -1,   -1,   -1,   -1,  387,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   61, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1, 
           -1,   -1,   -1,   -1,   -1,  262,  263,  264,   10,   -1, 
          267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   91, 
           -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   44,   -1,   -1,   -1,   -1,   -1,  305,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   -1,   -1, 
           -1,   63,   -1,   -1,   -1,   10,  279,  280,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   91, 
          347,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44, 
           -1,   -1,  262,  263,  264,   -1,  363,    0,  268,  269, 
           -1,  271,   -1,   -1,   59,  328,   -1,   10,  375,   -1, 
           -1,   -1,  335,  336,   -1,   -1,   -1,   -1,  341,  342, 
          387,   -1,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,  362, 
           -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263, 
          264,   -1,   -1,  267,  268,  269,   59,  271,   -1,   -1, 
           63,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1, 
           -1,   -1,  342,   -1,   -1,   -1,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,   -1,   91,   -1, 
           -1,  305,   -1,  363,   -1,   -1,   -1,  279,   -1,   -1, 
           -1,   -1,   -1,  317,  318,  319,  320,  321,  322,  323, 
          324,  325,  326,  327,  328,  329,  330,  387,   -1,  333, 
          334,  335,  336,   -1,   -1,  339,   -1,   -1,  342,   -1, 
           -1,  345,   -1,  347,   -1,  349,   -1,  351,    0,  353, 
          354,  355,  356,  357,  358,  359,  328,  361,   10,  363, 
          262,  263,  264,  335,  336,  267,  268,  269,   -1,  271, 
          342,  375,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
          282,   -1,   -1,  387,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  305,    0,   -1,   -1,   59,   -1,   -1, 
           -1,   -1,   -1,   -1,   10,   -1,  318,  319,  320,  321, 
          322,  323,  324,  325,  326,  327,  328,  329,  330,   -1, 
           -1,  333,  334,  335,  336,  280,   -1,  339,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,  349,   44,  351, 
           -1,  353,  354,  355,  356,  357,  358,  359,   -1,  361, 
           -1,  363,   -1,   59,   -1,   -1,   -1,   63,   -1,  262, 
          263,  264,   -1,  375,  267,  268,  269,   -1,  271,   -1, 
           -1,   -1,   -1,  328,   -1,  387,   -1,   -1,  281,  282, 
          335,  336,   -1,   -1,   -1,   91,  341,  290,  291,   -1, 
          293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,    0,  359,   -1,   -1,  362,   -1,   -1, 
           -1,   -1,   -1,   10,  317,  318,  319,  320,  321,  322, 
          323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1, 
          333,  334,  335,  336,   -1,   -1,  339,   -1,   -1,  342, 
           -1,   -1,  345,   -1,  347,   -1,  349,   44,  351,   -1, 
          353,  354,  355,  356,  357,  358,  359,   -1,  361,   -1, 
          363,   -1,   59,   -1,   -1,   -1,   63,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  387,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271, 
            0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281, 
           10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,   -1,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,   -1,   -1,   58,   59, 
           -1,   -1,   -1,   63,   -1,  281,  282,   -1,   -1,   -1, 
          342,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,  387,   -1,  333,  334,  335, 
          336,   -1,    0,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,   10,  349,   -1,  351,   -1,  353,  354,  355, 
          356,  357,  358,  359,   -1,  361,   -1,  363,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1, 
          267,  268,  269,   -1,  271,   -1,   44,   -1,   -1,   -1, 
           -1,  387,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1, 
           58,   59,   -1,  290,  291,   63,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   -1,   -1,  333,  334,  335,  336, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,  349,    0,  351,   -1,  353,  354,  355,  356, 
          357,  358,  359,   10,  361,   -1,  363,   -1,   -1,   -1, 
           -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          387,  281,  282,   -1,   -1,   -1,   -1,   44,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   58,   59,   -1,   -1,  305,   63,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,   -1,  329, 
          330,   -1,   -1,  333,  334,   -1,   -1,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349, 
           -1,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
            0,   -1,   -1,   -1,   -1,  375,   -1,   -1,   -1,   -1, 
           10,   -1,   -1,   -1,  262,  263,  264,  387,   -1,  267, 
          268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   44,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,   58,   59, 
           -1,   -1,   -1,   63,   -1,   -1,   -1,   -1,   -1,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
           -1,  329,  330,   -1,   -1,  333,  334,   -1,   -1,   -1, 
           -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347, 
           -1,  349,    0,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   10,  361,   -1,  363,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  375,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,  387, 
          267,  268,  269,   -1,  271,   -1,   44,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1, 
           58,   59,   -1,  290,  291,   63,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   10,   -1,   -1,   -1,  305,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,   -1,  329,  330,   -1,   -1,  333,  334,   44,   -1, 
           -1,   -1,  339,   -1,   -1,  342,   -1,   -1,  345,   -1, 
          347,   -1,  349,   10,  351,   61,  353,  354,  355,  356, 
          357,  358,  359,   -1,  361,   -1,  363,   -1,   -1,   -1, 
            0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  375,   -1, 
           10,   -1,   -1,   -1,   -1,   91,   -1,   44,   -1,   -1, 
          387,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           -1,  271,   59,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  281,  282,   -1,   44,   -1,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   58,   59, 
           -1,   -1,   -1,   63,   91,  305,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,   -1,  329, 
          330,   -1,   -1,  333,  334,   -1,   -1,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349, 
            0,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           10,  361,   -1,  363,  262,  263,  264,   -1,   -1,  267, 
          268,  269,   -1,  271,   -1,  375,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  281,  282,   -1,   -1,  387,   -1,   -1, 
           -1,   -1,  290,  291,   44,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  305,   58,   59, 
           -1,   -1,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
           -1,  329,  330,   -1,   -1,  333,  334,   -1,   -1,   -1, 
           -1,  339,   -1,  279,  342,   -1,   -1,  345,   -1,  347, 
           -1,  349,   -1,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,   -1,  363,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  375,   -1,   -1, 
            0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  387, 
           10,   -1,  328,  280,   -1,   -1,   -1,   -1,   -1,  335, 
          336,   -1,  262,  263,  264,   -1,  342,  267,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  281,  282,   -1,   44,   -1,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   58,   59, 
           -1,  328,   -1,   63,   -1,  305,   -1,   -1,  335,  336, 
           -1,   -1,   -1,   -1,  341,   -1,   -1,   -1,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,   -1,  329, 
          330,   -1,  359,  333,  334,  362,   -1,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349, 
            0,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           10,  361,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  262,  263,  264,  375,   -1,  267,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,  387,   -1,   -1, 
           -1,  281,  282,   -1,   44,   -1,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   59, 
           -1,   61,   -1,   63,   -1,  305,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,   -1,  329, 
          330,   91,   -1,  333,  334,   -1,   -1,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349, 
           -1,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
            0,   -1,   -1,   -1,   -1,  375,   -1,   -1,   -1,   -1, 
           10,   -1,   -1,   -1,   -1,   -1,   -1,  387,   -1,   -1, 
           -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  281,  282,   -1,   44,   -1,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   59, 
           -1,   61,   -1,   63,   -1,  305,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,   -1,  329, 
          330,   91,   -1,  333,  334,   -1,   -1,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1,  349, 
            0,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           10,  361,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  262,  263,  264,  375,   -1,   -1,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,  387,   -1,   -1, 
          280,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   59, 
           -1,   61,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   91,   -1,  333,  334,  335,  336,   -1,   -1,   -1, 
           -1,   -1,  342,   -1,   -1,   -1,   -1,   -1,   -1,  349, 
           -1,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,  362,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
            0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           10,   -1,   -1,   -1,   -1,   -1,   -1,  387,   -1,   -1, 
           -1,   -1,  262,  263,  264,   -1,   -1,   -1,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   59, 
           -1,   61,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   91,   -1,  333,  334,  335,  336,   -1,  338,   -1, 
           -1,   -1,  342,   -1,   -1,   -1,   -1,   -1,   -1,  349, 
            0,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           10,  361,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  262,  263,  264,   -1,   -1,   -1,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,  387,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   59, 
           -1,   61,   -1,   63,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   91,   -1,  333,  334,  335,  336,   -1,  338,   -1, 
           -1,   -1,  342,   -1,   -1,   -1,   -1,   -1,   -1,  349, 
           -1,  351,    0,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   10,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  387,   -1,   -1, 
           -1,   -1,  262,  263,  264,   -1,   44,   -1,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   59,   -1,   61,   -1,   63,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   91,   -1,   -1,   -1,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,    0,   -1,  333,  334,  335,  336,   -1,  338,   -1, 
           -1,   10,  342,   -1,   -1,   -1,   -1,   -1,   -1,  349, 
           -1,  351,   -1,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  262,  263,  264,   44,   -1,   -1,  268,  269, 
           -1,  271,   -1,   -1,   -1,   -1,   -1,  387,   -1,   -1, 
           59,   -1,   61,   -1,   63,   -1,   -1,   -1,   -1,   -1, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   91,   -1,   -1,   -1,   -1,  317,  318,  319, 
          320,  321,  322,  323,  324,  325,  326,  327,  328,  329, 
          330,   -1,   -1,  333,  334,  335,  336,   -1,  338,   -1, 
           -1,   -1,  342,   -1,   -1,   -1,   -1,   -1,   -1,  349, 
           -1,  351,    0,  353,  354,  355,  356,  357,  358,  359, 
           -1,  361,   10,  363,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,   -1, 
          268,  269,   -1,  271,   -1,   -1,   -1,  387,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   44,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   59,   -1,   61,   -1,   63,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   91,   -1,  333,  334,  335,  336,   -1, 
          338,   -1,   -1,   -1,  342,   -1,   -1,   -1,   -1,   -1, 
           -1,  349,    0,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   10,  361,   -1,  363,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  262,  263,  264,   -1,   -1,   -1,  268, 
          269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,  387, 
           -1,   -1,   -1,   -1,   -1,   -1,   44,   -1,   -1,   -1, 
           -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   59,   -1,   61,   -1,   63,   -1,   -1,   -1,   -1, 
           10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  317,  318, 
          319,  320,  321,  322,  323,  324,  325,  326,  327,  328, 
      };
   }

   private static final short[] yyCheck3() {
      return new short[] {

          329,  330,   -1,   91,  333,  334,  335,  336,   -1,  338, 
           -1,   -1,   -1,  342,   44,   -1,   -1,   -1,   -1,   -1, 
          349,   -1,  351,    0,  353,  354,  355,  356,  357,  358, 
          359,   61,  361,   10,  363,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  387,   -1, 
           -1,   91,   -1,   -1,  262,  263,  264,   44,   -1,   -1, 
          268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   59,   -1,   61,   -1,   63,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   91,   -1,   -1,   -1,   -1,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,    0,   -1,  333,  334,  335,  336,   -1, 
          338,   -1,   -1,   10,  342,   -1,   -1,   -1,   -1,   -1, 
           -1,  349,   -1,  351,   -1,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,   -1,  363,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  262,  263,  264,   44,   -1,   -1, 
          268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  387, 
           -1,   -1,   59,   -1,   61,   -1,   63,   -1,   -1,   -1, 
           -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   91,   -1,   -1,   -1,   -1,  317, 
          318,  319,  320,  321,  322,  323,  324,  325,  326,  327, 
          328,  329,  330,   -1,   -1,  333,  334,  335,  336,  279, 
          338,   -1,   -1,   -1,  342,   -1,   -1,   -1,   -1,   -1, 
           -1,  349,   -1,  351,    0,  353,  354,  355,  356,  357, 
          358,  359,   -1,  361,   10,  363,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1, 
           -1,  268,  269,   -1,  271,   -1,   -1,   -1,  328,  387, 
           -1,   -1,   -1,   -1,   -1,  335,  336,   -1,   44,   -1, 
           -1,   -1,  342,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   59,   -1,   61,   -1,   63,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   -1,   91,  333,  334,  335,  336, 
           -1,  338,   -1,   -1,   -1,  342,   -1,   -1,   -1,   -1, 
           -1,   -1,  349,    0,  351,   -1,  353,  354,  355,  356, 
          357,  358,  359,   10,  361,   -1,  363,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1, 
           -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1, 
          387,   -1,   -1,   -1,   -1,   -1,   -1,   44,   -1,   -1, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   59,   -1,   61,   -1,   63,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   91,   -1,  333,  334,  335,  336, 
           -1,  338,   -1,   -1,   -1,  342,   -1,   -1,   -1,   -1, 
           -1,   -1,  349,   -1,  351,    0,  353,  354,  355,  356, 
          357,  358,  359,   -1,  361,   10,  363,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          387,   -1,   -1,   -1,   -1,   -1,  262,  263,  264,   44, 
           -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   58,   59,   -1,   -1,   -1,   63,   -1, 
           -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  317,  318,  319,  320,  321,  322,  323,  324,  325, 
          326,  327,  328,  329,  330,    0,   -1,  333,  334,  335, 
          336,   -1,   -1,   -1,   -1,   10,  342,   -1,   -1,   -1, 
           -1,   -1,   -1,  349,   -1,  351,   -1,  353,  354,  355, 
          356,  357,  358,  359,   -1,  361,   -1,  363,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,   44, 
           -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1, 
           -1,  387,   -1,   58,   59,   -1,   -1,   -1,   63,   -1, 
           -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296, 
          297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          317,  318,  319,  320,  321,  322,  323,  324,  325,  326, 
          327,  328,  329,  330,   -1,   -1,  333,  334,  335,  336, 
           -1,   -1,   -1,   -1,   -1,  342,   -1,   -1,   -1,   -1, 
           -1,   -1,  349,   -1,  351,    0,  353,  354,  355,  356, 
          357,  358,  359,   -1,  361,   10,  363,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
          387,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   58,   59,   -1,   -1,   -1,   63,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   -1,   -1,  333,  334, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,    0,  351,   -1,   -1,   -1, 
          355,  356,  357,  358,  359,   10,  361,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264, 
          375,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,  387,   -1,   -1,   -1,  281,  282,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   58,   59,   -1,   -1,   -1,   63,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   -1,   -1,  333,  334, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,   -1,  351,   -1,   -1,   -1, 
          355,  356,  357,  358,  359,   -1,  361,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1, 
          375,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,  387,   -1,   -1,   -1,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   58,   59,   -1,   -1,   -1,   63,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   -1,   -1,  333,  334, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,    0,  351,   -1,   -1,   -1, 
           -1,   -1,  357,  358,  359,   10,  361,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264, 
          375,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,  387,   -1,   -1,   -1,  281,  282,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   58,   59,   -1,   -1,   -1,   63,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   -1,   -1,  333,  334, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,   -1,  351,   -1,   -1,   -1, 
           -1,   -1,  357,  358,  359,   -1,  361,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1, 
          375,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,  387,   -1,   -1,   -1,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   58,   59,   -1,   -1,   -1,   63,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,    0,  351,   -1,   -1,   -1, 
           -1,   -1,  357,  358,  359,   10,  361,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264, 
          375,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,  387,   -1,   -1,   -1,  281,  282,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   58,   59,   -1,   -1,   -1,   63,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  357,  358,  359,   -1,  361,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1, 
          375,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,  387,   -1,   -1,   -1,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   58,   59,   -1,   -1,   -1,   63,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,    0,   -1,   -1,   -1,   -1, 
           -1,   -1,  357,  358,  359,   10,  361,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264, 
          375,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,  387,   -1,   -1,   -1,  281,  282,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   58,   59,   -1,   -1,   -1,   63,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  357,  358,   -1,   -1,   -1,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1, 
          375,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,  387,   -1,   -1,   -1,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   58,   59,   -1,   -1,   -1,   63,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,    0,   -1,   -1,   -1,   -1, 
           -1,   -1,  357,  358,   -1,   10,   -1,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264, 
          375,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,  387,   -1,   -1,   -1,  281,  282,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   58,   59,   -1,   -1,   -1,   63,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  357,  358,   -1,   -1,   -1,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1, 
          375,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1, 
           -1,   -1,  387,   -1,   -1,   -1,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   58,   59,   -1,   -1,   -1,   63,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  318,  319,  320,  321,  322,  323,  324, 
          325,  326,  327,   -1,  329,  330,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,    0,   -1,   -1,   -1,   -1, 
           -1,   -1,  357,  358,   -1,   10,   -1,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264, 
          375,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,  387,   -1,   -1,   -1,  281,  282,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   58,   59,   -1,   -1,   -1,   63,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  324, 
          325,    0,   -1,   -1,  329,  330,   -1,   -1,   -1,   -1, 
           -1,   10,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1, 
          375,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58, 
           59,   -1,  387,   -1,   63,   -1,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,    0,   -1,   -1,   -1,   -1,   -1, 
          305,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  324, 
          325,   -1,   -1,   -1,  329,  330,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   44,   -1, 
          345,   -1,  347,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   58,   59,   -1,   -1,   -1,   63,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264, 
          375,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
            0,   -1,  387,   -1,   -1,   -1,  281,  282,   -1,   -1, 
           10,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,  324, 
          325,   -1,   -1,   -1,  329,  330,   -1,   -1,   58,   59, 
           -1,   -1,   -1,   63,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   -1,  271,   -1,   -1,   -1,    0,   -1,  363,   -1, 
           -1,   -1,  281,  282,   -1,   -1,   10,   -1,   -1,   -1, 
          375,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,  387,   -1,   -1,   -1,  305,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           44,   -1,   -1,   -1,   -1,  324,  325,   -1,   -1,   -1, 
          329,  330,   -1,   -1,   58,   59,   -1,   -1,   -1,   63, 
          339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  363,   -1,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,  375,   -1,   -1,    0, 
           -1,   -1,   -1,   -1,   -1,  281,  282,   -1,  387,   10, 
           -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  305, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,  324,  325, 
           -1,   -1,   -1,  329,  330,   -1,   -1,   58,   59,   -1, 
           -1,   -1,   63,  339,   -1,   -1,  342,   -1,   -1,  345, 
           -1,  347,  262,  263,  264,   -1,   -1,  267,  268,  269, 
           -1,  271,   -1,   -1,   -1,    0,   -1,  363,   -1,   -1, 
           -1,  281,  282,   -1,   -1,   10,   -1,   -1,   -1,  375, 
          290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1, 
           -1,  387,   -1,   -1,   -1,  305,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44, 
           -1,   -1,   -1,   -1,  324,  325,   -1,   -1,   -1,  329, 
          330,   -1,   -1,   58,   59,   -1,   -1,   -1,   -1,  339, 
           -1,   -1,  342,   -1,   -1,  345,   -1,  347,  262,  263, 
          264,   -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1, 
           -1,    0,   -1,  363,   -1,   -1,   -1,  281,  282,   -1, 
           -1,   10,   -1,   -1,   -1,  375,  290,  291,   -1,  293, 
          294,  295,  296,  297,   -1,   -1,   -1,  387,   -1,   -1, 
           -1,  305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1, 
           -1,  325,   -1,   -1,   -1,  329,  330,   -1,   -1,   58, 
           59,   -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1, 
           -1,  345,   -1,  347,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  363, 
           -1,  262,  263,  264,   -1,   -1,  267,  268,  269,    0, 
          271,  375,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10, 
          281,  282,   -1,  387,   -1,   -1,   -1,   -1,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  305,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,  262,  263,  264, 
           -1,   -1,  267,  268,  269,    0,  271,   -1,   -1,   -1, 
           -1,   -1,  363,   -1,   -1,   10,  281,  282,   -1,   -1, 
           -1,   -1,   -1,   -1,  375,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,  387,   -1,   -1,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   44, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   58,   59,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,  262,  263,  264,   -1,   -1,  267,  268, 
          269,    0,  271,   -1,   -1,   -1,   -1,   -1,  363,   -1, 
           -1,   10,  281,  282,   -1,   -1,   -1,   -1,   -1,   -1, 
          375,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,  387,   -1,   -1,   -1,  305,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58, 
           59,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0, 
           -1,  262,  263,  264,  363,   -1,  267,  268,  269,   10, 
          271,   -1,   -1,   -1,   -1,   -1,  375,   -1,   -1,   -1, 
          281,  282,   -1,   -1,   -1,   -1,   -1,   -1,  387,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   44,  305,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  339,   -1, 
           -1,  342,   -1,   -1,  345,    0,  347,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   10,  271,   -1,   -1,   -1, 
           -1,   -1,  363,   -1,   -1,   -1,  281,  282,   -1,   -1, 
           -1,   -1,   -1,   -1,  375,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,  387,   -1,   -1,   44, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   58,   59,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,    0,  347,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   10,  271,   -1,   -1,   -1,   -1,   -1,  363,   -1, 
           -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   -1,   -1, 
          375,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,  387,   -1,   -1,   44,  305,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           59,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347,    0, 
           -1,  262,  263,  264,   -1,   -1,  267,  268,  269,   10, 
          271,   -1,   -1,   -1,  363,   -1,   -1,   -1,   -1,   -1, 
          281,  282,   -1,   -1,   -1,   -1,  375,   -1,   -1,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,  387,   -1, 
           -1,   -1,   -1,   44,  305,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  339,   -1, 
           -1,  342,   -1,   -1,  345,    0,  347,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   10,  271,   -1,   -1,   -1, 
           -1,   -1,  363,   -1,   -1,   -1,  281,  282,   -1,   -1, 
           -1,   -1,   -1,   -1,  375,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   -1,   -1,  387,   -1,   -1,   44, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   58,   59,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,    0,  347,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   10,  271,   -1,   -1,   -1,   -1,   -1,  363,   -1, 
           -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   -1,   -1, 
          375,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,  387,   -1,   -1,   44,  305,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0, 
           59,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          339,   -1,   -1,  342,   -1,   -1,  345,   -1,  347,   -1, 
           -1,  262,  263,  264,   -1,   -1,  267,  268,  269,   -1, 
          271,   -1,   -1,   44,  363,   -1,   -1,   -1,   -1,   -1, 
          281,  282,   -1,   -1,   -1,   -1,  375,   -1,   59,  290, 
          291,   -1,  293,  294,  295,  296,   -1,   -1,  387,   -1, 
           -1,   -1,   -1,   -1,  305,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  339,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,  262,  263,  264, 
           -1,   -1,  267,  268,  269,    0,  271,   -1,   -1,   -1, 
           -1,   -1,  363,   -1,   -1,   10,  281,  282,   -1,   -1, 
           -1,   -1,   -1,   -1,  375,  290,  291,   59,  293,  294, 
          295,  296,   -1,   -1,   -1,   -1,  387,   -1,   -1,   -1, 
          305,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   59,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1, 
          345,   -1,  347,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   -1,  271,   -1,    0,   -1,   -1,   -1,  363,   -1, 
           -1,   -1,  281,  282,   10,   -1,   -1,   -1,   -1,   -1, 
          375,  290,  291,   -1,  293,  294,  295,  296,  297,   -1, 
           -1,   -1,  387,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1, 
           -1,  262,  263,  264,   10,   -1,  267,  268,  269,   -1, 
          271,   -1,   -1,   59,   -1,   -1,   -1,   -1,   -1,   -1, 
          281,  282,   -1,  342,   -1,   -1,  345,   -1,  347,  290, 
          291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  363,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,    0,   59,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,  387,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  342,   -1,   -1,  345,   -1,  347,   -1,   -1,   -1, 
          262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271, 
           -1,   -1,  363,   -1,   -1,    0,   -1,   -1,   -1,  281, 
          282,   59,   -1,   -1,   -1,   10,   -1,   -1,  290,  291, 
           -1,  293,  294,  295,  296,  297,  387,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1, 
           -1,    0,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1, 
           -1,   10,   -1,   -1,   -1,  290,  291,   -1,  293,  294, 
          295,  296,  297,   -1,   59,   -1,   -1,   -1,   -1,   -1, 
          342,   -1,   -1,  345,   -1,  347,   -1,   -1,   -1,   -1, 
           -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  363,   10,   -1,    0,   -1,   -1,   -1,   -1,   -1, 
           59,   -1,   -1,   -1,   10,   -1,   -1,  342,   -1,   -1, 
          345,   -1,   -1,   -1,   -1,  387,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,   44,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   44,   10, 
           -1,   59,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,  387,   59,   -1,   -1,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,  297,   63,   -1,   -1,   -1,  342,   -1,   -1,  345, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  262,  263,  264,  363,   -1,  267, 
          268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  281,    0,   -1,  342,   -1,   -1,  345, 
           -1,  387,  290,  291,   10,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  363,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264, 
           -1,   -1,  267,  268,  269,   -1,  271,   -1,   44,   -1, 
           -1,  387,   -1,   -1,   -1,   -1,  281,   -1,   -1,   10, 
           -1,   -1,   -1,   59,  342,  290,  291,   -1,  293,  294, 
          295,  296,  297,  262,  263,  264,   -1,   -1,  267,  268, 
          269,   -1,  271,   -1,   -1,  363,   -1,   -1,   -1,   -1, 
           -1,   -1,  281,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  290,  291,   -1,  293,  294,  295,  296,  297,  387, 
           -1,   -1,   -1,   -1,  262,  263,  264,  342,   -1,  267, 
          268,  269,   -1,  271,   -1,   -1,  262,  263,  264,   -1, 
           -1,  267,  268,  269,   -1,  271,   -1,   -1,  363,   -1, 
           -1,   -1,   -1,   -1,   -1,  293,  294,  295,  296,  297, 
           -1,   -1,   -1,  342,   -1,   -1,   -1,  293,  294,  295, 
          296,  297,  387,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  363,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  339,   -1,   10,  342,   -1,   -1,   -1,  387,   -1, 
           -1,   -1,   -1,  339,   -1,   -1,  342,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  363,  317,  318,  319,  320, 
          321,  322,  323,  324,  325,  326,  327,  363,  329,  330, 
           -1,   -1,  333,  334,   -1,   -1,   -1,   -1,   -1,  387, 
           -1,   -1,   59,   -1,   -1,   -1,   -1,   -1,  349,   -1, 
          351,  387,  353,  354,  355,  356,  357,  358,  359,   -1, 
          361,   -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1, 
           -1,   -1,  268,  269,   91,  271,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295, 
          296,   -1,   -1,   -1,   -1,   -1,  257,  258,  259,   -1, 
          261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270, 
           -1,  272,  273,  274,  275,  276,  277,  278,   -1,   10, 
           -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1, 
           -1,  292,   -1,   -1,   -1,   -1,  342,   -1,  299,   -1, 
           -1,  302,  303,  304,   -1,  306,  307,  308,  309,  310, 
          311,  312,  313,  314,  315,  316,   -1,  363,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340, 
           -1,  387,  343,  344,   -1,  346,   -1,  348,   -1,  350, 
           -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360, 
           91,   -1,   -1,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   -1,  384,  385,  386,   -1,   -1,   -1,  256, 
          257,  258,  259,  260,  261,  262,  263,  264,  265,  266, 
           -1,   -1,  269,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,  280,   -1,   10,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,  298,  299,   -1,  301,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  328,   -1,   -1,   59,   -1,   -1,   -1,  335,  336, 
          337,   -1,   -1,  340,  341,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
          357,   -1,   -1,  360,   -1,  362,   91,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,  384,  385,   -1, 
           -1,   -1,   -1,   -1,   -1,  256,  257,  258,  259,  260, 
          261,  262,  263,  264,  265,  266,   -1,   -1,  269,  270, 
           -1,  272,  273,  274,  275,  276,  277,  278,   -1,  280, 
           -1,   10,  283,  284,  285,  286,  287,  288,  289,   -1, 
           -1,  292,   -1,   -1,   -1,   -1,   -1,  298,  299,   -1, 
          301,  302,  303,  304,   -1,  306,  307,  308,  309,  310, 
          311,   -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  328,   -1,   -1, 
           59,   -1,   -1,   -1,  335,  336,  337,   -1,   -1,  340, 
          341,   -1,  343,  344,   -1,  346,   -1,  348,   -1,   -1, 
           -1,  352,   -1,   -1,   -1,   -1,  357,   -1,   -1,  360, 
           -1,  362,   91,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   -1,  384,  385,   -1,   -1,   -1,   -1,   -1, 
           -1,  256,  257,  258,  259,  260,  261,  262,  263,  264, 
          265,  266,   -1,   -1,  269,  270,   -1,  272,  273,  274, 
          275,  276,  277,  278,   -1,  280,   -1,   10,  283,  284, 
          285,  286,  287,  288,  289,   -1,   -1,  292,   -1,   -1, 
           -1,   -1,   -1,  298,  299,   -1,  301,  302,  303,  304, 
           -1,  306,  307,  308,  309,  310,  311,   -1,  313,  314, 
          315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  328,   -1,   -1,   59,   -1,   -1,   -1, 
          335,  336,  337,   -1,   -1,  340,  341,   -1,  343,  344, 
           -1,  346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1, 
           -1,   -1,  357,   -1,   -1,  360,   -1,  362,   91,   -1, 
          365,  366,  367,  368,  369,  370,   -1,   -1,   -1,  374, 
           -1,  376,  377,   -1,  379,  380,  381,  382,   -1,  384, 
          385,   -1,   -1,   -1,   -1,   -1,   -1,  256,  257,  258, 
          259,  260,  261,  262,  263,  264,  265,  266,   -1,   -1, 
          269,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   10,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,  298, 
          299,   -1,  301,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,   -1,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  328, 
           59,   -1,   -1,   -1,   -1,   -1,  335,  336,  337,   -1, 
           -1,  340,  341,   -1,  343,  344,   -1,  346,   -1,  348, 
           -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,  357,   -1, 
           -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,  384,  385,   -1,   -1,   -1, 
           -1,   -1,   -1,  256,  257,  258,  259,  260,  261,  262, 
          263,  264,  265,  266,   -1,   -1,  269,  270,   -1,  272, 
          273,  274,  275,  276,  277,  278,   -1,   10,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,  298,  299,   -1,  301,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,   -1, 
          313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  328,   59,   -1,   -1,   -1, 
           -1,   -1,  335,  336,  337,   -1,   -1,  340,   -1,   -1, 
          343,  344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352, 
           -1,   -1,   -1,   -1,  357,   -1,   -1,  360,   -1,   -1, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,  384,  385,   -1,   -1,   -1,   -1,  256,  257,  258, 
          259,  260,  261,  262,  263,  264,  265,  266,   -1,  268, 
          269,  270,  271,  272,  273,  274,  275,  276,  277,  278, 
           10,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,  298, 
          299,   -1,  301,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,   -1,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,  348, 
           -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,  384,  385,   -1,   -1,   -1, 
           -1,   -1,   -1,  256,  257,  258,  259,  260,  261,  262, 
          263,  264,  265,  266,   -1,   -1,  269,  270,   -1,  272, 
          273,  274,  275,  276,  277,  278,   10,   -1,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,  298,  299,   -1,  301,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,   -1, 
          313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   59,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1, 
          343,  344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,  362, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,  384,  385,   -1,   -1,   -1,  256,  257,  258,  259, 
          260,  261,  262,  263,  264,  265,  266,   -1,   -1,  269, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   10, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,  298,  299, 
           -1,  301,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,   -1,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1, 
           -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,  384,  385,   -1,   -1,   -1,   -1, 
           -1,   -1,  256,  257,  258,  259,  260,  261,  262,  263, 
          264,  265,  266,   -1,   -1,  269,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   10,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,  298,  299,   -1,  301,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,   -1,  313, 
          314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   59,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
          384,  385,   -1,   -1,   -1,  256,  257,  258,  259,  260, 
          261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270, 
           -1,  272,  273,  274,  275,  276,  277,  278,   10,   -1, 
           -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1, 
           -1,  292,   -1,   -1,   -1,   -1,   -1,  298,  299,   -1, 
          301,  302,  303,  304,   -1,  306,  307,  308,  309,  310, 
          311,   -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340, 
           -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1,   -1, 
           -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360, 
           -1,   -1,  363,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   -1,  384,  385,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  257,  258,  259,   -1,  261,   -1,   -1,   -1, 
          265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,  274, 
          275,  276,  277,  278,   10,   -1,   -1,   -1,  283,  284, 
          285,  286,  287,  288,  289,   -1,   -1,  292,   -1,   -1, 
           -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304, 
      };
   }

   private static final short[] yyCheck4() {
      return new short[] {

           -1,  306,  307,  308,  309,  310,  311,   -1,  313,  314, 
          315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344, 
           -1,  346,   -1,   -1,   -1,   -1,   -1,  352,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1, 
          365,  366,  367,  368,  369,  370,   -1,   -1,   -1,  374, 
           -1,  376,  377,   -1,  379,  380,  381,  382,   -1,  384, 
          385,   -1,   -1,   -1,   -1,  257,  258,  259,   -1,  261, 
           -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270,   -1, 
          272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
           44,  313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1, 
          342,  343,  344,   -1,  346,   -1,   -1,   -1,   -1,   -1, 
          352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1, 
           -1,   -1,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,  384,  385,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  257,  258,  259,   -1,  261,   -1,   -1,   -1,  265, 
          266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304,   44, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,   -1,   -1,   -1,   -1,  352,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,  384,  385, 
           -1,   -1,   -1,  257,  258,  259,   -1,  261,   -1,   -1, 
           -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,   -1,  313, 
           -1,   -1,  316,   59,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
          384,  385,  257,  258,  259,   -1,  261,   -1,   -1,   -1, 
          265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,  274, 
          275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284, 
          285,  286,  287,  288,  289,   -1,   -1,  292,   -1,   -1, 
           -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304, 
           -1,  306,  307,  308,  309,  310,  311,   -1,  313,   -1, 
           -1,  316,   59,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344, 
           -1,  346,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          365,  366,  367,  368,  369,  370,   -1,   -1,   -1,  374, 
           -1,  376,  377,   -1,  379,  380,  381,  382,   -1,  384, 
          385,  257,  258,  259,  260,  261,   -1,   -1,   -1,  265, 
          266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,  298,  299,  300,  301,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,  384,  385, 
          257,  258,  259,  260,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,  298,  299,   -1,  301,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,  384,  385,  257, 
          258,  259,  260,  261,  262,  263,  264,  265,  266,  267, 
          268,  269,  270,  271,  272,  273,  274,  275,  276,  277, 
          278,  279,  280,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,  290,  291,  292,   -1,   -1,   -1,   -1,  297, 
          298,  299,  300,  301,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,   -1,   -1,  314,  315,   -1,  317, 
          318,  319,  320,  321,  322,  323,   -1,   -1,  326,  327, 
           -1,   -1,   -1,  331,  332,  333,  334,   -1,   -1,   -1, 
           -1,   -1,   -1,  341,   -1,   -1,   -1,   -1,   -1,   -1, 
          348,  349,   -1,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,   -1,   -1,  364,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  386,  257, 
          258,  259,  260,  261,  262,  263,  264,  265,  266,  267, 
          268,  269,  270,  271,  272,  273,  274,  275,  276,  277, 
          278,  279,  280,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,  290,  291,  292,   -1,   -1,   -1,   -1,  297, 
          298,  299,  300,  301,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,   -1,   -1,  314,  315,   -1,  317, 
          318,  319,  320,  321,  322,  323,   -1,   -1,  326,  327, 
           -1,   -1,   -1,  331,  332,  333,  334,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          348,  349,   -1,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,   -1,   -1,  364,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  386,  257, 
          258,  259,  260,  261,  262,  263,  264,  265,  266,  267, 
          268,  269,  270,  271,  272,  273,  274,  275,  276,  277, 
          278,  279,  280,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,  290,  291,  292,   -1,   -1,   -1,   -1,  297, 
          298,  299,  300,  301,  302,  303,  304,   -1,  306,  307, 
          308,   -1,  310,   -1,   -1,   -1,  314,  315,   -1,  317, 
          318,  319,  320,  321,  322,  323,   -1,   -1,  326,  327, 
           -1,   -1,   -1,  331,  332,  333,  334,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          348,  349,   -1,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,   -1,   -1,  364,  365,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  386,  257, 
          258,  259,  260,  261,  262,  263,  264,  265,  266,  267, 
          268,  269,  270,  271,  272,  273,  274,  275,  276,  277, 
          278,  279,  280,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,  290,  291,  292,   -1,   -1,   -1,   -1,  297, 
          298,  299,  300,  301,  302,  303,  304,   -1,  306,  307, 
           -1,   -1,  310,   -1,   -1,   -1,  314,  315,   -1,  317, 
          318,  319,  320,  321,  322,  323,   -1,   -1,  326,  327, 
           -1,   -1,   -1,  331,  332,  333,  334,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          348,  349,   -1,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,   -1,   -1,  364,  365,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  386,  257, 
          258,  259,  260,  261,  262,  263,  264,  265,  266,  267, 
          268,  269,  270,  271,  272,  273,  274,  275,  276,  277, 
          278,  279,  280,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,  290,  291,  292,   -1,   -1,   -1,   -1,  297, 
          298,  299,  300,  301,  302,  303,  304,   -1,  306,  307, 
           -1,   -1,  310,   -1,   -1,   -1,  314,  315,   -1,  317, 
          318,  319,  320,  321,  322,  323,   -1,   -1,  326,  327, 
           -1,   -1,   -1,  331,  332,  333,  334,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          348,  349,   -1,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,   -1,   -1,  364,   -1,   -1,   -1, 
           -1,   -1,   -1,  256,  257,  258,  259,  260,  261,   -1, 
           -1,   -1,  265,  266,   -1,   -1,   -1,  270,  386,  272, 
          273,  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,  298,  299,  300,  301,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,   -1, 
          313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1, 
          343,  344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,  384,  385,  256,  257,  258,  259,  260,  261,   -1, 
           -1,   -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272, 
          273,  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,  298,  299,   -1,  301,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,   -1, 
          313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1, 
          343,  344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,  384,  385,  256,  257,  258,  259,  260,  261,   -1, 
           -1,   -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272, 
          273,  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,  298,  299,   -1,  301,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,   -1, 
          313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1, 
          343,  344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,  384,  385,  257,  258,  259,   -1,  261,   -1,   -1, 
           -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,  312,  313, 
          314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,  348,   -1,  350,   -1,  352,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
          384,  385,  386,  257,  258,  259,   -1,  261,   -1,   -1, 
           -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,  312,  313, 
          314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,  348,   -1,  350,   -1,  352,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
          384,  385,  386,  257,  258,  259,   -1,  261,   -1,   -1, 
           -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,  312,  313, 
          314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,  348,   -1,  350,   -1,  352,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
          384,  385,  386,  257,  258,  259,   -1,  261,   -1,   -1, 
           -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,  312,  313, 
          314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
          384,  385,  386,  257,  258,  259,   -1,  261,   -1,   -1, 
           -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,  312,  313, 
          314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,   -1,   -1,  350,   -1,  352,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
          384,  385,  386,  257,  258,  259,   -1,  261,   -1,   -1, 
           -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,  312,  313, 
          314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
          384,  385,  386,  257,  258,  259,   -1,  261,   -1,   -1, 
           -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,  312,  313, 
          314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,   -1,   -1,   -1,   -1,  352,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
          384,  385,  386,  257,  258,  259,  260,  261,   -1,   -1, 
           -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,  298,  299,  300,  301,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,   -1,  313, 
          314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,  348,   -1,   -1,   -1,  352,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
          384,  385,  257,  258,  259,  260,  261,   -1,   -1,   -1, 
          265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,  274, 
          275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284, 
          285,  286,  287,  288,  289,   -1,   -1,  292,   -1,   -1, 
           -1,   -1,   -1,  298,  299,   -1,  301,  302,  303,  304, 
           -1,  306,  307,  308,  309,  310,  311,   -1,  313,  314, 
          315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344, 
           -1,  346,   -1,  348,   -1,   -1,   -1,  352,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1, 
          365,  366,  367,  368,  369,  370,   -1,   -1,   -1,  374, 
           -1,  376,  377,   -1,  379,  380,  381,  382,   -1,  384, 
          385,  257,  258,  259,   -1,  261,   -1,   -1,   -1,  265, 
          266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1,  340,  341,   -1,  343,  344,   -1, 
          346,   -1,   -1,   -1,   -1,   -1,  352,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,  384,  385, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,  314,  315,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,  384,  385,  257, 
          258,  259,   -1,  261,   -1,   -1,   -1,  265,  266,   -1, 
           -1,   -1,  270,   -1,  272,  273,  274,  275,  276,  277, 
          278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1, 
           -1,  299,   -1,   -1,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,   -1,  313,  314,  315,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337, 
           -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1, 
          348,   -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,  384,  385,  257,  258, 
          259,   -1,  261,   -1,   -1,   -1,  265,  266,   -1,   -1, 
           -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,   -1,  313,  314,  315,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,  348, 
           -1,   -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  360,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,  384,  385,  257,  258,  259, 
           -1,  261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,   -1,  313,  314,  315,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1, 
           -1,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          360,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,  384,  385,  257,  258,  259,   -1, 
          261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270, 
           -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1, 
           -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1, 
           -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1, 
           -1,  302,  303,  304,   -1,  306,  307,  308,  309,  310, 
          311,   -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340, 
           -1,   -1,  343,  344,   -1,  346,   -1,  348,   -1,   -1, 
           -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360, 
           -1,   -1,   -1,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   -1,  384,  385,  257,  258,  259,   -1,  261, 
           -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270,   -1, 
          272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
           -1,  313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340,  341, 
           -1,  343,  344,   -1,  346,   -1,   -1,   -1,   -1,   -1, 
          352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1, 
           -1,   -1,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,  384,  385,  257,  258,  259,   -1,  261,   -1, 
           -1,   -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272, 
          273,  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1, 
          283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292, 
           -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302, 
          303,  304,   -1,  306,  307,  308,  309,  310,  311,   -1, 
          313,  314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1, 
          343,  344,   -1,  346,   -1,   -1,   -1,   -1,   -1,  352, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1, 
           -1,   -1,  365,  366,  367,  368,  369,  370,   -1,   -1, 
           -1,  374,   -1,  376,  377,   -1,  379,  380,  381,  382, 
           -1,  384,  385,  257,  258,  259,   -1,  261,   -1,   -1, 
           -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273, 
          274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283, 
          284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1, 
           -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303, 
          304,   -1,  306,  307,  308,  309,  310,  311,   -1,  313, 
          314,  315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343, 
          344,   -1,  346,   -1,   -1,   -1,   -1,   -1,  352,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1, 
           -1,  365,  366,  367,  368,  369,  370,   -1,   -1,   -1, 
          374,   -1,  376,  377,   -1,  379,  380,  381,  382,   -1, 
          384,  385,  257,  258,  259,   -1,  261,   -1,   -1,   -1, 
          265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,  274, 
          275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284, 
          285,  286,  287,  288,  289,   -1,   -1,  292,   -1,   -1, 
           -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304, 
           -1,  306,  307,  308,  309,  310,  311,   -1,  313,  314, 
          315,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344, 
           -1,  346,   -1,   -1,   -1,   -1,   -1,  352,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1, 
          365,  366,  367,  368,  369,  370,   -1,   -1,   -1,  374, 
           -1,  376,  377,   -1,  379,  380,  381,  382,   -1,  384, 
          385,  257,  258,  259,   -1,  261,   -1,   -1,   -1,  265, 
          266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275, 
          276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285, 
          286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1, 
           -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1, 
          306,  307,  308,  309,  310,  311,   -1,  313,  314,  315, 
          316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,  337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1, 
          346,   -1,   -1,   -1,   -1,   -1,  352,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,  365, 
          366,  367,  368,  369,  370,   -1,   -1,   -1,  374,   -1, 
          376,  377,   -1,  379,  380,  381,  382,   -1,  384,  385, 
          257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266, 
           -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276, 
          277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286, 
          287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1, 
           -1,   -1,  299,   -1,   -1,  302,  303,  304,   -1,  306, 
          307,  308,  309,  310,  311,   -1,  313,   -1,   -1,  316, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          337,   -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346, 
           -1,  348,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  365,  366, 
          367,  368,  369,  370,   -1,   -1,   -1,  374,   -1,  376, 
          377,   -1,  379,  380,  381,  382,   -1,  384,  385,  257, 
          258,  259,   -1,  261,   -1,   -1,   -1,  265,  266,   -1, 
           -1,   -1,  270,   -1,  272,  273,  274,  275,  276,  277, 
          278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287, 
          288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1, 
           -1,  299,   -1,   -1,  302,  303,  304,   -1,  306,  307, 
          308,  309,  310,  311,   -1,  313,   -1,   -1,  316,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337, 
           -1,   -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1, 
          348,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  365,  366,  367, 
          368,  369,  370,   -1,   -1,   -1,  374,   -1,  376,  377, 
           -1,  379,  380,  381,  382,   -1,  384,  385,  257,  258, 
          259,   -1,  261,   -1,   -1,   -1,  265,  266,   -1,   -1, 
           -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278, 
           -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288, 
          289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1, 
          299,   -1,   -1,  302,  303,  304,   -1,  306,  307,  308, 
          309,  310,  311,   -1,  313,   -1,   -1,  316,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1, 
           -1,  340,   -1,   -1,  343,  344,   -1,  346,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  365,  366,  367,  368, 
          369,  370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1, 
          379,  380,  381,  382,   -1,  384,  385,  257,  258,  259, 
           -1,  261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1, 
          270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1, 
           -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289, 
           -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299, 
           -1,   -1,  302,  303,  304,   -1,  306,  307,  308,  309, 
          310,  311,   -1,  313,   -1,   -1,  316,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1, 
          340,   -1,   -1,  343,  344,   -1,  346,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  365,  366,  367,  368,  369, 
          370,   -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379, 
          380,  381,  382,   -1,  384,  385,  257,  258,  259,   -1, 
          261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270, 
           -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1, 
           -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1, 
           -1,  292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1, 
           -1,  302,  303,  304,   -1,  306,  307,  308,  309,  310, 
          311,   -1,  313,   -1,   -1,  316,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340, 
           -1,   -1,  343,  344,   -1,  346,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,  365,  366,  367,  368,  369,  370, 
           -1,   -1,   -1,  374,   -1,  376,  377,   -1,  379,  380, 
          381,  382,   -1,  384,  385,  257,  258,  259,   -1,  261, 
           -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270,   -1, 
          272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1, 
           -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1, 
          292,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1, 
          302,  303,  304,   -1,  306,  307,  308,  309,  310,  311, 
           -1,  313,   -1,   -1,  316,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340,   -1, 
           -1,  343,  344,   -1,  346,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,  365,  366,  367,  368,  369,  370,   -1, 
           -1,   -1,  374,   -1,  376,  377,   -1,  379,  380,  381, 
          382,   -1,  384,  385,  306,  307,   -1,   -1,  310,   -1, 
           -1,   -1,  314,  315,   -1,  317,  318,  319,  320,  321, 
          322,  323,   -1,   -1,  326,  327,   -1,   -1,   -1,  331, 
          332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1,  341, 
           -1,   -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
          306,  307,  364,   -1,  310,   -1,   -1,   -1,  314,  315, 
           -1,  317,  318,  319,  320,  321,  322,  323,   -1,   -1, 
          326,  327,   -1,   -1,  386,  331,  332,  333,  334,   -1, 
           -1,   -1,   -1,   -1,   -1,  341,   -1,   -1,   -1,   -1, 
           -1,   -1,  348,  349,   -1,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,  306,  307,  364,   -1, 
          310,   -1,   -1,   -1,  314,  315,   -1,  317,  318,  319, 
          320,  321,  322,  323,   -1,   -1,  326,  327,   -1,   -1, 
          386,  331,  332,  333,  334,   -1,   -1,   -1,   -1,   -1, 
           -1,  341,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349, 
           -1,  351,  352,  353,  354,  355,  356,  357,  358,  359, 
          360,  361,  306,  307,  364,   -1,  310,   -1,   -1,   -1, 
          314,  315,   -1,  317,  318,  319,  320,  321,  322,  323, 
           -1,   -1,  326,  327,   -1,   -1,  386,  331,  332,  333, 
          334,   -1,   -1,   -1,   -1,   -1,   -1,  341,   -1,   -1, 
           -1,   -1,   -1,   -1,  348,  349,   -1,  351,  352,  353, 
          354,  355,  356,  357,  358,  359,  360,  361,  306,  307, 
          364,   -1,  310,   -1,   -1,   -1,  314,  315,   -1,  317, 
          318,  319,  320,  321,  322,  323,   -1,   -1,  326,  327, 
           -1,   -1,  386,  331,  332,  333,  334,   -1,   -1,   -1, 
           -1,   -1,   -1,  341,   -1,   -1,   -1,   -1,   -1,   -1, 
          348,  349,   -1,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,  306,  307,  364,   -1,  310,   -1, 
           -1,   -1,  314,  315,   -1,  317,  318,  319,  320,  321, 
          322,  323,   -1,   -1,  326,  327,   -1,   -1,  386,  331, 
          332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1,  341, 
           -1,   -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
          306,  307,  364,   -1,  310,   -1,   -1,   -1,  314,  315, 
           -1,  317,  318,  319,  320,  321,  322,  323,   -1,   -1, 
          326,  327,   -1,   -1,  386,  331,  332,  333,  334,   -1, 
           -1,   -1,   -1,   -1,   -1,  341,   -1,   -1,   -1,   -1, 
           -1,   -1,  348,  349,   -1,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,  306,  307,  364,   -1, 
          310,   -1,   -1,   -1,  314,  315,   -1,  317,  318,  319, 
          320,  321,  322,  323,   -1,   -1,  326,  327,   -1,   -1, 
          386,  331,  332,  333,  334,   -1,   -1,   -1,   -1,   -1, 
           -1,  341,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349, 
           -1,  351,  352,  353,  354,  355,  356,  357,  358,  359, 
          360,  361,  306,  307,  364,   -1,  310,   -1,   -1,   -1, 
          314,  315,   -1,  317,  318,  319,  320,  321,  322,  323, 
           -1,   -1,  326,  327,   -1,   -1,  386,  331,  332,  333, 
          334,   -1,   -1,   -1,   -1,   -1,   -1,  341,   -1,   -1, 
           -1,   -1,   -1,   -1,  348,  349,   -1,  351,  352,  353, 
          354,  355,  356,  357,  358,  359,  360,  361,  306,  307, 
          364,   -1,  310,   -1,   -1,   -1,  314,  315,   -1,  317, 
          318,  319,  320,  321,  322,  323,   -1,   -1,  326,  327, 
           -1,   -1,  386,  331,  332,  333,  334,   -1,   -1,   -1, 
           -1,   -1,   -1,  341,   -1,   -1,   -1,   -1,   -1,   -1, 
          348,  349,   -1,  351,  352,  353,  354,  355,  356,  357, 
          358,  359,  360,  361,  306,  307,  364,   -1,  310,   -1, 
           -1,   -1,  314,  315,   -1,  317,  318,  319,  320,  321, 
          322,  323,   -1,   -1,  326,  327,   -1,   -1,  386,  331, 
          332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1,  341, 
           -1,   -1,   -1,   -1,   -1,   -1,  348,  349,   -1,  351, 
          352,  353,  354,  355,  356,  357,  358,  359,  360,  361, 
          306,  307,  364,   -1,  310,   -1,   -1,   -1,  314,  315, 
           -1,  317,  318,  319,  320,  321,  322,  323,   -1,   -1, 
          326,  327,   -1,   -1,  386,  331,  332,  333,  334,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,  348,  349,   -1,  351,  352,  353,  354,  355, 
          356,  357,  358,  359,  360,  361,   -1,   -1,  364,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
           -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, 
          386, 
      };
   }

}
// @formatter:on
